package com.chelvc.framework.base.cache;

import java.util.Collection;
import java.util.List;
import java.util.function.BiFunction;
import java.util.function.Supplier;

import org.apache.commons.lang3.tuple.Pair;

/**
 * 信息缓存接口
 *
 * @author Woody
 * @date 2024/1/30
 */
public interface Caching {
    /**
     * 获取缓存内容
     *
     * @param naming 缓存命名
     * @param id     对象标识
     * @param <K>    缓存标识类型
     * @param <V>    缓存内容类型
     * @return 缓存对象实例
     */
    <K, V> V get(Naming naming, K id);

    /**
     * 获取缓存内容
     *
     * @param naming   缓存命名
     * @param id       对象标识
     * @param supplier 缓存对象缺失回调函数
     * @param <K>      缓存标识类型
     * @param <V>      缓存内容类型
     * @return 缓存对象实例
     */
    <K, V> V get(Naming naming, K id, Supplier<V> supplier);

    /**
     * 批量获取缓存内容
     *
     * @param naming 缓存命名
     * @param min    最小排序数字
     * @param max    最大排序数字
     * @param size   获取数量
     * @param <V>    缓存内容类型
     * @return 缓存对象实例列表
     */
    <V> List<V> list(Naming naming, long min, long max, int size);

    /**
     * 批量获取缓存内容
     *
     * @param naming    缓存命名
     * @param min       最小排序数字
     * @param max       最大排序数字
     * @param size      获取数量
     * @param supplier  缓存对象缺失回调函数
     * @param converter 缓存信息转换器
     * @param <K>       缓存标识类型
     * @param <V>       缓存内容类型
     * @return 缓存对象实例列表
     */
    <K, V> List<V> list(Naming naming, long min, long max, int size, BiFunction<Naming, Boolean, List<V>> supplier,
                        Converter<K, V> converter);

    /**
     * 批量随机获取缓存内容
     *
     * @param naming 缓存命名
     * @param min    最小排序数字
     * @param max    最大排序数字
     * @param size   获取数量（当size < 0 时获取全部内容）
     * @param <V>    缓存内容类型
     * @return 缓存对象实例列表
     */
    <V> List<V> random(Naming naming, long min, long max, int size);

    /**
     * 批量随机获取缓存内容
     *
     * @param naming    缓存命名
     * @param min       最小排序数字
     * @param max       最大排序数字
     * @param size      获取数量（当size < 0 时获取全部内容）
     * @param supplier  缓存对象缺失回调函数
     * @param converter 缓存信息转换器
     * @param <K>       缓存标识类型
     * @param <V>       缓存内容类型
     * @return 缓存对象实例列表
     */
    <K, V> List<V> random(Naming naming, long min, long max, int size, BiFunction<Naming, Boolean, List<V>> supplier,
                          Converter<K, V> converter);

    /**
     * 设置缓存内容
     *
     * @param naming 缓存命名
     * @param id     对象标识
     * @param value  缓存内容
     * @param <K>    缓存标识类型
     * @param <V>    缓存内容类型
     * @return true/false
     */
    <K, V> boolean set(Naming naming, K id, V value);

    /**
     * 设置缓存内容
     *
     * @param naming 缓存命名
     * @param id     对象标识
     * @param value  缓存内容
     * @param order  排序数字
     * @param <K>    缓存标识类型
     * @param <V>    缓存内容类型
     * @return true/false
     */
    <K, V> boolean set(Naming naming, K id, V value, long order);

    /**
     * 批量设置缓存内容
     *
     * @param naming    缓存命名
     * @param values    缓存内容集合
     * @param converter 缓存信息转换器
     * @param <K>       缓存标识类型
     * @param <V>       缓存内容类型
     * @return true/false
     */
    <K, V> boolean set(Naming naming, Collection<V> values, Converter<K, V> converter);

    /**
     * 更新缓存内容
     *
     * @param naming 缓存命名
     * @param id     对象标识
     * @param value  缓存内容
     * @param order  排序数字
     * @param <K>    缓存标识类型
     * @param <V>    缓存内容类型
     * @return true/false
     */
    <K, V> boolean update(Naming naming, K id, V value, long order);

    /**
     * 更新缓存内容
     *
     * @param naming    缓存命名
     * @param value     缓存内容
     * @param converter 缓存信息转换器
     * @param <K>       缓存标识类型
     * @param <V>       缓存内容类型
     * @return true/false
     */
    <K, V> boolean update(Naming naming, V value, Converter<K, V> converter);

    /**
     * 批量更新缓存内容
     *
     * @param pairs     缓存命名/对象集合
     * @param converter 缓存信息转换器
     * @param <K>       缓存标识类型
     * @param <V>       缓存内容类型
     * @return true/false
     */
    <K, V> boolean update(Collection<Pair<Naming, V>> pairs, Converter<K, V> converter);

    /**
     * 移除缓存数据
     *
     * @param naming 缓存命名
     * @return true/false
     */
    boolean remove(Naming naming);

    /**
     * 移除缓存数据
     *
     * @param naming 缓存命名
     * @param id     对象标识
     * @param <K>    缓存标识类型
     * @return true/false
     */
    <K> boolean remove(Naming naming, K id);

    /**
     * 移除缓存数据
     *
     * @param pairs 缓存命名/标识集合
     * @param <K>   缓存标识类型
     * @return true/false
     */
    <K> boolean remove(Collection<Pair<Naming, K>> pairs);

    /**
     * 清理缓存数据，移除所有相关缓存数据
     *
     * @param naming 缓存命名
     * @param id     对象标识
     * @param <K>    缓存标识类型
     * @return true/false
     */
    <K> boolean clear(Naming naming, K id);

    /**
     * 清理缓存数据，移除所有相关缓存数据
     *
     * @param pairs 移除命名/标识集合
     * @param <K>   缓存标识类型
     * @return true/false
     */
    <K> boolean clear(Collection<Pair<Naming, K>> pairs);
}
