package com.chelvc.framework.base.cache;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.Getter;
import lombok.NonNull;

/**
 * 缓存命名对象
 *
 * @author Woody
 * @date 2024/1/30
 */
@Getter
public class Naming implements Serializable {
    /**
     * 主体缓存名称
     */
    private final String primary;

    /**
     * 索引缓存名称集合
     */
    private final List<String> indexes;

    /**
     * 获取缓存标识列表
     */
    private final List<String> keys;

    /**
     * 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     */
    private final long duration;

    /**
     * 缓存内容是否降序排序
     */
    private final boolean reversed;

    private Naming(@NonNull String primary, @NonNull List<String> indexes, long duration, boolean reversed) {
        AssertUtils.nonempty(primary, () -> "Naming primary must not be empty");
        AssertUtils.nonempty(indexes, () -> "Naming index must not be empty");
        this.primary = primary;
        this.indexes = Collections.unmodifiableList(indexes);
        this.duration = duration;
        this.reversed = reversed;

        // 组装缓存标识列表
        int i = 0;
        String[] keys = new String[indexes.size() + 1];
        keys[i++] = primary;
        for (String index : indexes) {
            keys[i++] = index;
        }
        this.keys = Arrays.asList(keys);
    }

    @Override
    public String toString() {
        return StringUtils.join(this.keys, "|");
    }

    /**
     * 构建缓存命名对象
     *
     * @param name 缓存名称
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String name) {
        return of(name, name, -1, false);
    }

    /**
     * 构建缓存命名对象
     *
     * @param name     缓存名称
     * @param duration 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String name, long duration) {
        return of(name, name, duration, false);
    }

    /**
     * 构建缓存命名对象
     *
     * @param name     缓存名称
     * @param reversed 缓存内容是否降序排序
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String name, boolean reversed) {
        return of(name, name, -1, reversed);
    }

    /**
     * 构建缓存命名对象
     *
     * @param name     缓存名称
     * @param duration 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     * @param reversed 缓存内容是否降序排序
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String name, long duration, boolean reversed) {
        return of(name, name, duration, reversed);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary 主体缓存名称
     * @param index   索引缓存名称
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull String index) {
        return of(primary, index, -1, false);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary  主体缓存名称
     * @param index    索引缓存名称
     * @param duration 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull String index, long duration) {
        return of(primary, index, duration, false);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary  主体缓存名称
     * @param index    索引缓存名称
     * @param reversed 缓存内容是否降序排序
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull String index, boolean reversed) {
        return of(primary, index, -1, reversed);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary  主体缓存名称
     * @param index    索引缓存名称
     * @param duration 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     * @param reversed 缓存内容是否降序排序
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull String index, long duration, boolean reversed) {
        AssertUtils.nonempty(index, () -> "Naming index must not be empty");
        return of(primary, Collections.singletonList(index), duration, reversed);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary 主体缓存名称
     * @param indexes 索引缓存名称列表
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull List<String> indexes) {
        return new Naming(primary, indexes, -1, false);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary  主体缓存名称
     * @param indexes  索引缓存名称列表
     * @param duration 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull List<String> indexes, long duration) {
        return new Naming(primary, indexes, duration, false);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary  主体缓存名称
     * @param indexes  索引缓存名称列表
     * @param reversed 缓存内容是否降序排序
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull List<String> indexes, boolean reversed) {
        return new Naming(primary, indexes, -1, reversed);
    }

    /**
     * 构建缓存命名对象
     *
     * @param primary  主体缓存名称
     * @param indexes  索引缓存名称列表
     * @param duration 缓存有效时长（秒），当 duration > 0 时有效，否则表示永久有效
     * @param reversed 缓存内容是否降序排序
     * @return 缓存命名对象实例
     */
    public static Naming of(@NonNull String primary, @NonNull List<String> indexes, long duration, boolean reversed) {
        return new Naming(primary, indexes, duration, reversed);
    }
}
