package com.chelvc.framework.base.cache;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.function.BiFunction;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import org.apache.commons.lang3.tuple.Pair;

/**
 * 对象缓存抽象实现
 *
 * @param <K> 对象标识类型
 * @param <Q> 查询对象类型
 * @param <V> 缓存对象类型
 * @author Woody
 * @date 2024/1/30
 */
public abstract class ObjectCaching<K, Q, V> {
    private Caching caching;
    private Converter<K, V> converter;
    private NamingAdapter<Q, V> namingAdapter;

    /**
     * 缓存开关是否开启
     *
     * @return true/false
     */
    protected boolean isEnabled() {
        return true;
    }

    /**
     * 初始化数据缓存
     *
     * @return 缓存对象实例
     */
    protected abstract Caching initializeCaching();

    /**
     * 初始化缓存信息转换器
     *
     * @return 缓存信息转换器实例
     */
    protected abstract Converter<K, V> initializeConverter();

    /**
     * 初始化缓存命名适配器
     *
     * @return 缓存命名适配器对象实例
     */
    protected abstract NamingAdapter<Q, V> initializeNamingAdapter();

    /**
     * 获取数据缓存器
     *
     * @return 缓存对象实例
     */
    protected final Caching getCaching() {
        if (this.caching == null) {
            synchronized (Caching.class) {
                if (this.caching == null) {
                    this.caching = Objects.requireNonNull(this.initializeCaching());
                }
            }
        }
        return this.caching;
    }

    /**
     * 获取缓存信息转换器
     *
     * @return 缓存信息转换器实例
     */
    protected final Converter<K, V> getConverter() {
        if (this.converter == null) {
            synchronized (Converter.class) {
                if (this.converter == null) {
                    this.converter = Objects.requireNonNull(this.initializeConverter());
                }
            }
        }
        return this.converter;
    }

    /**
     * 获取缓存命名适配器
     *
     * @return 缓存命名适配器实例
     */
    protected final NamingAdapter<Q, V> getNamingAdapter() {
        if (this.namingAdapter == null) {
            synchronized (NamingAdapter.class) {
                if (this.namingAdapter == null) {
                    this.namingAdapter = Objects.requireNonNull(this.initializeNamingAdapter());
                }
            }
        }
        return this.namingAdapter;
    }

    /**
     * 获取缓存对象
     *
     * @param id 对象标识
     * @return 对象实例
     */
    public V get(@NonNull K id) {
        if (!this.isEnabled()) {
            return null;
        }
        Naming naming = this.getNamingAdapter().primary();
        return this.getCaching().get(naming, id);
    }

    /**
     * 获取缓存对象
     *
     * @param id       对象标识
     * @param supplier 缓存对象缺失回调函数
     * @return 对象实例
     */
    public V get(@NonNull K id, @NonNull Supplier<V> supplier) {
        if (this.isEnabled()) {
            Naming naming = this.getNamingAdapter().primary();
            return this.getCaching().get(naming, id, supplier);
        }
        return supplier.get();
    }

    /**
     * 批量获取缓存对象
     *
     * @param query 查询对象
     * @param min   最小排序数字
     * @param max   最大排序数字
     * @param size  获取数量
     * @return 缓存对象实例列表
     */
    public List<V> list(@NonNull Q query, long min, long max, int size) {
        if (this.isEnabled()) {
            Naming naming = this.getNamingAdapter().querying(query);
            return this.getCaching().list(naming, min, max, size);
        }
        return Collections.emptyList();
    }

    /**
     * 批量获取缓存对象
     *
     * @param query    查询对象
     * @param min      最小排序数字
     * @param max      最大排序数字
     * @param size     获取数量
     * @param supplier 缓存对象缺失回调函数
     * @return 缓存对象实例列表
     */
    public List<V> list(@NonNull Q query, long min, long max, int size,
                        @NonNull BiFunction<Naming, Boolean, List<V>> supplier) {
        Naming naming = this.getNamingAdapter().querying(query);
        if (this.isEnabled()) {
            return this.getCaching().list(naming, min, max, size, supplier, this.getConverter());
        }
        List<V> objects = supplier.apply(naming, true);
        if (ObjectUtils.isEmpty(objects)) {
            return Collections.emptyList();
        }
        return objects.size() <= size ? objects : objects.subList(0, size);
    }

    /**
     * 批量随机获取缓存对象
     *
     * @param query 查询对象
     * @param min   最小排序数字
     * @param max   最大排序数字
     * @param size  获取数量（当size < 0 时获取全部内容）
     * @return 缓存对象实例列表
     */
    public List<V> random(@NonNull Q query, long min, long max, int size) {
        if (this.isEnabled()) {
            Naming naming = this.getNamingAdapter().querying(query);
            return this.getCaching().random(naming, min, max, size);
        }
        return Collections.emptyList();
    }

    /**
     * 批量随机获取缓存对象
     *
     * @param query    查询对象
     * @param min      最小排序数字
     * @param max      最大排序数字
     * @param size     获取数量（当size < 0 时获取全部内容）
     * @param supplier 补充缓存列表回调函数
     * @return 缓存对象实例列表
     */
    public List<V> random(@NonNull Q query, long min, long max, int size,
                          @NonNull BiFunction<Naming, Boolean, List<V>> supplier) {
        Naming naming = this.getNamingAdapter().querying(query);
        if (this.isEnabled()) {
            return this.getCaching().random(naming, min, max, size, supplier, this.getConverter());
        }
        List<V> objects = supplier.apply(naming, true);
        if (ObjectUtils.isEmpty(objects)) {
            return Collections.emptyList();
        }
        return size < 0 || objects.size() <= size ? objects : objects.subList(0, size);
    }

    /**
     * 清空缓存数据
     *
     * @return true/false
     */
    public boolean clear() {
        Naming naming = this.getNamingAdapter().primary();
        return this.getCaching().remove(naming);
    }

    /**
     * 设置缓存对象
     *
     * @param object 对象实例
     * @return true/false
     */
    public boolean set(@NonNull V object) {
        if (!this.isEnabled()) {
            return false;
        }
        K id = this.getConverter().id(object);
        Naming naming = this.getNamingAdapter().primary();
        return this.getCaching().set(naming, id, object);
    }

    /**
     * 批量设置缓存对象
     *
     * @param objects 对象实例集合
     * @return true/false
     */
    public boolean set(@NonNull Collection<V> objects) {
        if (!this.isEnabled() || ObjectUtils.isEmpty(objects)) {
            return false;
        }
        Naming naming = this.getNamingAdapter().primary();
        return this.getCaching().set(naming, objects, this.getConverter());
    }

    /**
     * 更新缓存对象
     *
     * @param object 对象实例
     * @return true/false
     */
    public boolean update(@NonNull V object) {
        if (!this.isEnabled()) {
            return false;
        }
        K id = this.getConverter().id(object);
        long order = this.getConverter().order(object);
        Naming naming = this.getNamingAdapter().modifying(object);
        return this.getCaching().update(naming, id, object, order);
    }

    /**
     * 批量更新缓存对象
     *
     * @param objects 对象实例集合
     * @return true/false
     */
    public boolean update(@NonNull Collection<V> objects) {
        if (!this.isEnabled() || ObjectUtils.isEmpty(objects)) {
            return false;
        }
        NamingAdapter<Q, V> adapter = this.getNamingAdapter();
        List<Pair<Naming, V>> pairs = objects.stream().map(object -> Pair.of(adapter.modifying(object), object))
                .collect(Collectors.toList());
        return this.getCaching().update(pairs, this.getConverter());
    }

    /**
     * 移除缓存对象
     *
     * @param object 对象实例
     * @return true/false
     */
    public boolean remove(@NonNull V object) {
        if (!this.isEnabled()) {
            return false;
        }
        K id = this.getConverter().id(object);
        Naming naming = this.getNamingAdapter().modifying(object);
        return this.getCaching().clear(naming, id);
    }

    /**
     * 批量移除缓存对象
     *
     * @param objects 对象实例集合
     * @return true/false
     */
    public boolean remove(@NonNull Collection<V> objects) {
        if (!this.isEnabled() || ObjectUtils.isEmpty(objects)) {
            return false;
        }
        Converter<K, V> converter = this.getConverter();
        NamingAdapter<Q, V> adapter = this.getNamingAdapter();
        List<Pair<Naming, K>> pairs = objects.stream()
                .map(object -> Pair.of(adapter.modifying(object), converter.id(object))).collect(Collectors.toList());
        return this.getCaching().clear(pairs);
    }
}
