package com.chelvc.framework.base.cache;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.TreeMap;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Lists;
import lombok.NonNull;
import org.apache.commons.lang3.tuple.Pair;

/**
 * 基于对象的缓存命名适配器实现
 *
 * @param <Q> 查询对象类型
 * @param <V> 缓存对象类型
 * @author Woody
 * @date 2024/1/30
 */
public class ObjectNamingAdapter<Q, V> implements NamingAdapter<Q, V> {
    /**
     * 主体缓存命名
     */
    private final Naming primary;

    /**
     * 对象属性/Getter方法映射表
     */
    private final Map<String, Pair<Function<Q, ?>, Function<V, ?>>> getters;

    public ObjectNamingAdapter(@NonNull Naming primary,
                               @NonNull Map<String, Pair<Function<Q, ?>, Function<V, ?>>> getters) {
        AssertUtils.nonempty(getters, () -> "Naming adapter getters must not be empty");
        getters.forEach((field, functions) -> {
            AssertUtils.nonempty(field, () -> "Naming adapter getter field must not be empty");
            AssertUtils.nonnull(functions, () -> "Naming adapter getters must not be null");
            AssertUtils.nonnull(functions.getLeft(), () -> "Naming adapter getters left must not be null");
            AssertUtils.nonnull(functions.getRight(), () -> "Naming adapter getters right must not be null");
        });
        this.primary = primary;
        this.getters = new TreeMap<>(getters);
    }

    @Override
    public Naming primary() {
        return this.primary;
    }

    @Override
    public List<String> query2string(@NonNull Q object) {
        StringBuilder buffer = new StringBuilder(this.primary.getPrimary());
        this.getters.forEach((field, functions) -> {
            Object value = functions.getLeft().apply(object);
            buffer.append(":").append(field).append(StringUtils.EQUAL);
            if (ObjectUtils.notBlank(value)) {
                int length = buffer.length();
                ObjectUtils.stream(value).filter(StringUtils::notEmpty).map(Object::toString)
                        .sorted().forEach(item -> buffer.append("#").append(item));
                if (buffer.length() > length) {
                    buffer.append("#");
                }
            }
        });
        return Collections.singletonList(buffer.toString());
    }

    @Override
    public List<String> modify2string(@NonNull V object) {
        // 对象属性值排列
        List<List<StringBuilder>> conditions = Lists.newArrayListWithCapacity(this.getters.size());
        this.getters.forEach((field, functions) -> {
            Stream<?> values = ObjectUtils.stream(functions.getRight().apply(object));
            List<StringBuilder> buffers = values.distinct().map(value -> {
                StringBuilder buffer = new StringBuilder(field).append(StringUtils.EQUAL);
                if (StringUtils.notEmpty(value)) {
                    buffer.append("*#").append(value).append("#*");
                }
                return buffer;
            }).collect(Collectors.toList());
            if (ObjectUtils.notEmpty(buffers)) {
                conditions.add(buffers);
            }
        });
        if (ObjectUtils.isEmpty(conditions)) {
            return Collections.emptyList();
        }

        // 组合匹配条件
        List<StringBuilder> roots = Collections.emptyList();
        for (List<StringBuilder> buffers : conditions) {
            if (ObjectUtils.isEmpty(roots)) {
                buffers.forEach(buffer -> buffer.insert(0, ":").insert(0, this.primary.getPrimary()));
                roots = buffers;
            } else if (buffers.size() > 1) {
                roots = roots.stream().flatMap(root -> buffers.stream().map(
                        buffer -> new StringBuilder(root).append(":").append(buffer)
                )).collect(Collectors.toList());
            } else {
                roots.forEach(root -> root.append(":").append(buffers.get(0)));
            }
        }
        return roots.stream().map(StringBuilder::toString).collect(Collectors.toList());
    }

    @Override
    public boolean equals(V object, V other) {
        if (object == other || ObjectUtils.isEmpty(this.getters)) {
            return Objects.equals(object, other);
        } else if (object != null && other != null) {
            for (Pair<Function<Q, ?>, Function<V, ?>> functions : this.getters.values()) {
                Function<V, ?> function = functions.getRight();
                if (!ObjectUtils.equals(function.apply(object), function.apply(other))) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }
}
