package com.chelvc.framework.base.config;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.impl.client.DefaultConnectionKeepAliveStrategy;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.protocol.HttpContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.RestTemplate;

/**
 * Http请求配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class HttpClientConfigurer {
    private final ObjectMapper objectMapper;
    private final HttpClientProperties properties;

    /**
     * 初始化默认的消息转换配置
     *
     * @param restTemplate RestTemplate实例
     */
    private void initializeDefaultMessageConverter(RestTemplate restTemplate) {
        List<HttpMessageConverter<?>> messageConverters = restTemplate.getMessageConverters().stream()
                .map(converter -> {
                    if (converter instanceof StringHttpMessageConverter) {
                        return new StringHttpMessageConverter(StandardCharsets.UTF_8);
                    } else if (converter instanceof MappingJackson2HttpMessageConverter) {
                        MappingJackson2HttpMessageConverter jacksonMessageConverter =
                                new MappingJackson2HttpMessageConverter(this.objectMapper);
                        jacksonMessageConverter.setSupportedMediaTypes(Arrays.asList(
                                MediaType.TEXT_PLAIN,
                                MediaType.APPLICATION_JSON,
                                new MediaType("application", "*+json")
                        ));
                        return jacksonMessageConverter;
                    }
                    return converter;
                }).collect(Collectors.toList());
        restTemplate.setMessageConverters(messageConverters);
    }

    @Bean
    public HttpClient httpClient() {
        HttpClientBuilder builder = HttpClientBuilder.create();
        PoolingHttpClientConnectionManager manager = new PoolingHttpClientConnectionManager();
        manager.setMaxTotal(this.properties.getMaxTotal());
        manager.setDefaultMaxPerRoute(this.properties.getMaxPerRoute());
        builder.setConnectionManager(manager);
        builder.setRetryHandler(new DefaultHttpRequestRetryHandler(this.properties.getRetryCount(), false));
        builder.setKeepAliveStrategy(new DefaultConnectionKeepAliveStrategy() {
            @Override
            public long getKeepAliveDuration(HttpResponse response, HttpContext context) {
                long duration = super.getKeepAliveDuration(response, context);
                return duration < 0 ? properties.getKeepAliveTime() : duration;
            }
        });

        // 设置后台线程剔除失效连接
        builder.evictExpiredConnections();
        builder.evictIdleConnections(this.properties.getMaxIdleTime(), TimeUnit.MILLISECONDS);

        // 禁用Cookie回话保持功能
        builder.disableCookieManagement();
        return builder.build();
    }

    @Bean
    public RestTemplate restTemplate() {
        HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory(this.httpClient());
        factory.setConnectTimeout(this.properties.getConnectTimeout());
        factory.setReadTimeout(this.properties.getSocketTimeout());
        factory.setConnectionRequestTimeout(this.properties.getConnectionRequestTimeout());
        RestTemplate template = new RestTemplate(factory);
        this.initializeDefaultMessageConverter(template);
        return template;
    }
}
