package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.Set;
import java.util.function.Predicate;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import com.beust.jcommander.internal.Sets;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.config.YamlPropertiesFactoryBean;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.annotation.Order;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.stereotype.Component;

/**
 * 应用上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class ApplicationContextHolder implements ApplicationContextAware {
    /**
     * 应用名称属性
     */
    public static final String APPLICATION_NAME_PROPERTY = "spring.application.name";

    /**
     * application.yml文件搜索路径
     */
    public static final String APPLICATION_YML_PATH =
            ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX + "/application.yml";

    /**
     * application.properties文件搜索路径
     */
    public static final String APPLICATION_PROPERTIES_PATH =
            ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX + "/application.properties";

    /**
     * 应用上下文对象
     */
    private static ApplicationContext APPLICATION_CONTEXT;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        APPLICATION_CONTEXT = applicationContext;
    }

    /**
     * 获取应用上下文对象
     *
     * @return 应用上下文对象
     */
    public static ApplicationContext getApplicationContext() {
        return getApplicationContext(true);
    }

    /**
     * 获取应用上下文对象
     *
     * @param required 应用上下文是否必须
     * @return 应用上下文对象
     */
    public static ApplicationContext getApplicationContext(boolean required) {
        if (required) {
            return AssertUtils.nonnull(APPLICATION_CONTEXT, () -> "Application context has not been initialized");
        }
        return APPLICATION_CONTEXT;
    }

    /**
     * 注册对象实例
     *
     * @param index    对象序号
     * @param type     对象类型
     * @param supplier 对象实例提供函数
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T register(int index, @NonNull Class<T> type, @NonNull Supplier<T> supplier) {
        AssertUtils.check(index > -1, () -> "Bean index must be greater than -1");
        String name = type.getName() + "_" + index;
        return register(name, type, supplier);
    }

    /**
     * 注册对象实例
     *
     * @param name     对象名称
     * @param type     对象类型
     * @param supplier 对象实例提供函数
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T register(@NonNull String name, @NonNull Class<T> type, @NonNull Supplier<T> supplier) {
        ApplicationContext applicationContext = getApplicationContext();
        ((GenericApplicationContext) getApplicationContext()).registerBean(name, type, supplier);
        return applicationContext.getBean(name, type);
    }

    /**
     * 获取应用环境对象
     *
     * @return 应用环境对象
     */
    public static Environment getEnvironment() {
        return getEnvironment(true);
    }

    /**
     * 获取应用环境对象
     *
     * @param required 应用环境是否必须
     * @return 应用环境对象
     */
    public static Environment getEnvironment(boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        Environment environment = ObjectUtils.ifNull(applicationContext, ApplicationContext::getEnvironment);
        if (required) {
            return AssertUtils.nonnull(environment, () -> "Application environment has not been initialized");
        }
        return environment;
    }

    /**
     * 获取环境变量值
     *
     * @param key 变量名称
     * @return 变量值
     */
    public static String getProperty(String key) {
        return getEnvironment().getProperty(key);
    }

    /**
     * 获取安全的环境变量值
     *
     * @param key 变量名称
     * @return 变量值
     */
    public static String getSafeProperty(String key) {
        Environment environment = getEnvironment(false);
        if (environment == null) {
            return null;
        }
        try {
            return environment.getProperty(key);
        } catch (Exception e) {
            log.warn("Application property get failed: {}, {}", key, e.getMessage());
        }
        return null;
    }

    /**
     * 获取环境变量值
     *
     * @param key          变量名称
     * @param defaultValue 默认值
     * @return 变量值
     */
    public static String getProperty(String key, String defaultValue) {
        return getEnvironment().getProperty(key, defaultValue);
    }

    /**
     * 获取安全的环境变量值
     *
     * @param key          变量名称
     * @param defaultValue 默认值
     * @return 变量值
     */
    public static String getSafeProperty(String key, String defaultValue) {
        Environment environment = getEnvironment(false);
        if (environment == null) {
            return defaultValue;
        }
        try {
            return environment.getProperty(key, defaultValue);
        } catch (Exception e) {
            log.warn("Application property get failed: {}, {}", key, e.getMessage());
        }
        return defaultValue;
    }

    /**
     * 获取环境变量值
     *
     * @param key        变量名称
     * @param targetType 变量值类型
     * @param <T>        变量值类型
     * @return 变量值
     */
    public static <T> T getProperty(String key, Class<T> targetType) {
        return getEnvironment().getProperty(key, targetType);
    }

    /**
     * 获取安全的环境变量值
     *
     * @param key        变量名称
     * @param targetType 变量值类型
     * @param <T>        变量值类型
     * @return 变量值
     */
    public static <T> T getSafeProperty(String key, Class<T> targetType) {
        Environment environment = getEnvironment(false);
        if (environment == null) {
            return null;
        }
        try {
            return environment.getProperty(key, targetType);
        } catch (Exception e) {
            log.warn("Application property get failed: {}, {}", key, e.getMessage());
        }
        return null;
    }

    /**
     * 获取环境变量值
     *
     * @param key        变量名称
     * @param targetType 变量值类型
     * @param <T>        变量值类型
     * @return 变量值
     */
    public static <T> T getProperty(String key, TypeReference<T> targetType) {
        return JacksonUtils.deserialize(getProperty(key), targetType);
    }

    /**
     * 获取安全的环境变量值
     *
     * @param key        变量名称
     * @param targetType 变量值类型
     * @param <T>        变量值类型
     * @return 变量值
     */
    public static <T> T getSafeProperty(String key, TypeReference<T> targetType) {
        try {
            return JacksonUtils.deserialize(getSafeProperty(key), targetType);
        } catch (Exception e) {
            log.warn("Application property get failed: {}, {}", key, e.getMessage());
        }
        return null;
    }

    /**
     * 获取环境变量值
     *
     * @param key          变量名称
     * @param targetType   变量值类型
     * @param defaultValue 默认值
     * @param <T>          变量值类型
     * @return 环境变量值
     */
    public static <T> T getProperty(String key, Class<T> targetType, T defaultValue) {
        return getEnvironment().getProperty(key, targetType, defaultValue);
    }

    /**
     * 获取安全的环境变量值
     *
     * @param key          变量名称
     * @param targetType   变量值类型
     * @param defaultValue 默认值
     * @param <T>          变量值类型
     * @return 环境变量值
     */
    public static <T> T getSafeProperty(String key, Class<T> targetType, T defaultValue) {
        Environment environment = getEnvironment(false);
        if (environment == null) {
            return defaultValue;
        }
        try {
            return environment.getProperty(key, targetType, defaultValue);
        } catch (Exception e) {
            log.warn("Application property get failed: {}, {}", key, e.getMessage());
        }
        return defaultValue;
    }

    /**
     * 获取环境变量值
     *
     * @param key          变量名称
     * @param targetType   变量值类型
     * @param defaultValue 默认值
     * @param <T>          变量值类型
     * @return 环境变量值
     */
    public static <T> T getProperty(String key, TypeReference<T> targetType, T defaultValue) {
        return ObjectUtils.ifNull(getProperty(key, targetType), defaultValue);
    }

    /**
     * 获取安全的环境变量值
     *
     * @param key          变量名称
     * @param targetType   变量值类型
     * @param defaultValue 默认值
     * @param <T>          变量值类型
     * @return 环境变量值
     */
    public static <T> T getSafeProperty(String key, TypeReference<T> targetType, T defaultValue) {
        Environment environment = getEnvironment(false);
        if (environment == null) {
            return defaultValue;
        }
        try {
            return ObjectUtils.ifNull(getSafeProperty(key, targetType), defaultValue);
        } catch (Exception e) {
            log.warn("Application property get failed: {}, {}", key, e.getMessage());
        }
        return defaultValue;
    }

    /**
     * 获取必须的环境变量
     *
     * @param key 变量名称
     * @return 变量值
     * @throws IllegalStateException 状态异常
     */
    public static String getRequiredProperty(String key) throws IllegalStateException {
        return getEnvironment().getRequiredProperty(key);
    }

    /**
     * 获取必须的环境变量
     *
     * @param key        变量名称
     * @param targetType 变量值类型
     * @param <T>        变量值类型
     * @return 变量值
     * @throws IllegalStateException 状态异常
     */
    public static <T> T getRequiredProperty(String key, Class<T> targetType) throws IllegalStateException {
        return getEnvironment().getRequiredProperty(key, targetType);
    }

    /**
     * 获取必须的环境变量
     *
     * @param key        变量名称
     * @param targetType 变量值类型
     * @param <T>        变量值类型
     * @return 变量值
     * @throws IllegalStateException 状态异常
     */
    public static <T> T getRequiredProperty(String key, TypeReference<T> targetType) throws IllegalStateException {
        return JacksonUtils.deserialize(getRequiredProperty(key), targetType);
    }

    /**
     * 获取对象实例
     *
     * @param type 对象类型
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(Class<T> type) {
        return getBean(type, true);
    }

    /**
     * 获取对象实例
     *
     * @param type     对象类型
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(Class<T> type, boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        if (applicationContext == null) {
            return null;
        }
        try {
            return applicationContext.getBean(type);
        } catch (NoSuchBeanDefinitionException e) {
            if (required) {
                throw e;
            }
            return null;
        }
    }

    /**
     * 获取对象实例
     *
     * @param name 对象名称
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(String name) {
        return getBean(name, true);
    }

    /**
     * 获取对象实例
     *
     * @param name     对象名称
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    @SuppressWarnings("unchecked")
    public static <T> T getBean(String name, boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        if (applicationContext == null) {
            return null;
        }
        try {
            return (T) applicationContext.getBean(name);
        } catch (NoSuchBeanDefinitionException e) {
            if (required) {
                throw e;
            }
            return null;
        }
    }

    /**
     * 获取对象实例
     *
     * @param name 对象名称
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getSafeBean(String name) {
        return getSafeBean(name, true);
    }

    /**
     * 获取对象实例
     *
     * @param name     对象名称
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T getSafeBean(String name, boolean required) {
        T bean = getBean(name, false);
        if (bean != null) {
            return bean;
        }

        // 如果名称包含包路径，则尝试获取简单类名获取
        int separator = name.lastIndexOf('.');
        if (separator >= 0 && (bean = getBean(StringUtils.hump(name.substring(separator + 1)), false)) != null) {
            return bean;
        }
        if (required) {
            throw new NoSuchBeanDefinitionException(name);
        }
        return null;
    }

    /**
     * 获取对象名称/实例映射表
     *
     * @param type 对象类型
     * @param <T>  对象类型
     * @return 对象名称/实例映射表
     */
    public static <T> Map<String, T> getBeans(@NonNull Class<T> type) {
        return getApplicationContext().getBeansOfType(type);
    }

    /**
     * 获取所有application.yml文件资源
     *
     * @return 资源对象数组
     */
    public static List<Resource> getApplicationResources() {
        Resource[] ymls, properties;
        ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();

        // 查找yml配置文件
        try {
            ymls = resourcePatternResolver.getResources(APPLICATION_YML_PATH);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 查找properties配置文件
        try {
            properties = resourcePatternResolver.getResources(APPLICATION_PROPERTIES_PATH);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 构建配置文件列表
        if (ObjectUtils.isEmpty(ymls) && ObjectUtils.isEmpty(properties)) {
            return Collections.emptyList();
        }
        List<Resource> resources = Lists.newArrayListWithCapacity(ymls.length + properties.length);
        resources.addAll(Arrays.asList(ymls));
        resources.addAll(Arrays.asList(properties));
        return resources;
    }

    /**
     * 获取应用名称
     *
     * @return 应用名称
     */
    public static String getApplicationName() {
        return getApplicationName(getApplicationContext());
    }

    /**
     * 获取应用名称
     *
     * @param applicationContext 应用上下文实例
     * @return 应用名称
     */
    public static String getApplicationName(@NonNull ApplicationContext applicationContext) {
        return applicationContext.getEnvironment().getProperty(APPLICATION_NAME_PROPERTY);
    }

    /**
     * 根据application.yml/application.properties文件资源获取应用名称
     *
     * @param resource application.yml/application.properties文件资源对象实例
     * @return 应用名称
     */
    public static String getApplicationName(@NonNull Resource resource) {
        Properties properties;
        String filename = resource.getFilename();
        if (StringUtils.isEmpty(filename) || filename.endsWith(".yml")) {
            // application.yml
            YamlPropertiesFactoryBean yaml = new YamlPropertiesFactoryBean();
            yaml.setResources(resource);
            properties = Objects.requireNonNull(yaml.getObject());
        } else {
            // application.properties
            properties = new Properties();
            try (InputStream input = resource.getInputStream()) {
                properties.load(input);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return properties.getProperty(APPLICATION_NAME_PROPERTY);
    }

    /**
     * 获取资源前缀
     *
     * @param resource 资源对象
     * @return 资源前缀标识
     */
    public static String getResourcePrefix(@NonNull Resource resource) {
        String application = getApplicationName(resource);
        return StringUtils.isEmpty(application) ? application : application.replace('.', '/');
    }

    /**
     * 判断类对象是否属于指定资源
     *
     * @param resource 资源对象
     * @param clazz    类对象
     * @return true/false
     */
    public static boolean isResourceClass(@NonNull Resource resource, @NonNull Class<?> clazz) {
        String file = clazz.getProtectionDomain().getCodeSource().getLocation().getPath();
        try {
            return resource.getURL().getPath().contains(file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 查找类对象所属资源
     *
     * @param clazz     类对象
     * @param resources 资源对象集合
     * @return 资源对象
     */
    public static Resource lookupClassResource(@NonNull Class<?> clazz, @NonNull Collection<Resource> resources) {
        if (ObjectUtils.isEmpty(resources)) {
            return null;
        }
        return resources.stream().filter(resource -> isResourceClass(resource, clazz)).findAny().orElse(null);
    }

    /**
     * 查找应用中的类对象
     *
     * @param condition 类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> lookupApplicationClasses(@NonNull Predicate<Class<?>> condition) {
        return lookupApplicationClasses(getApplicationContext(), condition);
    }

    /**
     * 查找应用中的类对象
     *
     * @param applicationContext 应用上下文对象
     * @param condition          类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> lookupApplicationClasses(@NonNull ApplicationContext applicationContext,
                                                          @NonNull Predicate<Class<?>> condition) {
        Set<String> packages = Sets.newHashSet();
        applicationContext.getBeansWithAnnotation(SpringBootApplication.class).forEach((key, value) -> {
            Class<?> clazz = value.getClass();

            // 加载@ComponentScan包扫描路径配置
            ComponentScan componentScan = AnnotationUtils.findAnnotation(clazz, ComponentScan.class);
            if (componentScan != null) {
                packages.addAll(Arrays.asList(componentScan.basePackages()));
            }

            // 加载@SpringBootApplication包扫描路径配置
            SpringBootApplication springBootApplication =
                    AnnotationUtils.findAnnotation(clazz, SpringBootApplication.class);
            if (springBootApplication != null) {
                packages.addAll(Arrays.asList(springBootApplication.scanBasePackages()));
            }
        });

        // 根据包扫描路径查找类对象
        if (packages.isEmpty()) {
            return SpringUtils.lookupClasses(StringUtils.EMPTY, condition);
        }
        return packages.stream().flatMap(pack -> SpringUtils.lookupClasses(pack, condition).stream())
                .distinct().collect(Collectors.toList());
    }

    /**
     * 获取国际化消息
     *
     * @param code 消息标识
     * @param args 消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull String code, @NonNull Object... args) {
        return getMessage(code, code, args);
    }

    /**
     * 获取国际化消息
     *
     * @param code     消息标识
     * @param defaults 默认消息
     * @param args     消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull String code, String defaults, @NonNull Object... args) {
        return getMessage(LocaleContextHolder.getLocale(), code, defaults, args);
    }

    /**
     * 获取国际化消息
     *
     * @param locale 本地信息
     * @param code   消息标识
     * @param args   消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull Locale locale, @NonNull String code, @NonNull Object... args) {
        return getMessage(locale, code, code, args);
    }

    /**
     * 获取国际化消息
     *
     * @param locale   本地信息
     * @param code     消息标识
     * @param defaults 默认消息
     * @param args     消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull Locale locale, @NonNull String code, String defaults,
                                    @NonNull Object... args) {
        if (Objects.isNull(defaults)) {
            defaults = code;
        }
        ApplicationContext applicationContext = getApplicationContext(false);
        if (applicationContext != null) {
            try {
                return applicationContext.getMessage(code, args, defaults, locale);
            } catch (Exception e) {
                log.warn("Application message get failed [{}]: {}, {}", locale, code, e.getMessage());
            }
        }
        return defaults;
    }
}
