package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Objects;
import java.util.function.Function;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Result;
import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.JsonEncoding;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * 会话上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SessionContextHolder implements ServletRequestListener {
    /**
     * 会话上下文名称
     */
    public static final String NAME = "session";

    /**
     * 身份标识请求头
     */
    public static final String HEADER_ID = "id";

    /**
     * 应用范围请求头
     */
    public static final String HEADER_SCOPE = "scope";

    /**
     * 设备标识请求头
     */
    public static final String HEADER_DEVICE = "device";

    /**
     * 渠道来源请求头
     */
    public static final String HEADER_CHANNEL = "channel";

    /**
     * 应用平台请求头
     */
    public static final String HEADER_PLATFORM = "platform";

    /**
     * 终端编号请求头
     */
    public static final String HEADER_TERMINAL = "terminal";

    /**
     * 终端版本号请求头
     */
    public static final String HEADER_VERSION = "version";

    /**
     * 是否匿名请求头
     */
    public static final String HEADER_ANONYMOUS = "anonymous";

    /**
     * 时间戳请求头
     */
    public static final String HEADER_TIMESTAMP = "timestamp";

    /**
     * 签名信息请求头
     */
    public static final String HEADER_SIGNATURE = "signature";

    /**
     * 设备指纹请求头
     */
    public static final String HEADER_FINGERPRINT = "fingerprint";

    /**
     * 认证信息请求头
     */
    public static final String HEADER_AUTHORIZATION = "authorization";

    /**
     * 空会话对象实例
     */
    private static final Session EMPTY_SESSION = new Session();

    /**
     * 用户会话上下文
     */
    private static final ThreadLocal<Deque<Session>> SESSION_CONTEXT = ThreadLocal.withInitial(ArrayDeque::new);

    /**
     * 获取会话信息
     *
     * @return 会话信息
     */
    public static Session getSession() {
        return getSession(true);
    }

    /**
     * 获取会话信息
     *
     * @param required 是否必须
     * @return 会话信息
     */
    public static Session getSession(boolean required) {
        Session session = SESSION_CONTEXT.get().peek();
        if (session == EMPTY_SESSION) {
            session = null;
        }
        if (required) {
            return AssertUtils.nonnull(session, () -> "Session has not been initialized");
        }
        return session;
    }

    /**
     * 设置会话信息
     *
     * @param session 会话信息
     */
    public static void setSession(Session session) {
        SESSION_CONTEXT.get().push(ObjectUtils.ifNull(session, EMPTY_SESSION));
    }

    /**
     * 初始化会话信息
     *
     * @param id        主体标识
     * @param scope     应用范围
     * @param anonymous 是否匿名
     * @return 会话信息
     */
    public static Session initializeSession(@NonNull Long id, @NonNull String scope, boolean anonymous) {
        Deque<Session> deque = SESSION_CONTEXT.get();
        Session session, original = deque.poll();
        if (original == null || original == EMPTY_SESSION) {
            session = Session.builder().id(id).scope(scope).anonymous(anonymous)
                    .timestamp(System.currentTimeMillis()).build();
        } else {
            session = Session.builder().id(id).scope(scope).anonymous(anonymous).host(original.getHost())
                    .device(original.getDevice()).channel(original.getChannel()).platform(original.getPlatform())
                    .terminal(original.getTerminal()).version(original.getVersion())
                    .fingerprint(original.getFingerprint()).timestamp(original.getTimestamp()).build();
        }
        deque.push(session);
        return session;
    }

    /**
     * 获取当前用户ID
     *
     * @return 用户ID
     */
    public static Long getId() {
        return ObjectUtils.ifNull(getSession(false), Session::getId);
    }

    /**
     * 获取客户端身份标识
     *
     * @param request Http请求对象
     * @return 身份标识
     */
    public static Long getId(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_ID), Long::parseLong);
    }

    /**
     * 获取当前会话请求地址
     *
     * @return 请求地址
     */
    public static String getHost() {
        return ObjectUtils.ifNull(getSession(false), Session::getHost);
    }

    /**
     * 获取客户端主机地址
     *
     * @param request Http请求对象
     * @return 主机地址
     */
    public static String getHost(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(HttpUtils.getHost(request), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @return 应用范围
     */
    public static String getScope() {
        return ObjectUtils.ifNull(getSession(false), Session::getScope);
    }

    /**
     * 获取应用范围
     *
     * @param request Http请求对象
     * @return 应用范围
     */
    public static String getScope(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_SCOPE), (String) null);
    }

    /**
     * 获取当前会话设备标识
     *
     * @return 设备标识
     */
    public static String getDevice() {
        return ObjectUtils.ifNull(getSession(false), Session::getDevice);
    }

    /**
     * 获取客户端设备标识
     *
     * @param request Http请求对象
     * @return 设备标识
     */
    public static String getDevice(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_DEVICE), (String) null);
    }

    /**
     * 获取当前会话渠道来源
     *
     * @return 渠道来源
     */
    public static String getChannel() {
        return ObjectUtils.ifNull(getSession(false), Session::getChannel);
    }

    /**
     * 获取客户端渠道来源
     *
     * @param request Http请求对象
     * @return 渠道来源
     */
    public static String getChannel(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_CHANNEL), (String) null);
    }

    /**
     * 获取当前会话平台信息
     *
     * @return 平台信息
     */
    public static Platform getPlatform() {
        return ObjectUtils.ifNull(getSession(false), Session::getPlatform);
    }

    /**
     * 获取客户端平台标识
     *
     * @param request Http请求对象
     * @return 平台标识
     */
    public static Platform getPlatform(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_PLATFORM), Platform::valueOf);
    }

    /**
     * 获取当前会话终端信息
     *
     * @return 终端信息
     */
    public static Terminal getTerminal() {
        return ObjectUtils.ifNull(getSession(false), Session::getTerminal);
    }

    /**
     * 获取客户端终端
     *
     * @param request Http请求对象
     * @return 终端标识
     */
    public static Terminal getTerminal(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_TERMINAL), Terminal::valueOf);
    }

    /**
     * 获取当前会话版本信息
     *
     * @return 版本信息
     */
    public static String getVersion() {
        return ObjectUtils.ifNull(getSession(false), Session::getVersion);
    }

    /**
     * 获取客户端版本号
     *
     * @param request Http请求对象
     * @return 版本号
     */
    public static String getVersion(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_VERSION), (String) null);
    }

    /**
     * 是否是匿名用户
     *
     * @return true/false
     */
    public static boolean isAnonymous() {
        Session session = getSession(false);
        return session == null || session.isAnonymous();
    }

    /**
     * 是否是匿名用户
     *
     * @param request Http请求对象
     * @return true/false
     */
    public static boolean isAnonymous(@NonNull HttpServletRequest request) {
        String anonymous = request.getHeader(HEADER_ANONYMOUS);
        return StringUtils.isEmpty(anonymous) || Objects.equals(anonymous, StringUtils.TRUE);
    }

    /**
     * 获取请求时间戳
     *
     * @return 时间戳
     */
    public static Long getTimestamp() {
        return ObjectUtils.ifNull(getSession(false), Session::getTimestamp);
    }

    /**
     * 获取请求时间戳
     *
     * @param request Http请求对象
     * @return 时间戳
     */
    public static Long getTimestamp(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_TIMESTAMP), Long::parseLong);
    }

    /**
     * 获取签名信息
     *
     * @return 签名信息
     */
    public static String getSignature() {
        return ObjectUtils.ifNull(getRequest(), SessionContextHolder::getSignature, () -> null);
    }

    /**
     * 获取签名信息
     *
     * @param request Http请求对象
     * @return 签名信息
     */
    public static String getSignature(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_SIGNATURE), (String) null);
    }

    /**
     * 获取设备指纹
     *
     * @return 设备指纹
     */
    public static String getFingerprint() {
        return ObjectUtils.ifNull(getSession(false), Session::getFingerprint);
    }

    /**
     * 获取设备指纹
     *
     * @param request Http请求对象
     * @return 设备指纹
     */
    public static String getFingerprint(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_FINGERPRINT), (String) null);
    }

    /**
     * 获取认证信息
     *
     * @return 认证信息
     */
    public static String getAuthorization() {
        HttpServletRequest request = getRequest();
        return request == null ? null : getAuthorization(request);
    }

    /**
     * 获取认证信息
     *
     * @param request Http请求对象
     * @return 认证信息
     */
    public static String getAuthorization(@NonNull HttpServletRequest request) {
        return StringUtils.ifEmpty(request.getHeader(HEADER_AUTHORIZATION), (String) null);
    }

    /**
     * 获取当前请求对象
     *
     * @return Http请求对象
     */
    public static HttpServletRequest getRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return ObjectUtils.ifNull(attributes, ServletRequestAttributes::getRequest);
    }

    /**
     * 获取当前响应对象
     *
     * @return Http响应对象
     */
    public static HttpServletResponse getResponse() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return ObjectUtils.ifNull(attributes, ServletRequestAttributes::getResponse);
    }

    /**
     * 获取请求参数
     *
     * @param name 参数名称
     * @return 参数值
     */
    public static String getParameter(@NonNull String name) {
        return getParameter(name, value -> value);
    }

    /**
     * 获取请求参数
     *
     * @param name    参数名称
     * @param adapter 参数值适配器
     * @return 参数值
     */
    public static <T> T getParameter(@NonNull String name, @NonNull Function<String, T> adapter) {
        return StringUtils.ifEmpty(ObjectUtils.ifNull(getRequest(), request -> request.getParameter(name)), adapter);
    }

    /**
     * 将Http状态码转换成请求结果
     *
     * @param status Http状态码
     * @param <T>    结果数据类型
     * @return 请求结果对象实例
     */
    public static <T> Result<T> result(@NonNull HttpStatus status) {
        if (status.is5xxServerError()) {
            return Result.failure();
        } else if (status.isError()) {
            return Result.of(status.name(), null, status.getReasonPhrase());
        }
        return Result.success();
    }

    /**
     * 将Http状态码转换成请求结果
     *
     * @param status  Http状态码
     * @param message 结果消息
     * @param <T>     结果数据类型
     * @return 请求结果对象实例
     */
    public static <T> Result<T> result(@NonNull HttpStatus status, String message) {
        return Result.of(status.name(), null, message);
    }

    /**
     * 将响应结果转换成Http状态码
     *
     * @param result 结果对象实例
     * @return Http状态码
     */
    public static HttpStatus result2status(Result<?> result) {
        if (result == null || result.isSuccess()) {
            return HttpStatus.OK;
        }
        return result.isFailure() ? HttpStatus.INTERNAL_SERVER_ERROR : HttpStatus.BAD_REQUEST;
    }

    /**
     * 相应请求结果
     *
     * @param response Http响应对象
     * @param status   Http状态码
     * @throws IOException I/O异常
     */
    public static void response(@NonNull HttpServletResponse response, @NonNull HttpStatus status) throws IOException {
        response(response, result(status));
    }

    /**
     * 相应请求结果
     *
     * @param response Http响应对象
     * @param status   Http状态码
     * @param message  结果消息
     * @throws IOException I/O异常
     */
    public static void response(@NonNull HttpServletResponse response, @NonNull HttpStatus status, String message)
            throws IOException {
        response(response, result(status, message));
    }

    /**
     * 相应请求结果
     *
     * @param response Http响应对象
     * @param result   请求结果
     * @throws IOException I/O异常
     */
    public static void response(@NonNull HttpServletResponse response, @NonNull Result<?> result) throws IOException {
        response.setStatus(result2status(result).value());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.setCharacterEncoding(JsonEncoding.UTF8.getJavaName());
        OutputStream output = response.getOutputStream();
        JacksonContextHolder.write(output, result);
        output.flush();
    }


    /**
     * 清空上下文信息
     */
    public static void clearSessionContext() {
        Deque<Session> sessions = SESSION_CONTEXT.get();
        sessions.poll();
        if (sessions.isEmpty()) {
            SESSION_CONTEXT.remove();
        }
    }

    @Override
    public void requestInitialized(ServletRequestEvent event) {
        HttpServletRequest request = (HttpServletRequest) event.getServletRequest();
        Session session = Session.builder().id(getId(request)).host(getHost(request)).scope(getScope(request))
                .device(getDevice(request)).channel(getChannel(request)).platform(getPlatform(request))
                .terminal(getTerminal(request)).version(getVersion(request)).fingerprint(getFingerprint(request))
                .anonymous(isAnonymous(request)).timestamp(getTimestamp(request)).build();
        setSession(session);
    }

    @Override
    public void requestDestroyed(ServletRequestEvent event) {
        clearSessionContext();
    }
}
