package com.chelvc.framework.base.context;

import java.util.Collection;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import java.util.concurrent.ForkJoinPool;
import java.util.function.Consumer;
import java.util.function.Supplier;

import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.util.ThreadUtils;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.task.TaskExecutor;
import org.springframework.scheduling.TaskScheduler;
import org.springframework.util.CollectionUtils;

/**
 * 多线程上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class ThreadContextHolder {
    /**
     * 空任务结果数组
     */
    private static final CompletableFuture<?>[] EMPTY_FUTURES = new CompletableFuture<?>[0];

    /**
     * 任务执行线程池
     */
    private static TaskExecutor TASK_EXECUTOR;

    /**
     * 任务调度器
     */
    private static TaskScheduler TASK_SCHEDULER;

    /**
     * 默认任务执行器
     */
    private static final Executor DEFAULT_EXECUTOR = ForkJoinPool.getCommonPoolParallelism() > 1 ?
            ForkJoinPool.commonPool() : ThreadUtils::run;

    private ThreadContextHolder() {
    }

    /**
     * 获取默认任务执行器
     *
     * @return 任务执行器实例
     */
    public static Executor getDefaultExecutor() {
        return DEFAULT_EXECUTOR;
    }

    /**
     * 获取任务执行线程池
     *
     * @return 任务执行线程池实例
     */
    public static TaskExecutor getTaskExecutor() {
        if (TASK_EXECUTOR == null) {
            synchronized (TaskExecutor.class) {
                if (TASK_EXECUTOR == null) {
                    TASK_EXECUTOR = ApplicationContextHolder.getBean(TaskExecutor.class);
                }
            }
        }
        return TASK_EXECUTOR;
    }

    /**
     * 获取任务调度器
     *
     * @return 任务调度器实例
     */
    public static TaskScheduler getTaskScheduler() {
        if (TASK_SCHEDULER == null) {
            synchronized (TaskScheduler.class) {
                if (TASK_SCHEDULER == null) {
                    TASK_SCHEDULER = ApplicationContextHolder.getBean(TaskScheduler.class);
                }
            }
        }
        return TASK_SCHEDULER;
    }

    /**
     * 异步调用方法
     *
     * @param runnable Runnable实例
     * @return CompletableFuture实例
     */
    public static CompletableFuture<Void> run(@NonNull Runnable runnable) {
        return run(getDefaultExecutor(), runnable);
    }

    /**
     * 异步调用方法
     *
     * @param executor 线程执行器
     * @param runnable Runnable实例
     * @return CompletableFuture实例
     */
    public static CompletableFuture<Void> run(@NonNull Executor executor, @NonNull Runnable runnable) {
        Session session = SessionContextHolder.getSession(false);
        return CompletableFuture.runAsync(
                () -> {
                    SessionContextHolder.setSession(session);
                    try {
                        runnable.run();
                    } finally {
                        SessionContextHolder.clearSessionContext();
                    }
                },
                executor
        ).exceptionally(e -> {
            log.error(e.getMessage(), e);
            return null;
        });
    }

    /**
     * 异步调用方法
     *
     * @param supplier 回调方法实例
     * @param <T>      任务结果类型
     * @return CompletableFuture实例
     */
    public static <T> CompletableFuture<T> supply(@NonNull Supplier<T> supplier) {
        return supply(getDefaultExecutor(), supplier);
    }

    /**
     * 异步调用方法
     *
     * @param executor 线程执行器
     * @param supplier 回调方法实例
     * @param <T>      任务结果类型
     * @return CompletableFuture实例
     */
    public static <T> CompletableFuture<T> supply(@NonNull Executor executor, @NonNull Supplier<T> supplier) {
        Session session = SessionContextHolder.getSession(false);
        return CompletableFuture.supplyAsync(
                () -> {
                    SessionContextHolder.setSession(session);
                    try {
                        return supplier.get();
                    } finally {
                        SessionContextHolder.clearSessionContext();
                    }
                },
                executor
        ).exceptionally(e -> {
            log.error(e.getMessage(), e);
            return null;
        });
    }

    /**
     * 异步执行方法（等待执行完成）
     *
     * @param array Runnable实例数组
     */
    public static void execute(@NonNull Runnable... array) {
        execute(getDefaultExecutor(), array);
    }

    /**
     * 异步执行方法（等待执行完成）
     *
     * @param executor 线程执行器
     * @param array    Runnable实例数组
     */
    public static void execute(@NonNull Executor executor, @NonNull Runnable... array) {
        if (array != null && array.length == 1) {
            run(executor, array[0]).join();
        } else if (array != null) {
            CompletableFuture<?>[] futures = new CompletableFuture<?>[array.length];
            for (int i = 0; i < array.length; i++) {
                futures[i] = run(executor, array[i]);
            }
            CompletableFuture.allOf(futures).join();
        }
    }

    /**
     * 异步执行方法（等待执行完成）
     *
     * @param array 回调方法数组
     * @param <T>   任务结果类型
     * @return CompletableFuture实例
     */
    @SuppressWarnings("unchecked")
    public static <T> CompletableFuture<T>[] execute(@NonNull Supplier<T>... array) {
        return execute(getDefaultExecutor(), array);
    }

    /**
     * 异步执行方法（等待执行完成）
     *
     * @param executor 线程执行器
     * @param array    回调方法数组
     * @param <T>      任务结果类型
     * @return CompletableFuture实例
     */
    @SuppressWarnings("unchecked")
    public static <T> CompletableFuture<T>[] execute(@NonNull Executor executor, @NonNull Supplier<T>... array) {
        if (array == null || array.length == 0) {
            return new CompletableFuture[0];
        }
        CompletableFuture<T>[] futures = new CompletableFuture[array.length];
        for (int i = 0; i < array.length; i++) {
            futures[i] = supply(executor, array[i]);
        }
        CompletableFuture.allOf(futures).join();
        return futures;
    }

    /**
     * 等待任务执行完成
     *
     * @param futures 任务数组
     * @param <T>     任务结果类型
     */
    @SafeVarargs
    public static <T> void join(@NonNull CompletableFuture<T>... futures) {
        if (futures != null && futures.length > 0) {
            CompletableFuture.allOf(futures).join();
        }
    }

    /**
     * 等待任务执行完成
     *
     * @param futures 任务集合
     * @param <T>     任务结果类型
     */
    public static <T> void join(@NonNull Collection<CompletableFuture<T>> futures) {
        if (!CollectionUtils.isEmpty(futures)) {
            CompletableFuture.allOf(futures.toArray(EMPTY_FUTURES)).join();
        }
    }

    /**
     * 获取任务执行结果
     *
     * @param futures  任务数组
     * @param consumer 任务结果处理方法
     * @param <T>      任务结果对象类型
     */
    public static <T> void results(@NonNull CompletableFuture<T>[] futures, @NonNull Consumer<T> consumer) {
        if (futures != null && futures.length > 0) {
            join(futures);
            for (CompletableFuture<T> future : futures) {
                try {
                    consumer.accept(future.get());
                } catch (InterruptedException | ExecutionException e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }

    /**
     * 获取任务执行结果
     *
     * @param futures  任务集合
     * @param consumer 任务结果处理方法
     * @param <T>      任务结果对象类型
     */
    public static <T> void results(@NonNull Collection<CompletableFuture<T>> futures, @NonNull Consumer<T> consumer) {
        if (!CollectionUtils.isEmpty(futures)) {
            join(futures);
            futures.forEach(future -> {
                try {
                    consumer.accept(future.get());
                } catch (InterruptedException | ExecutionException e) {
                    throw new RuntimeException(e);
                }
            });
        }
    }
}
