package com.chelvc.framework.base.interceptor;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.ValidationException;

import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.exception.ParameterInvalidException;
import com.chelvc.framework.common.model.Result;
import com.chelvc.framework.common.util.ErrorUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.exc.InvalidFormatException;
import com.google.common.collect.ImmutableMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.TypeMismatchException;
import org.springframework.boot.autoconfigure.web.servlet.error.AbstractErrorController;
import org.springframework.boot.web.error.ErrorAttributeOptions;
import org.springframework.boot.web.servlet.error.DefaultErrorAttributes;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotAcceptableException;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingMatrixVariableException;
import org.springframework.web.bind.MissingPathVariableException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.multipart.MultipartException;
import org.springframework.web.multipart.support.MissingServletRequestPartException;

/**
 * 异常处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Controller
@RestControllerAdvice
public class GlobalExceptionInterceptor extends AbstractErrorController {
    private final ErrorAttributeOptions options = ErrorAttributeOptions.of(ErrorAttributeOptions.Include.values());

    public GlobalExceptionInterceptor() {
        super(new DefaultErrorAttributes(), Collections.emptyList());
    }

    @Override
    public String getErrorPath() {
        return null;
    }

    /**
     * 参数绑定结果格式化
     *
     * @param result 参数绑定结果
     * @return 参数名称/异常消息映射表
     */
    private Map<String, String> format(BindingResult result) {
        List<FieldError> errors = ObjectUtils.ifNull(result, BindingResult::getFieldErrors);
        if (CollectionUtils.isEmpty(errors)) {
            return Collections.emptyMap();
        }
        return errors.stream().collect(Collectors.toMap(FieldError::getField,
                e -> ObjectUtils.ifNull(e.getDefaultMessage(), StringUtils.EMPTY), (o, n) -> n));
    }

    /**
     * 参数类型异常格式化
     *
     * @param e 参数格式异常
     * @return 参数名称/异常消息映射表
     */
    private Map<String, String> format(InvalidFormatException e) {
        List<JsonMappingException.Reference> references = e.getPath();
        if (CollectionUtils.isEmpty(references)) {
            return Collections.emptyMap();
        }
        return references.stream().collect(Collectors.toMap(
                JsonMappingException.Reference::getFieldName, r -> "Invalid value"
        ));
    }

    /**
     * 参数校验异常格式化
     *
     * @param e 参数校验异常
     * @return 参数名称/异常消息映射表
     */
    private Map<String, String> format(ConstraintViolationException e) {
        Set<ConstraintViolation<?>> errors =
                ObjectUtils.ifNull(e, ConstraintViolationException::getConstraintViolations);
        if (CollectionUtils.isEmpty(errors)) {
            return Collections.emptyMap();
        }
        return errors.stream().filter(Objects::nonNull).collect(Collectors.toMap(
                error -> {
                    String path = String.valueOf(error.getPropertyPath());
                    int index = path.lastIndexOf('.');
                    return index > -1 ? path.substring(index + 1) : path;
                },
                error -> ObjectUtils.ifNull(error.getMessage(), StringUtils.EMPTY)
        ));
    }

    /**
     * 参数类型异常格式化
     *
     * @param e 参数类型异常
     * @return 参数名称/异常消息映射表
     */
    private Map<String, String> format(MethodArgumentTypeMismatchException e) {
        return ImmutableMap.of(e.getName(),
                ObjectUtils.ifNull(e.getRequiredType(), Class::getSimpleName, () -> StringUtils.EMPTY));
    }

    /**
     * 将异常转换成结果对象
     *
     * @param e 异常对象
     * @return 结果对象
     */
    private Result<?> error2result(Throwable e) {
        if (e instanceof HttpMessageNotReadableException) {
            Throwable cause = e.getCause();
            if (cause instanceof InvalidFormatException) {
                return ParameterInvalidException.result(this.format((InvalidFormatException) cause));
            }
            return Result.of(HttpStatus.BAD_REQUEST.name(), null, HttpStatus.BAD_REQUEST.getReasonPhrase());
        } else if (e instanceof MissingPathVariableException) {
            return ParameterInvalidException.result(ImmutableMap.of(
                    ((MissingPathVariableException) e).getVariableName(), e.getMessage()
            ));
        } else if (e instanceof MissingMatrixVariableException) {
            return ParameterInvalidException.result(ImmutableMap.of(
                    ((MissingMatrixVariableException) e).getVariableName(), e.getMessage()
            ));
        } else if (e instanceof MissingServletRequestParameterException) {
            return ParameterInvalidException.result(ImmutableMap.of(
                    ((MissingServletRequestParameterException) e).getParameterName(), e.getMessage()
            ));
        } else if (e instanceof BindException) {
            return ParameterInvalidException.result(this.format(((BindException) e).getBindingResult()));
        } else if (e instanceof ConstraintViolationException) {
            return ParameterInvalidException.result(this.format((ConstraintViolationException) e));
        } else if (e instanceof MethodArgumentNotValidException) {
            return ParameterInvalidException.result(
                    this.format(((MethodArgumentNotValidException) e).getBindingResult())
            );
        } else if (e instanceof MethodArgumentTypeMismatchException) {
            return ParameterInvalidException.result(this.format((MethodArgumentTypeMismatchException) e));
        } else if (e instanceof MissingServletRequestPartException) {
            return ParameterInvalidException.result(ImmutableMap.of(
                    ((MissingServletRequestPartException) e).getRequestPartName(), e.getMessage()
            ));
        } else if (e instanceof ValidationException || e instanceof TypeMismatchException
                || e instanceof MultipartException || e instanceof HttpMessageConversionException
                || e instanceof ServletRequestBindingException || e instanceof HttpRequestMethodNotSupportedException
                || e instanceof HttpMediaTypeNotAcceptableException
                || e instanceof HttpMediaTypeNotSupportedException) {
            return Result.of(HttpStatus.BAD_REQUEST.name(), null, HttpStatus.BAD_REQUEST.getReasonPhrase());
        }

        // 未知异常处理，如果是服务器内部错误则屏蔽详细的异常信息
        String code = Result.ERROR;
        Object data = null;
        if (e instanceof FrameworkException) {
            code = ((FrameworkException) e).getCode();
            data = ((FrameworkException) e).getData();
        }
        return Objects.equals(code, Result.ERROR) ? Result.failure() : Result.of(code, data, e.getMessage());
    }

    /**
     * 异常方法
     *
     * @param request  Http请求对象
     * @param response Http响应对象
     * @return 异常结果
     */
    @RequestMapping("${server.error.path:${error.path:/error}}")
    public Result<?> error(HttpServletRequest request, HttpServletResponse response) {
        HttpStatus status = this.getStatus(request);
        Map<String, Object> attributes = this.getErrorAttributes(request, this.options);
        String uri = (String) attributes.get("path");
        String method = request.getMethod();
        String message = (String) attributes.get("message");
        log.warn(LoggingContextHolder.message(uri, method, message));
        Result<?> result = status.is5xxServerError() ? Result.failure() :
                Result.of(status.name(), null, status.getReasonPhrase());
        response.setStatus(SessionContextHolder.result2status(result).value());
        return result;
    }

    /**
     * 异常转换
     *
     * @param request  Http请求对象
     * @param response Http响应对象
     * @param e        异常对象
     * @return 异常结果
     */
    @ExceptionHandler(Exception.class)
    public Result<?> exception(HttpServletRequest request, HttpServletResponse response, Exception e) {
        Result<?> result = this.error2result(ErrorUtils.real(e));
        if (Objects.equals(result.getCode(), Result.ERROR)) {
            LoggingContextHolder.error(log, request, e);
        } else {
            LoggingContextHolder.warn(log, request, e);
        }
        response.setStatus(SessionContextHolder.result2status(result).value());
        return result;
    }
}