package com.chelvc.framework.base.interceptor;

import com.chelvc.framework.base.annotation.ResponseWrapping;
import lombok.NonNull;
import org.springframework.core.MethodParameter;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.method.support.ModelAndViewContainer;
import org.springframework.web.servlet.mvc.method.annotation.RequestResponseBodyMethodProcessor;

/**
 * 响应结果包装拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class ResponseWrappingInterceptor implements HandlerMethodReturnValueHandler {
    private final ResponseWrapper wrapper;
    private final RequestResponseBodyMethodProcessor processor;

    public ResponseWrappingInterceptor(@NonNull ResponseWrapper wrapper,
                                       @NonNull RequestResponseBodyMethodProcessor processor) {
        this.wrapper = wrapper;
        this.processor = processor;
    }

    /**
     * 判断目标方法是否使用@ResponseBody注解
     *
     * @param method 目标方法参数对象
     * @return true/false
     */
    private boolean isResponseBodyMethod(MethodParameter method) {
        return method.hasMethodAnnotation(ResponseBody.class)
                || method.getDeclaringClass().isAnnotationPresent(ResponseBody.class)
                || method.getDeclaringClass().isAnnotationPresent(RestController.class);
    }

    /**
     * 判断目标方法是否使用了@ResponseWrapping注解
     *
     * @param method 目标方法参数对象
     * @return true/false
     */
    private boolean isResponseWrappingMethod(MethodParameter method) {
        ResponseWrapping annotation = method.getMethodAnnotation(ResponseWrapping.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(ResponseWrapping.class);
        }
        return annotation != null && annotation.enabled();
    }

    @Override
    public boolean supportsReturnType(MethodParameter method) {
        return this.isResponseBodyMethod(method) && this.isResponseWrappingMethod(method);
    }

    @Override
    public void handleReturnValue(Object value, MethodParameter method, ModelAndViewContainer container,
                                  NativeWebRequest request) throws Exception {
        value = this.wrapper.wrap(request, method, value);
        this.processor.handleReturnValue(value, method, container, request);
    }
}
