package com.chelvc.framework.base.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.interceptor.BufferedRequestWrapper;
import com.chelvc.framework.common.util.HostUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.springframework.http.MediaType;

/**
 * Http工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class HttpUtils {
    /**
     * 资源分隔符匹配模式
     */
    private static final Pattern URI_SEPARATOR_PATTERN = Pattern.compile("/+");

    /**
     * 内容类型字符集匹配模式
     */
    private static final Pattern CONTENT_TYPE_CHARSET_PATTERN = Pattern.compile("(?<=charset=)([\\w\\-]+)");

    /**
     * 客户端主机地址请求头数组
     */
    private static final String[] REQUEST_ADDRESS_HEADER_ARRAY = new String[]{
            "X-Real-IP", "X-Forwarded-For", "Proxy-Client-IP", "WL-Proxy-Client-IP"
    };

    private HttpUtils() {
    }

    /**
     * 构建资源地址
     *
     * @param original 原始地址
     * @return 资源地址
     */
    public static String uri(String original) {
        if (StringUtils.isEmpty(original)) {
            return StringUtils.URI_SEPARATOR;
        } else if (original.charAt(0) != StringUtils.URI_SEPARATOR.charAt(0)) {
            return StringUtils.URI_SEPARATOR + original;
        }
        return original;
    }

    /**
     * 构建资源地址
     *
     * @param prefix 地址前缀
     * @param suffix 地址后缀
     * @return 资源地址
     */
    public static String uri(String prefix, String suffix) {
        if (StringUtils.isEmpty(prefix) && StringUtils.isEmpty(suffix)) {
            return StringUtils.URI_SEPARATOR;
        } else if (StringUtils.isEmpty(suffix)) {
            return uri(prefix);
        } else if (StringUtils.isEmpty(prefix)) {
            return uri(suffix);
        }
        char separator = StringUtils.URI_SEPARATOR.charAt(0);
        if (prefix.charAt(prefix.length() - 1) != separator && suffix.charAt(0) != separator) {
            return uri(prefix + StringUtils.URI_SEPARATOR + suffix);
        } else if (prefix.charAt(prefix.length() - 1) == separator && suffix.charAt(0) == separator) {
            return uri(prefix + suffix.substring(1));
        }
        return uri(prefix + suffix);
    }

    /**
     * 构建资源地址列表
     *
     * @param originals 原始地址集合
     * @return 资源地址列表
     */
    public static List<String> uris(Collection<String> originals) {
        if (ObjectUtils.isEmpty(originals)) {
            return Collections.emptyList();
        }
        return originals.stream().map(HttpUtils::uri).distinct().collect(Collectors.toList());
    }

    /**
     * 构建资源地址列表
     *
     * @param prefixes 地址前缀集合
     * @param suffixes 地址后缀集合
     * @return 资源地址列表
     */
    public static List<String> uris(Collection<String> prefixes, Collection<String> suffixes) {
        if (ObjectUtils.isEmpty(prefixes) && ObjectUtils.isEmpty(suffixes)) {
            return Collections.emptyList();
        } else if (ObjectUtils.isEmpty(suffixes)) {
            return prefixes.stream().map(HttpUtils::uri).distinct().collect(Collectors.toList());
        } else if (ObjectUtils.isEmpty(prefixes)) {
            return suffixes.stream().map(HttpUtils::uri).distinct().collect(Collectors.toList());
        }
        return prefixes.stream().flatMap(prefix -> suffixes.stream().map(suffix -> uri(prefix, suffix)))
                .distinct().collect(Collectors.toList());
    }

    /**
     * 获取请求域名
     *
     * @param request Http请求对象
     * @return 域名
     */
    public static String getDomain(@NonNull HttpServletRequest request) {
        return request.getScheme() + "://" + request.getServerName() +
                ":" + request.getServerPort() + request.getContextPath();
    }

    /**
     * 获取请求客户端主机地址
     *
     * @param request Http请求对象
     * @return 请求客户端主机地址
     */
    public static String getHost(@NonNull HttpServletRequest request) {
        // 获取请求主机地址
        String ip = null;
        for (String header : REQUEST_ADDRESS_HEADER_ARRAY) {
            if (StringUtils.notEmpty(ip = request.getHeader(header)) && !HostUtils.UNKNOWN_ADDRESS.equals(ip)) {
                break;
            }
        }
        if (StringUtils.isEmpty(ip)) {
            ip = request.getRemoteAddr();
        }

        // 如果地址存在多个则获取第一个
        int separatorIndex = ip.indexOf(',');
        if (separatorIndex > -1) {
            ip = ip.substring(0, separatorIndex);
        }

        return HostUtils.DEFAULT_LOCAL_ADDRESS_IPV6.equals(ip) ? HostUtils.LOCAL_ADDRESS : ip;
    }

    /**
     * 获取Body请求参数
     *
     * @param request Http请求对象
     * @return 请求体字节数组
     * @throws IOException I/O异常
     */
    public static byte[] getBody(@NonNull HttpServletRequest request) throws IOException {
        try (InputStream input = request.getInputStream()) {
            return getBody(input);
        }
    }

    /**
     * 获取Body请求参数
     *
     * @param input 输入流
     * @return 请求体字节数组
     * @throws IOException I/O异常
     */
    public static byte[] getBody(@NonNull InputStream input) throws IOException {
        // 如果请求体已缓存则直接获取
        if (input instanceof BufferedRequestWrapper.BufferedServletInputStream) {
            return ((BufferedRequestWrapper.BufferedServletInputStream) input).getBody();
        }

        // 从请求输入流中读取请求体
        int n;
        byte[] buffer = new byte[1024];
        ByteArrayOutputStream output = new ByteArrayOutputStream(Math.max(32, input.available()));
        while ((n = input.read(buffer)) != -1) {
            output.write(buffer, 0, n);
        }
        return output.toByteArray();
    }

    /**
     * 获取表单请求参数
     *
     * @param request Http请求对象
     * @return 参数键/值映射表
     */
    public static Map<String, String[]> getParameters(@NonNull HttpServletRequest request) {
        // 获取表单参数
        Map<String, String[]> parameters = Maps.newHashMap();
        Enumeration<String> enumeration = request.getParameterNames();
        while (enumeration.hasMoreElements()) {
            String name = enumeration.nextElement();
            if (StringUtils.notEmpty(name)) {
                parameters.put(name, request.getParameterValues(name));
            }
        }
        return parameters;
    }

    /**
     * 响应文件
     *
     * @param response Http响应对象
     * @param file     文件对象
     * @throws IOException I/O异常
     */
    public static void write(@NonNull HttpServletResponse response, @NonNull File file) throws IOException {
        write(response, file, file.getName());
    }

    /**
     * 响应文件
     *
     * @param response Http响应对象
     * @param file     文件对象
     * @param filename 文件名称
     * @throws IOException I/O异常
     */
    public static void write(@NonNull HttpServletResponse response, @NonNull File file, @NonNull String filename)
            throws IOException {
        // 设置响应头
        response.reset();
        response.addHeader("Content-Length", String.valueOf(file.length()));
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename, "utf-8"));
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.addHeader("Access-Control-Allow-Origin", "*");
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);

        // 将文件写入响应流
        int length;
        byte[] buffer = new byte[2048];
        try (InputStream input = new FileInputStream(file); OutputStream output = response.getOutputStream()) {
            while ((length = input.read(buffer)) > 0) {
                output.write(buffer, 0, length);
            }
            output.flush();
        }
    }

    /**
     * 对象URL编码
     *
     * @param original 原始信息
     * @return 对象编码结果
     */
    public static String encode(String original) {
        if (StringUtils.isEmpty(original)) {
            return StringUtils.EMPTY;
        }
        try {
            return URLEncoder.encode(original, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象URL解码
     *
     * @param encode 编码信息
     * @return 对象编码结果
     */
    public static String decode(String encode) {
        if (StringUtils.isEmpty(encode)) {
            return encode;
        }
        try {
            return URLDecoder.decode(encode, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * URL地址拼接
     *
     * @param domain 域名
     * @param uris   资源地址数组
     * @return URL地址
     */
    public static String url(@NonNull String domain, @NonNull String... uris) {
        if (uris == null || uris.length == 0) {
            return domain;
        }
        Matcher matcher = URI_SEPARATOR_PATTERN.matcher(StringUtils.join(uris, StringUtils.URI_SEPARATOR));
        String uri = matcher.replaceAll(StringUtils.URI_SEPARATOR);
        if (domain.charAt(domain.length() - 1) == StringUtils.URI_SEPARATOR.charAt(0)) {
            if (uri.charAt(0) == StringUtils.URI_SEPARATOR.charAt(0)) {
                return domain + uri.substring(1);
            }
            return domain + uri;
        } else if (uri.charAt(0) == StringUtils.URI_SEPARATOR.charAt(0)) {
            return domain + uri;
        }
        return domain + StringUtils.URI_SEPARATOR + uri;
    }

    /**
     * Http请求参数序列化
     *
     * @param key    参数名称
     * @param values 参数值
     * @return 参数序列化结果
     */
    public static String serialize(@NonNull String key, String[] values) {
        if (values == null || values.length == 0) {
            return key + StringUtils.EQUAL;
        } else if (values.length == 1) {
            return key + StringUtils.EQUAL + StringUtils.ifEmpty(values[0], StringUtils.EMPTY);
        }
        StringBuilder buffer = new StringBuilder();
        for (String value : values) {
            if (StringUtils.notEmpty(value)) {
                if (buffer.length() > 0) {
                    buffer.append("&");
                }
                buffer.append(key).append(StringUtils.EQUAL).append(value);
            }
        }
        return buffer.length() == 0 ? (key + StringUtils.EQUAL) : buffer.toString();
    }

    /**
     * Http请求参数序列化
     *
     * @param request Http请求对象
     * @return 请求参数
     * @throws IOException I/O异常
     */
    public static String serialize(@NonNull HttpServletRequest request) throws IOException {
        return serialize(request, false);
    }

    /**
     * Http请求参数序列化
     *
     * @param request  Http请求对象
     * @param ordering 参数是否排序
     * @return 请求参数
     * @throws IOException I/O异常
     */
    public static String serialize(@NonNull HttpServletRequest request, boolean ordering) throws IOException {
        String type = request.getContentType();
        if (StringUtils.notEmpty(type) && type.contains(MediaType.APPLICATION_JSON_VALUE)) {
            // Body参数
            return new String(getBody(request));
        }

        // 表单参数 + URL参数
        Map<String, String[]> parameters = getParameters(request);
        if (ordering) {
            return StringUtils.join(parameters, "&", HttpUtils::serialize, Comparator.naturalOrder());
        }
        return StringUtils.join(parameters, "&", (BiFunction<String, String[], String>) HttpUtils::serialize);
    }

    /**
     * 获取内容类型字符集
     *
     * @param type Content-Type
     * @return 字符集
     */
    public static Charset getContentTypeCharset(String type) {
        if (StringUtils.isEmpty(type)) {
            return StandardCharsets.UTF_8;
        }
        Matcher matcher = CONTENT_TYPE_CHARSET_PATTERN.matcher(type);
        return matcher.find() ? Charset.forName(matcher.group(1)) : StandardCharsets.UTF_8;
    }
}
