package com.chelvc.framework.base.util;

import java.io.File;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.util.StringUtils;
import lombok.NonNull;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.core.type.classreading.CachingMetadataReaderFactory;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.MetadataReaderFactory;
import org.springframework.stereotype.Controller;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.ClassUtils;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Spring工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class SpringUtils {
    /**
     * 资源路径匹配器
     */
    private static final AntPathMatcher PATH_MATCHER = new AntPathMatcher(File.separator);

    private SpringUtils() {
    }

    /**
     * 判断资源路径是否符合指定匹配模式
     *
     * @param path     资源路径
     * @param patterns 匹配模式数组
     * @return true/false
     */
    public static boolean isPath(String path, String... patterns) {
        if (StringUtils.isEmpty(path) || patterns == null || patterns.length == 0) {
            return false;
        }
        for (String pattern : patterns) {
            if (StringUtils.notEmpty(pattern) && PATH_MATCHER.match(pattern, path)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取接口地址
     *
     * @param mapping 请求映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull RequestMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取POST接口地址
     *
     * @param mapping POST方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull PostMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取GET接口地址
     *
     * @param mapping GET方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull GetMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取PUT接口地址
     *
     * @param mapping PUT方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull PutMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取DELETE接口地址
     *
     * @param mapping DELETE方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull DeleteMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取PATCH接口地址
     *
     * @param mapping PATCH方法映射注解实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull PatchMapping mapping) {
        String api;
        if (!(api = mapping.name().trim()).isEmpty()) {
            return Collections.singletonList(api);
        } else if (mapping.value().length > 0) {
            return Stream.of(mapping.value()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        } else if (mapping.path().length > 0) {
            return Stream.of(mapping.path()).map(String::trim).filter(StringUtils::notEmpty)
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 获取接口地址
     *
     * @param clazz 接口对象
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull Class<?> clazz) {
        String mapping;
        Annotation annotation;
        if ((annotation = clazz.getAnnotation(Controller.class)) != null
                && !(mapping = ((Controller) annotation).value().trim()).isEmpty()) {
            return Collections.singletonList(mapping);
        } else if ((annotation = clazz.getAnnotation(RestController.class)) != null
                && !(mapping = ((RestController) annotation).value().trim()).isEmpty()) {
            return Collections.singletonList(mapping);
        } else if ((annotation = clazz.getAnnotation(RequestMapping.class)) != null) {
            return getApis((RequestMapping) annotation);
        }
        return Collections.emptyList();
    }

    /**
     * 获取接口地址
     *
     * @param method 接口方法对象实例
     * @return 接口地址列表
     */
    public static List<String> getApis(@NonNull Method method) {
        Annotation annotation;
        List<String> prefixes = getApis(method.getDeclaringClass());
        if ((annotation = method.getAnnotation(RequestMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((RequestMapping) annotation));
        } else if ((annotation = method.getAnnotation(PostMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((PostMapping) annotation));
        } else if ((annotation = method.getAnnotation(GetMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((GetMapping) annotation));
        } else if ((annotation = method.getAnnotation(PutMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((PutMapping) annotation));
        } else if ((annotation = method.getAnnotation(DeleteMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((DeleteMapping) annotation));
        } else if ((annotation = method.getAnnotation(PatchMapping.class)) != null) {
            return HttpUtils.uris(prefixes, getApis((PatchMapping) annotation));
        }
        return Collections.emptyList();
    }

    /**
     * 查找工程内部的对象类型
     *
     * @param pack 包名
     * @return 类对象列表
     */
    public static List<Class<?>> lookupClasses(@NonNull String pack) {
        return lookupClasses(pack, clazz -> true);
    }

    /**
     * 查找工程内部的对象类型
     *
     * @param pack      包名
     * @param condition 类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> lookupClasses(@NonNull String pack, @NonNull Predicate<Class<?>> condition) {
        ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();
        String pattern = (pack = pack.trim()).isEmpty() ? "**/*.class" : (
                ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX
                        + ClassUtils.convertClassNameToResourcePath(pack)
                        + "/**/*.class"
        );

        // 获取对象资源文件
        Resource[] resources;
        try {
            resources = resourcePatternResolver.getResources(pattern);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 将资源文件转换为类对象
        Stream.Builder<Class<?>> classes = Stream.builder();
        MetadataReaderFactory readerFactory = new CachingMetadataReaderFactory(resourcePatternResolver);
        ClassLoader classLoader = Objects.requireNonNull(ClassUtils.getDefaultClassLoader());
        for (Resource resource : resources) {
            try {
                // 加载类对象
                MetadataReader reader = readerFactory.getMetadataReader(resource);
                Class<?> clazz = classLoader.loadClass(reader.getClassMetadata().getClassName());
                if (clazz != null && condition.test(clazz)) {
                    classes.add(clazz);
                }
            } catch (IOException | ClassNotFoundException e) {
                throw new RuntimeException();
            }
        }
        return classes.build().distinct().collect(Collectors.toList());
    }

    /**
     * 查找接口对象实例
     *
     * @param applicationContext 应用上下文
     * @return 接口对象实例列表
     */
    public static List<Object> lookupControllers(@NonNull ApplicationContext applicationContext) {
        return Stream.of(applicationContext.getBeansWithAnnotation(Controller.class).values(),
                applicationContext.getBeansWithAnnotation(RestController.class).values())
                .flatMap(Collection::stream).distinct().collect(Collectors.toList());
    }
}
