package com.chelvc.framework.common.model;

import java.io.Serializable;
import java.util.Objects;

import com.chelvc.framework.common.exception.FrameworkException;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NonNull;
import lombok.ToString;

/**
 * 请求结果对象
 *
 * @author Woody
 * @date 2024/1/30
 */
@Getter
@ToString
@EqualsAndHashCode
public class Result<T> implements Serializable {
    /**
     * 成功编码
     */
    public static final String OK = "OK";

    /**
     * 错误编码
     */
    public static final String ERROR = "ERROR";

    /**
     * 结果数据
     */
    private T data;

    /**
     * 状态码
     */
    private String code;

    /**
     * 结果信息
     */
    private String message;

    /**
     * 时间戳
     */
    private final Long timestamp = System.currentTimeMillis();

    /**
     * 构建成功结果
     *
     * @return 结果对象
     */
    public static <M> Result<M> success() {
        return success(null);
    }

    /**
     * 构建成功结果
     *
     * @param data 结果数据
     * @return 结果对象
     */
    public static <M> Result<M> success(M data) {
        return success(data, "Success");
    }

    /**
     * 构建成功结果
     *
     * @param data    结果数据
     * @param message 结果信息
     * @return 结果对象
     */
    public static <M> Result<M> success(M data, String message) {
        return of(OK, data, message);
    }

    /**
     * 构建异常结果
     *
     * @return 结果对象
     */
    public static <M> Result<M> failure() {
        return failure("Failure");
    }

    /**
     * 构建异常结果
     *
     * @param message 异常消息
     * @return 结果对象
     */
    public static <M> Result<M> failure(String message) {
        return of(ERROR, null, message);
    }

    /**
     * 构建异常结果
     * param data    结果数据
     *
     * @param message 异常消息
     * @return 结果对象
     */
    public static <M> Result<M> failure(M data, String message) {
        return of(ERROR, data, message);
    }

    /**
     * 构建请求结果
     *
     * @param e 异常对象实例
     * @return 结果对象
     */
    public static Result<?> of(@NonNull FrameworkException e) {
        return of(e.getCode(), e.getData(), e.getMessage());
    }

    /**
     * 构建请求结果
     *
     * @param code 状态码
     * @param data 结果数据
     * @return 结果对象
     */
    public static <M> Result<M> of(@NonNull String code, M data) {
        return of(code, data, null);
    }

    /**
     * 构建请求结果
     *
     * @param code    状态码
     * @param data    结果数据
     * @param message 结果信息
     * @return 结果对象
     */
    public static <M> Result<M> of(@NonNull String code, M data, String message) {
        Result<M> result = new Result<>();
        result.code = code;
        result.data = data;
        result.message = message;
        return result;
    }

    /**
     * 是否成功
     *
     * @return true/false
     */
    @JsonIgnore
    public boolean isSuccess() {
        return Objects.equals(this.code, OK);
    }

    /**
     * 是否失败
     *
     * @return true/false
     */
    @JsonIgnore
    public boolean isFailure() {
        return Objects.equals(this.code, ERROR);
    }
}
