package com.chelvc.framework.common.util;

import java.security.AlgorithmParameters;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import lombok.NonNull;
import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

/**
 * 数据加解密处理工具类
 *
 * @author woody
 * @date 2024/1/30
 */
public final class AESUtils {
    /**
     * 密钥长度
     */
    public static final int KEY_SIZE = 128;

    /**
     * AES算法名称
     */
    public static final String ALGORITHM = "AES";

    /**
     * CBC密码处理器算法
     */
    public static final String CBC_CIPHER_ALGORITHM = "AES/CBC/PKCS5Padding";

    /**
     * ECB密码处理器算法
     */
    public static final String ECB_CIPHER_ALGORITHM = "AES/ECB/PKCS5Padding";

    static {
        Security.addProvider(new BouncyCastleProvider());
        try {
            KeyGenerator.getInstance(ALGORITHM).init(KEY_SIZE);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    private AESUtils() {
    }

    /**
     * 修正密钥
     *
     * @param key 密钥标识
     * @return 修正后的密钥
     */
    public static String revise(String key) {
        if (key == null) {
            return null;
        }

        // 如果小于16位则左边补0到16位
        if (key.length() < 16) {
            return StringUtils.rjust(key, 16, '0');
        }

        // 如果长度不是8的倍数则左边补0到8的倍数
        int offset = 8 - key.length() % 8;
        return offset == 8 ? key : StringUtils.rjust(key, key.length() + offset, '0');
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull String secret) {
        return getCipher(mode, revise(secret).getBytes());
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull byte[] secret) {
        try {
            Cipher cipher = Cipher.getInstance(ECB_CIPHER_ALGORITHM);
            cipher.init(mode, new SecretKeySpec(secret, ALGORITHM));
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull String secret, @NonNull String iv) {
        return getCipher(mode, revise(secret).getBytes(), revise(iv).getBytes());
    }

    /**
     * 获取密码处理器
     *
     * @param mode   加解密模式
     * @param secret 密钥
     * @param iv     向量
     * @return 密码处理器实例
     */
    public static Cipher getCipher(int mode, @NonNull byte[] secret, @NonNull byte[] iv) {
        try {
            Cipher cipher = Cipher.getInstance(CBC_CIPHER_ALGORITHM);
            Key key = new SecretKeySpec(secret, ALGORITHM);
            AlgorithmParameters parameters = AlgorithmParameters.getInstance(ALGORITHM);
            parameters.init(new IvParameterSpec(iv));
            cipher.init(mode, key, parameters);
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * AES加解密
     *
     * @param cipher 密码处理器实例
     * @param bytes  字节数组
     * @return 明文/密文
     */
    public static byte[] codec(@NonNull Cipher cipher, byte[] bytes) {
        if (bytes == null) {
            return null;
        }
        try {
            return cipher.doFinal(bytes);
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * AES加密
     *
     * @param cipher    密码处理器实例
     * @param plaintext 明文
     * @return 密文（base64）
     */
    public static String encode(@NonNull Cipher cipher, String plaintext) {
        return plaintext == null ? null : Base64.encodeBase64String(codec(cipher, plaintext.getBytes()));
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @return 密文
     */
    public static byte[] encode(byte[] plaintext, @NonNull String secret) {
        return plaintext == null ? null : codec(getCipher(Cipher.ENCRYPT_MODE, secret), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @return 密文（base64）
     */
    public static String encode(String plaintext, @NonNull String secret) {
        return plaintext == null ? null : encode(getCipher(Cipher.ENCRYPT_MODE, secret), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @return 密文
     */
    public static byte[] encode(byte[] plaintext, @NonNull byte[] secret) {
        return plaintext == null ? null : codec(getCipher(Cipher.ENCRYPT_MODE, secret), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @return 密文（base64）
     */
    public static String encode(String plaintext, @NonNull byte[] secret) {
        return plaintext == null ? null : encode(getCipher(Cipher.ENCRYPT_MODE, secret), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @param iv        向量
     * @return 密文
     */
    public static byte[] encode(byte[] plaintext, @NonNull String secret, @NonNull String iv) {
        return plaintext == null ? null : codec(getCipher(Cipher.ENCRYPT_MODE, secret, iv), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @param iv        向量
     * @return 密文（base64）
     */
    public static String encode(String plaintext, @NonNull String secret, @NonNull String iv) {
        return plaintext == null ? null : encode(getCipher(Cipher.ENCRYPT_MODE, secret, iv), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @param iv        向量
     * @return 密文
     */
    public static byte[] encode(byte[] plaintext, @NonNull byte[] secret, @NonNull byte[] iv) {
        return plaintext == null ? null : codec(getCipher(Cipher.ENCRYPT_MODE, secret, iv), plaintext);
    }

    /**
     * AES加密
     *
     * @param plaintext 明文
     * @param secret    密钥
     * @param iv        向量
     * @return 密文（base64）
     */
    public static String encode(String plaintext, @NonNull byte[] secret, @NonNull byte[] iv) {
        return plaintext == null ? null : encode(getCipher(Cipher.ENCRYPT_MODE, secret, iv), plaintext);
    }

    /**
     * AES解密
     *
     * @param cipher     密码处理器实例
     * @param ciphertext 密文（base64）
     * @return 明文
     */
    public static String decode(@NonNull Cipher cipher, String ciphertext) {
        return ciphertext == null ? null : new String(codec(cipher, Base64.decodeBase64(ciphertext)));
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文
     * @param secret     密钥
     * @return 明文
     */
    public static byte[] decode(byte[] ciphertext, @NonNull String secret) {
        return ciphertext == null ? null : codec(getCipher(Cipher.DECRYPT_MODE, secret), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文（base64）
     * @param secret     密钥
     * @return 明文
     */
    public static String decode(String ciphertext, @NonNull String secret) {
        return ciphertext == null ? null : decode(getCipher(Cipher.DECRYPT_MODE, secret), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文
     * @param secret     密钥
     * @return 明文
     */
    public static byte[] decode(byte[] ciphertext, @NonNull byte[] secret) {
        return ciphertext == null ? null : codec(getCipher(Cipher.DECRYPT_MODE, secret), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文（base64）
     * @param secret     密钥
     * @return 明文
     */
    public static String decode(String ciphertext, @NonNull byte[] secret) {
        return ciphertext == null ? null : decode(getCipher(Cipher.DECRYPT_MODE, secret), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文
     * @param secret     密钥
     * @param iv         向量
     * @return 明文
     */
    public static byte[] decode(byte[] ciphertext, @NonNull String secret, @NonNull String iv) {
        return ciphertext == null ? null : codec(getCipher(Cipher.DECRYPT_MODE, secret, iv), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文（base64）
     * @param secret     密钥
     * @param iv         向量
     * @return 明文
     */
    public static String decode(String ciphertext, @NonNull String secret, @NonNull String iv) {
        return ciphertext == null ? null : decode(getCipher(Cipher.DECRYPT_MODE, secret, iv), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文
     * @param secret     密钥
     * @param iv         向量
     * @return 明文
     */
    public static byte[] decode(byte[] ciphertext, @NonNull byte[] secret, @NonNull byte[] iv) {
        return ciphertext == null ? null : codec(getCipher(Cipher.DECRYPT_MODE, secret, iv), ciphertext);
    }

    /**
     * AES解密
     *
     * @param ciphertext 密文（base64）
     * @param secret     密钥
     * @param iv         向量
     * @return 明文
     */
    public static String decode(String ciphertext, @NonNull byte[] secret, @NonNull byte[] iv) {
        return ciphertext == null ? null : decode(getCipher(Cipher.DECRYPT_MODE, secret, iv), ciphertext);
    }
}
