package com.chelvc.framework.common.util;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Region;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.BeanDescription;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationConfig;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.BeanPropertyWriter;
import com.fasterxml.jackson.databind.ser.BeanSerializerModifier;
import com.fasterxml.jackson.databind.ser.SerializerFactory;
import lombok.NonNull;

/**
 * Jackson工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class JacksonUtils {
    /**
     * JavaScript最大整型数（2的53次方）
     */
    private static final BigInteger JAVA_SCRIPT_MAX_INTEGER_VALUE = BigInteger.valueOf(2).pow(53);

    /**
     * JavaScript最大小数（2的53次方）
     */
    private static final BigDecimal JAVA_SCRIPT_MAX_DECIMAL_VALUE = BigDecimal.valueOf(2).pow(53);

    /**
     * Jackson处理基础模块
     */
    private static SimpleModule BASE_MODULE;

    /**
     * Jackson处理安全模块
     */
    private static SimpleModule SAFETY_MODULE;

    /**
     * Jackson null对象序列化处理器更新处理实现
     */
    private static BeanSerializerModifier NULL_MODIFIER;

    /**
     * 默认对象序列化处理器实例
     */
    private static final ObjectMapper DEFAULT_SERIALIZER = initializeSerializer();

    /**
     * 安全的对象序列化处理器实例
     */
    private static final ObjectMapper SAFELY_SERIALIZER = initializeSerializer(true);

    private JacksonUtils() {
    }

    /**
     * 构建序列化处理器基础模块
     *
     * @return 模块实例
     */
    private static SimpleModule generateBaseModule() {
        SimpleModule module = new SimpleModule();

        // 设置本地日期序列化处理器
        module.addSerializer(new JsonSerializer<LocalDate>() {
            @Override
            public Class<LocalDate> handledType() {
                return LocalDate.class;
            }

            @Override
            public void serialize(LocalDate date, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeObject(DateUtils.convert(date));
            }
        });

        // 设置本地日期时间处理器
        module.addSerializer(new JsonSerializer<LocalDateTime>() {
            @Override
            public Class<LocalDateTime> handledType() {
                return LocalDateTime.class;
            }

            @Override
            public void serialize(LocalDateTime datetime, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeObject(DateUtils.convert(datetime));
            }
        });

        // 设置时间周期序列化处理器
        module.addSerializer(new JsonSerializer<Period>() {
            @Override
            public Class<Period> handledType() {
                return Period.class;
            }

            @Override
            public void serialize(Period period, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                String begin = ObjectUtils.ifNull(period.getBegin(), DateUtils::format, () -> StringUtils.EMPTY);
                String end = ObjectUtils.ifNull(period.getEnd(), DateUtils::format, () -> StringUtils.EMPTY);
                generator.writeString(begin + "," + end);
            }
        });

        // 设置数据分页序列化处理器
        module.addSerializer(new JsonSerializer<Paging>() {
            @Override
            public Class<Paging> handledType() {
                return Paging.class;
            }

            @Override
            public void serialize(Paging paging, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeString(paging.getNumber() + "," + paging.getSize());
            }
        });

        // 设置地区序列化处理器
        module.addSerializer(new JsonSerializer<Region>() {
            @Override
            public Class<Region> handledType() {
                return Region.class;
            }

            @Override
            public void serialize(Region region, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeNumber(region.getCode());
            }
        });

        // 设置时间周期序列化处理器
        module.addSerializer(new JsonSerializer<Duration>() {
            @Override
            public Class<Duration> handledType() {
                return Duration.class;
            }

            @Override
            public void serialize(Duration duration, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeString(duration.toString());
            }
        });

        // 设置时间反序列化处理器
        module.addDeserializer(Date.class, new JsonDeserializer<Date>() {
            @Override
            public Class<?> handledType() {
                return Date.class;
            }

            @Override
            public Date deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return DateUtils.parse(parser.getValueAsString());
            }
        });

        // 设置时间周期反序列化处理器
        module.addDeserializer(Period.class, new JsonDeserializer<Period>() {
            @Override
            public Class<?> handledType() {
                return Period.class;
            }

            @Override
            public Period deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Period.parse(parser.getValueAsString());
            }
        });

        // 设置数据分页反序列化处理器
        module.addDeserializer(Paging.class, new JsonDeserializer<Paging>() {
            @Override
            public Class<?> handledType() {
                return Paging.class;
            }

            @Override
            public Paging deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Paging.parse(parser.getValueAsString());
            }
        });

        // 设置地区反序列化处理器
        module.addDeserializer(Region.class, new JsonDeserializer<Region>() {
            @Override
            public Class<?> handledType() {
                return Region.class;
            }

            @Override
            public Region deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Region.parse(parser.getValueAsString());
            }
        });

        // 设置时间周期反序列化处理器
        module.addDeserializer(Duration.class, new JsonDeserializer<Duration>() {
            @Override
            public Class<?> handledType() {
                return Duration.class;
            }

            @Override
            public Duration deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Duration.parse(parser.getValueAsString());
            }
        });
        return module;
    }

    /**
     * 构建序列化处理器安全模块
     *
     * @return 模块实例
     */
    private static SimpleModule generateSafetyModule() {
        SimpleModule module = new SimpleModule();

        // 设置整数序列化处理器
        module.addSerializer(new JsonSerializer<Long>() {
            @Override
            public Class<Long> handledType() {
                return Long.class;
            }

            @Override
            public void serialize(Long number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toString());
                }
            }
        });

        // 设置浮点数序列化处理器
        module.addSerializer(new JsonSerializer<Double>() {
            @Override
            public Class<Double> handledType() {
                return Double.class;
            }

            @Override
            public void serialize(Double number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toString());
                }
            }
        });

        // 设置大整数序列化处理器
        module.addSerializer(new JsonSerializer<BigInteger>() {
            @Override
            public Class<BigInteger> handledType() {
                return BigInteger.class;
            }

            @Override
            public void serialize(BigInteger number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toString());
                }
            }
        });

        // 设置大浮点数序列化处理器
        module.addSerializer(new JsonSerializer<BigDecimal>() {
            @Override
            public Class<BigDecimal> handledType() {
                return BigDecimal.class;
            }

            @Override
            public void serialize(BigDecimal number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toPlainString());
                }
            }
        });
        return module;
    }

    /**
     * 构建null对象序列化处理器更新对象
     *
     * @return null对象序列化处理更新实例
     */
    private static BeanSerializerModifier generateNullModifier() {
        return new BeanSerializerModifier() {
            /**
             * 空字典序列化处理器
             */
            private final JsonSerializer<Object> mappingSerializer = new JsonSerializer<Object>() {
                @Override
                public void serialize(Object value, JsonGenerator generator, SerializerProvider provider)
                        throws IOException {
                    if (value == null) {
                        generator.writeStartObject();
                        generator.writeEndObject();
                    } else {
                        generator.writeObject(value);
                    }
                }
            };

            /**
             * 空集合序列化处理器
             */
            private final JsonSerializer<Object> collectionSerializer = new JsonSerializer<Object>() {
                @Override
                public void serialize(Object value, JsonGenerator generator, SerializerProvider provider)
                        throws IOException {
                    if (value == null) {
                        generator.writeStartArray();
                        generator.writeEndArray();
                    } else {
                        generator.writeObject(value);
                    }
                }
            };

            @Override
            public List<BeanPropertyWriter> changeProperties(SerializationConfig config, BeanDescription bean,
                                                             List<BeanPropertyWriter> writers) {
                writers.forEach(writer -> {
                    Class<?> clazz = writer.getType().getRawClass();
                    if (Map.class.isAssignableFrom(clazz)) {
                        writer.assignNullSerializer(this.mappingSerializer);
                    } else if (clazz.isArray() || Collection.class.isAssignableFrom(clazz)) {
                        writer.assignNullSerializer(this.collectionSerializer);
                    }
                });
                return writers;
            }
        };
    }

    /**
     * 构建序列化处理器基础模块
     *
     * @return 模块实例
     */
    private static SimpleModule initializeBaseModule() {
        if (BASE_MODULE == null) {
            synchronized (SimpleModule.class) {
                if (BASE_MODULE == null) {
                    BASE_MODULE = generateBaseModule();
                }
            }
        }
        return BASE_MODULE;
    }

    /**
     * 初始化序列化处理器安全模块
     *
     * @return 模块实例
     */
    private static SimpleModule initializeSafetyModule() {
        if (SAFETY_MODULE == null) {
            synchronized (SimpleModule.class) {
                if (SAFETY_MODULE == null) {
                    SAFETY_MODULE = generateSafetyModule();
                }
            }
        }
        return SAFETY_MODULE;
    }

    /**
     * 初始化null对象序列化处理器更新对象
     *
     * @return null对象序列化处理更新实例
     */
    private static BeanSerializerModifier initializeNullModifier() {
        if (NULL_MODIFIER == null) {
            synchronized (BeanSerializerModifier.class) {
                if (NULL_MODIFIER == null) {
                    NULL_MODIFIER = generateNullModifier();
                }
            }
        }
        return NULL_MODIFIER;
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer() {
        return initializeSerializer(false);
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @param safely 是否注册安全模块
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer(boolean safely) {
        return initializeSerializer(new ObjectMapper(), safely);
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @param mapper 对象序列化处理器实例
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer(@NonNull ObjectMapper mapper) {
        return initializeSerializer(mapper, false);
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @param mapper 对象序列化处理器实例
     * @param safely 是否注册安全模块
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer(@NonNull ObjectMapper mapper, boolean safely) {
        // 设置基本属性
        mapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY)
                .configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, true)
                .configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false)
                .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

        // 注册基础模型
        mapper.registerModule(initializeBaseModule());

        // 注册安全模型
        if (safely) {
            mapper.registerModule(initializeSafetyModule());
            SerializerFactory factory = mapper.getSerializerFactory();
            mapper.setSerializerFactory(factory.withSerializerModifier(initializeNullModifier()));
        }
        return mapper;
    }

    /**
     * 判断数字是否安全
     *
     * @param number 数字
     * @return true/false
     */
    public static boolean isSafelyNumber(Number number) {
        if (number instanceof Long) {
            return BigInteger.valueOf((Long) number).compareTo(JAVA_SCRIPT_MAX_INTEGER_VALUE) <= 0;
        } else if (number instanceof Double) {
            return BigDecimal.valueOf((Double) number).compareTo(JAVA_SCRIPT_MAX_DECIMAL_VALUE) <= 0;
        } else if (number instanceof BigInteger) {
            return ((BigInteger) number).compareTo(JAVA_SCRIPT_MAX_INTEGER_VALUE) <= 0;
        } else if (number instanceof BigDecimal) {
            return ((BigDecimal) number).compareTo(JAVA_SCRIPT_MAX_DECIMAL_VALUE) <= 0;
        }
        return true;
    }

    /**
     * 获取默认的对象序列化处理器
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper getDefaultSerializer() {
        return DEFAULT_SERIALIZER;
    }

    /**
     * 获取安全的对象序列化处理器
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper getSafelySerializer() {
        return SAFELY_SERIALIZER;
    }

    /**
     * 将对象写入输出流
     *
     * @param output 输出流对象
     * @param value  对象实例
     * @throws IOException I/O异常
     */
    public static void write(@NonNull OutputStream output, Object value) throws IOException {
        write(getDefaultSerializer(), output, value);
    }

    /**
     * 将对象写入输出流
     *
     * @param mapper 对象序列化处理器实例
     * @param output 输出流对象
     * @param value  对象实例
     * @throws IOException I/O异常
     */
    public static void write(@NonNull ObjectMapper mapper, @NonNull OutputStream output, Object value)
            throws IOException {
        mapper.writeValue(output, value);
    }

    /**
     * 对象序列化
     *
     * @param object 目标对象实例
     * @return JSON字符串
     */
    public static String serialize(Object object) {
        return serialize(object, false);
    }

    /**
     * 对象序列化
     *
     * @param object 目标对象实例
     * @param safely 是否是安全模式
     * @return JSON字符串
     */
    public static String serialize(Object object, boolean safely) {
        return safely ? serialize(getSafelySerializer(), object) : serialize(getDefaultSerializer(), object);
    }

    /**
     * 对象序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param object 目标对象实例
     * @return JSON字符串
     */
    public static String serialize(@NonNull ObjectMapper mapper, Object object) {
        if (object == null || object instanceof String) {
            return (String) object;
        }
        try {
            return mapper.writeValueAsString(object);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @return 对象实例
     */
    public static Object deserialize(String json) {
        return deserialize(getDefaultSerializer(), json, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, String json) {
        return deserialize(mapper, json, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes) {
        return deserialize(getDefaultSerializer(), bytes, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, byte[] bytes) {
        return deserialize(mapper, bytes, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @return 对象实例
     */
    public static Object deserialize(String json, @NonNull Type type) {
        return deserialize(getDefaultSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @param type   类型对象
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, String json, @NonNull Type type) {
        if (type instanceof Class) {
            return deserialize(mapper, json, (Class<?>) type);
        } else if (json == null) {
            return null;
        }
        JavaType javaType = mapper.getTypeFactory().constructType(type);
        try {
            return mapper.readValue(json, javaType);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, @NonNull Class<T> type) {
        return deserialize(getDefaultSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @param type   类型对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    @SuppressWarnings("unchecked")
    public static <T> T deserialize(@NonNull ObjectMapper mapper, String json, @NonNull Class<T> type) {
        if (json == null || type == String.class) {
            return (T) json;
        }
        try {
            return mapper.readValue(json, type);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型引用对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, @NonNull TypeReference<T> type) {
        return deserialize(getDefaultSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @param type   类型引用对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    @SuppressWarnings("unchecked")
    public static <T> T deserialize(@NonNull ObjectMapper mapper, String json, @NonNull TypeReference<T> type) {
        if (json == null || (type != null && type.getType() == String.class)) {
            return (T) json;
        }
        try {
            return mapper.readValue(json, type);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes, @NonNull Type type) {
        return deserialize(getDefaultSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @param type   类型对象
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, byte[] bytes, @NonNull Type type) {
        if (type instanceof Class) {
            return deserialize(mapper, bytes, (Class<?>) type);
        } else if (bytes == null) {
            return null;
        }
        JavaType javaType = mapper.getTypeFactory().constructType(type);
        try {
            return mapper.readValue(bytes, javaType);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull Class<T> type) {
        return deserialize(getDefaultSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @param type   类型对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, byte[] bytes, @NonNull Class<T> type) {
        if (bytes == null || type == byte[].class) {
            return (T) bytes;
        }
        try {
            return mapper.readValue(bytes, type);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull TypeReference<T> type) {
        return deserialize(getDefaultSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @param type   类型引用对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, byte[] bytes, @NonNull TypeReference<T> type) {
        if (bytes == null || (type != null && type.getType() == byte[].class)) {
            return (T) bytes;
        }
        try {
            return mapper.readValue(bytes, type);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
