package com.chelvc.framework.common.util;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.Serializable;
import java.lang.invoke.SerializedLambda;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.function.Getter;
import com.chelvc.framework.common.function.Setter;
import com.chelvc.framework.common.model.Modification;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import io.protostuff.LinkedBuffer;
import io.protostuff.ProtobufIOUtil;
import io.protostuff.Schema;
import io.protostuff.runtime.RuntimeSchema;
import lombok.NonNull;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.MappingContext;
import ma.glasnost.orika.converter.BidirectionalConverter;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import ma.glasnost.orika.metadata.Type;

/**
 * 对象工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class ObjectUtils {
    /**
     * 空字节数组
     */
    public static final byte[] EMPTY_BYTES = new byte[0];

    /**
     * CPU数量
     */
    public static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

    /**
     * 对象转换映射工厂
     */
    private static final MapperFactory MAPPER_FACTORY = new DefaultMapperFactory.Builder().build();

    /**
     * 对象字段映射表
     */
    private static final Map<Class<?>, Field[]> CLASS_FIELD_MAPPING = new ConcurrentHashMap<>();

    /**
     * SerializedLambda 反序列化缓存
     */
    private static final Map<String, SerializedLambda> LAMBDA_FUNCTION_MAPPING = new ConcurrentHashMap<>();

    /**
     * Protostuff对象类型/Schema映射表
     */
    private static final Map<Class<?>, Schema<?>> PROTOSTUFF_CLASS_SCHEMA_MAPPING = new ConcurrentHashMap<>();

    /**
     * 类对象/属性描述映射表
     */
    private static final Map<Class<?>, List<PropertyDescriptor>> CLASS_PROPERTY_DESCRIPTOR_MAPPING =
            new ConcurrentHashMap<>();

    static {
        // 注册简单的对象拷贝转换器
        MAPPER_FACTORY.getConverterFactory().registerConverter("simple",
                new BidirectionalConverter<Object, Object>() {
                    @Override
                    public Object convertTo(Object object, Type type, MappingContext mappingContext) {
                        return object;
                    }

                    @Override
                    public Object convertFrom(Object object, Type type, MappingContext mappingContext) {
                        return object;
                    }
                });

        // 注册Modification属性转换器
        MAPPER_FACTORY.classMap(Modification.class, Modification.class)
                .fieldMap("before", "before").converter("simple").add()
                .fieldMap("after", "after").converter("simple").add().byDefault().register();
    }

    private ObjectUtils() {
    }

    /**
     * 判断数组是否为空
     *
     * @param array 字节数组
     * @return true/false
     */
    public static boolean isEmpty(byte[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 短整型数组
     * @return true/false
     */
    public static boolean isEmpty(short[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 整型数组
     * @return true/false
     */
    public static boolean isEmpty(int[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 长整型数组
     * @return true/false
     */
    public static boolean isEmpty(long[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 单精度数组
     * @return true/false
     */
    public static boolean isEmpty(float[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 双精度数组
     * @return true/false
     */
    public static boolean isEmpty(double[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 字符数组
     * @return true/false
     */
    public static boolean isEmpty(char[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 布尔数组
     * @return true/false
     */
    public static boolean isEmpty(boolean[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 对象数组
     * @param <T>   对象类型
     * @return true/false
     */
    public static <T> boolean isEmpty(T[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断字典是否为空
     *
     * @param map 键/值映射表
     * @param <K> 键类型
     * @param <V> 值类型
     * @return true/false
     */
    public static <K, V> boolean isEmpty(Map<K, V> map) {
        return map == null || map.isEmpty();
    }

    /**
     * 判断集合是否为空
     *
     * @param collection 对象集合
     * @param <T>        对象类型
     * @return true/false
     */
    public static <T> boolean isEmpty(Collection<T> collection) {
        return collection == null || collection.isEmpty();
    }

    /**
     * 判断对象是否为空
     *
     * @param object 对象实例
     * @return true/false
     */
    public static boolean isBlank(Object object) {
        return StringUtils.isEmpty(object)
                || (object instanceof byte[] && ((byte[]) object).length == 0)
                || (object instanceof short[] && ((short[]) object).length == 0)
                || (object instanceof int[] && ((int[]) object).length == 0)
                || (object instanceof long[] && ((long[]) object).length == 0)
                || (object instanceof float[] && ((float[]) object).length == 0)
                || (object instanceof double[] && ((double[]) object).length == 0)
                || (object instanceof char[] && ((char[]) object).length == 0)
                || (object instanceof boolean[] && ((boolean[]) object).length == 0)
                || (object instanceof Object[] && ((Object[]) object).length == 0)
                || (object instanceof Map && ((Map<?, ?>) object).isEmpty())
                || (object instanceof Collection && ((Collection<?>) object).isEmpty());
    }

    /**
     * 判断数组是否非空
     *
     * @param array 字节数组
     * @return true/false
     */
    public static boolean notEmpty(byte[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 短整型数组
     * @return true/false
     */
    public static boolean notEmpty(short[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 整型数组
     * @return true/false
     */
    public static boolean notEmpty(int[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 长整型数组
     * @return true/false
     */
    public static boolean notEmpty(long[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 单精度数组
     * @return true/false
     */
    public static boolean notEmpty(float[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 双精度数组
     * @return true/false
     */
    public static boolean notEmpty(double[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 字符数组
     * @return true/false
     */
    public static boolean notEmpty(char[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 布尔数组
     * @return true/false
     */
    public static boolean notEmpty(boolean[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 对象数组
     * @param <T>   对象类型
     * @return true/false
     */
    public static <T> boolean notEmpty(T[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断字典是否非空
     *
     * @param map 键/值映射表
     * @param <K> 键类型
     * @param <V> 值类型
     * @return true/false
     */
    public static <K, V> boolean notEmpty(Map<K, V> map) {
        return !isEmpty(map);
    }

    /**
     * 判断集合是否非空
     *
     * @param collection 对象集合
     * @param <T>        对象类型
     * @return true/false
     */
    public static <T> boolean notEmpty(Collection<T> collection) {
        return !isEmpty(collection);
    }

    /**
     * 判断对象是否非空
     *
     * @param object 对象实例
     * @return true/false
     */
    public static boolean notBlank(Object object) {
        return !isBlank(object);
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object 目标对象
     * @param value  替换对象
     * @param <T>    对象类型泛型
     * @return 返回对象
     */
    public static <T> T ifNull(T object, T value) {
        return object == null ? value : object;
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object   目标对象
     * @param supplier 替换对象提供函数
     * @param <T>      对象类型泛型
     * @return 返回对象
     */
    public static <T> T ifNull(T object, @NonNull Supplier<T> supplier) {
        return object == null ? supplier.get() : object;
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object   目标对象
     * @param function 对象处理函数
     * @param <T>      目标对象类型泛型
     * @param <R>      返回对象类型泛型
     * @return 返回对象
     */
    public static <T, R> R ifNull(T object, @NonNull Function<T, R> function) {
        return object == null ? null : function.apply(object);
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object   目标对象
     * @param function 对象处理函数
     * @param supplier 替换对象提供函数
     * @param <T>      目标对象类型泛型
     * @param <R>      返回对象类型泛型
     * @return 返回对象
     */
    public static <T, R> R ifNull(T object, @NonNull Function<T, R> function, @NonNull Supplier<R> supplier) {
        return object == null ? supplier.get() : ifNull(function.apply(object), supplier);
    }

    /**
     * 获取对象字段
     *
     * @param clazz 目标对象
     * @param name  字段名称
     * @return 字段实例
     */
    public static Field getField(@NonNull Class<?> clazz, @NonNull String name) {
        try {
            return clazz.getDeclaredField(name);
        } catch (NoSuchFieldException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 查找对象字段（包含在所有父类中查找）
     *
     * @param clazz 目标对象
     * @param name  字段名称
     * @return 字段实例
     */
    public static Field lookupField(@NonNull Class<?> clazz, @NonNull String name) {
        Field field = null;
        NoSuchFieldException undefined = null;
        do {
            try {
                field = clazz.getDeclaredField(name);
            } catch (NoSuchFieldException e) {
                if (undefined == null) {
                    undefined = e;
                }
            }
        } while (field == null && (clazz = clazz.getSuperclass()) != Object.class);
        if (field == null) {
            throw new RuntimeException(undefined);
        }
        return field;
    }

    /**
     * 获取对象字段值
     *
     * @param object 对象实例
     * @param field  字段对象
     * @return 字段值
     */
    public static Object getValue(Object object, @NonNull Field field) {
        if (object == null) {
            return null;
        } else if (!field.isAccessible()) {
            field.setAccessible(true);
        }
        try {
            return field.get(object);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取对象属性值
     *
     * @param object   对象实例
     * @param property 属性名
     * @return 属性值
     */
    public static Object getValue(Object object, @NonNull String property) {
        return object == null ? null : getValue(object, lookupField(object.getClass(), property));
    }

    /**
     * 设置对象字段值
     *
     * @param object 对象实例
     * @param field  属性字段对象
     * @param value  字段值
     */
    public static void setValue(Object object, @NonNull Field field, Object value) {
        if (object == null) {
            return;
        } else if (!field.isAccessible()) {
            field.setAccessible(true);
        }
        try {
            field.set(object, value);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 设置对象属性值
     *
     * @param object   对象实例
     * @param property 属性名
     * @param value    属性值
     */
    public static void setValue(Object object, @NonNull String property, Object value) {
        if (object != null) {
            setValue(object, lookupField(object.getClass(), property), value);
        }
    }

    /**
     * 类及父类实例字段访问
     *
     * @param clazz    对象类型
     * @param consumer 字段接收器
     */
    public static void iterateFields(@NonNull Class<?> clazz, @NonNull BiConsumer<Integer, Field> consumer) {
        iterateFields(clazz, (index, field) -> {
            consumer.accept(index, field);
            return true;
        });
    }

    /**
     * 类及父类实例字段访问
     *
     * @param clazz    对象类型
     * @param function 字段接收器（返回false则中断循环）
     */
    public static void iterateFields(@NonNull Class<?> clazz, @NonNull BiFunction<Integer, Field, Boolean> function) {
        int index = 0;
        boolean continuing = true;
        do {
            for (Field field : CLASS_FIELD_MAPPING.computeIfAbsent(clazz, Class::getDeclaredFields)) {
                if (!Modifier.isStatic(field.getModifiers()) && !field.getName().startsWith("this$")) {
                    if (!(continuing = Boolean.TRUE.equals(function.apply(index++, field)))) {
                        break;
                    }
                }
            }
        } while (continuing && (clazz = clazz.getSuperclass()) != Object.class);
    }

    /**
     * 获取对象属性复制处理器工厂
     *
     * @return 对象属性复制处理器工厂
     */
    public static MapperFactory getCopierBuilder() {
        return MAPPER_FACTORY;
    }

    /**
     * 对象属性复制
     *
     * @param source 源对象实例
     * @param target 目标对象实例
     * @param <T>    源对象类型
     * @param <R>    目标对象类型
     */
    public static <T, R> void copying(T source, R target) {
        getCopierBuilder().getMapperFacade().map(source, target);
    }

    /**
     * 对象属性复制
     *
     * @param source 源对象实例
     * @param target 目标对象
     * @param <T>    源对象类型
     * @param <R>    目标对象类型
     * @return 目标对象实例
     */
    public static <T, R> R copying(T source, Class<R> target) {
        return getCopierBuilder().getMapperFacade().map(source, target);
    }

    /**
     * 对象属性复制
     *
     * @param sources 源对象实例数组
     * @param target  目标对象
     * @param <T>     源对象类型
     * @param <R>     目标对象类型
     * @return 目标对象实例列表
     */
    public static <T, R> List<R> copying(T[] sources, @NonNull Class<R> target) {
        return isEmpty(sources) ? Collections.emptyList() :
                getCopierBuilder().getMapperFacade().mapAsList(sources, target);
    }

    /**
     * 对象属性复制
     *
     * @param sources 源对象实例列表
     * @param target  目标对象
     * @param <T>     源对象类型
     * @param <R>     目标对象类型
     * @return 目标对象实例列表
     */
    public static <T, R> List<R> copying(Collection<T> sources, @NonNull Class<R> target) {
        return isEmpty(sources) ? Collections.emptyList() :
                getCopierBuilder().getMapperFacade().mapAsList(sources, target);
    }

    /**
     * 判断数组是否为空，如果数组为空则调用适配函数并返回
     *
     * @param array    对象数组
     * @param supplier 默认数组提供函数
     * @param <T>      对象类型
     * @return 对象数组
     */
    public static <T> T[] ifEmpty(T[] array, @NonNull Supplier<T[]> supplier) {
        return isEmpty(array) ? supplier.get() : array;
    }

    /**
     * 判断数组是否为空，如果集合非空则使用适配函数
     *
     * @param array    对象数组
     * @param function 数组处理函数
     * @param <T>      对象类型
     * @param <R>      目标类型
     * @return 对象数组
     */
    public static <T, R> R[] ifEmpty(T[] array, @NonNull Function<T[], R[]> function) {
        return isEmpty(array) ? null : function.apply(array);
    }

    /**
     * 判断数组是否为空，如果集合非空则使用适配函数
     *
     * @param array    对象数组
     * @param function 数组处理函数
     * @param supplier 默认数组提供函数
     * @param <T>      对象类型
     * @param <R>      目标类型
     * @return 对象数组
     */
    public static <T, R> R[] ifEmpty(T[] array, @NonNull Function<T[], R[]> function, @NonNull Supplier<R[]> supplier) {
        return isEmpty(array) ? supplier.get() : ifEmpty(function.apply(array), supplier);
    }

    /**
     * 判断集合是否为空，如果集合为空则调用适配函数并返回
     *
     * @param collection 对象集合
     * @param supplier   默认集合提供函数
     * @param <T>        对象类型
     * @param <C>        集合类型
     * @return 对象集合
     */
    public static <T, C extends Collection<T>> C ifEmpty(C collection, @NonNull Supplier<C> supplier) {
        return isEmpty(collection) ? supplier.get() : collection;
    }

    /**
     * 判断集合是否为空，如果集合非空则使用适配函数
     *
     * @param collection 对象集合
     * @param function   集合处理函数
     * @param <T>        对象类型
     * @param <R>        目标类型
     * @param <C>        集合类型
     * @return 对象集合
     */
    public static <T, R, C extends Collection<T>> List<R> ifEmpty(C collection,
                                                                  @NonNull Function<C, List<R>> function) {
        return isEmpty(collection) ? Collections.emptyList() : function.apply(collection);
    }

    /**
     * 判断集合是否为空，如果集合非空则使用适配函数
     *
     * @param collection 对象集合
     * @param function   集合处理函数
     * @param supplier   默认集合提供函数
     * @param <T>        对象类型
     * @param <R>        目标类型
     * @param <C>        集合类型
     * @return 对象集合
     */
    public static <T, R, C extends Collection<T>> List<R> ifEmpty(C collection, @NonNull Function<C, List<R>> function,
                                                                  @NonNull Supplier<List<R>> supplier) {
        return isEmpty(collection) ? supplier.get() : ifEmpty(function.apply(collection), supplier);
    }

    /**
     * 合并集合
     *
     * @param collections 集合数组
     * @param <T>         对象类型
     * @return 对象列表
     */
    @SafeVarargs
    public static <T> List<T> merge(Collection<T>... collections) {
        if (collections == null || collections.length == 0) {
            return Collections.emptyList();
        }
        return Stream.of(collections).filter(Objects::nonNull).flatMap(Collection::stream).filter(Objects::nonNull)
                .distinct().collect(Collectors.toList());
    }

    /**
     * 获取Protostuff schema
     *
     * @param clazz 目标对象
     * @param <T>   目标对象类型
     * @return Schema对象实例
     */
    @SuppressWarnings("unchecked")
    private static <T> Schema<T> getProtostuffSchema(Class<T> clazz) {
        return (Schema<T>) PROTOSTUFF_CLASS_SCHEMA_MAPPING.computeIfAbsent(clazz, k -> RuntimeSchema.getSchema(clazz));
    }

    /**
     * 序列化方法，把指定对象序列化成字节数组
     *
     * @param object 被序列化对象实例
     * @return 字节数组
     */
    @SuppressWarnings("unchecked")
    public static <T> byte[] serialize(T object) {
        if (object == null) {
            return null;
        }
        Schema<T> schema = getProtostuffSchema((Class<T>) object.getClass());
        LinkedBuffer buffer = LinkedBuffer.allocate(LinkedBuffer.DEFAULT_BUFFER_SIZE);
        try {
            return ProtobufIOUtil.toByteArray(object, schema, buffer);
        } finally {
            buffer.clear();
        }
    }

    /**
     * 反序列化方法，将字节数组反序列化成指定Class类型
     *
     * @param bytes 字节数组
     * @param clazz 目标对象类型
     * @return 目标对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull Class<T> clazz) {
        if (bytes == null) {
            return null;
        }
        Schema<T> schema = getProtostuffSchema(clazz);
        T object = schema.newMessage();
        ProtobufIOUtil.mergeFrom(bytes, object, schema);
        return object;
    }

    /**
     * 获取对象属性描述列表
     *
     * @param clazz 目标对象
     * @return 属性描述列表
     */
    public static List<PropertyDescriptor> getPropertyDescriptors(@NonNull Class<?> clazz) {
        return CLASS_PROPERTY_DESCRIPTOR_MAPPING.computeIfAbsent(clazz, c -> {
            BeanInfo bean;
            try {
                bean = Introspector.getBeanInfo(c, Object.class);
            } catch (IntrospectionException e) {
                throw new RuntimeException(e);
            }
            return Collections.unmodifiableList(Arrays.asList(bean.getPropertyDescriptors()));
        });
    }

    /**
     * 查找属性对应的Getter方法
     *
     * @param clazz    目标对象
     * @param property 属性名称
     * @return 方法对象
     */
    public static Method lookupPropertyGetter(@NonNull Class<?> clazz, @NonNull String property) {
        return getPropertyDescriptors(clazz).stream().filter(descriptor -> descriptor.getName().equals(property))
                .findAny().map(PropertyDescriptor::getReadMethod).orElse(null);
    }

    /**
     * 查找函数Lambda对象
     *
     * @param function 可序列化函数对象实例
     * @return Lambda对象序列化对象实例
     */
    private static SerializedLambda lookupFunctionLambda(Serializable function) {
        Class<?> clazz = function.getClass();
        return LAMBDA_FUNCTION_MAPPING.computeIfAbsent(clazz.getName(), name -> {
            try {
                Method method = clazz.getDeclaredMethod("writeReplace");
                method.setAccessible(true);
                return (SerializedLambda) method.invoke(function);
            } catch (ReflectiveOperationException e) {
                throw new RuntimeException(e);
            }
        });
    }

    /**
     * 查找Getter函数Lambda对象
     *
     * @param getter Getter函数实例
     * @param <T>    函数调用对象类型
     * @param <R>    函数调用返回类型
     * @return Lambda对象序列化对象实例
     */
    public static <T, R> SerializedLambda lookupGetterLambda(@NonNull Getter<T, R> getter) {
        return lookupFunctionLambda(getter);
    }

    /**
     * 查找Setter函数Lambda对象
     *
     * @param setter Setter函数实例
     * @param <T>    函数调用对象类型
     * @param <U>    函数调用参数类型
     * @return Lambda对象序列化对象实例
     */
    public static <T, U> SerializedLambda lookupSetterLambda(@NonNull Setter<T, U> setter) {
        return lookupFunctionLambda(setter);
    }

    /**
     * 查找Getter函数对应属性名
     *
     * @param getter Getter函数实例
     * @param <T>    函数调用对象类型
     * @param <R>    函数调用返回类型
     * @return 属性名
     */
    public static <T, R> String lookupGetterProperty(@NonNull Getter<T, R> getter) {
        SerializedLambda lambda = lookupGetterLambda(getter);
        String method = lambda.getImplMethodName();
        if (method.startsWith("is")) {
            return StringUtils.hump(method.substring(2));
        } else if (method.startsWith("get")) {
            return StringUtils.hump(method.substring(3));
        }
        throw new RuntimeException("Invalid getter method: " + method);
    }

    /**
     * 查找Setter函数对应属性名
     *
     * @param setter Setter函数实例
     * @param <T>    函数调用对象类型
     * @param <U>    函数调用参数类型
     * @return 属性名
     */
    public static <T, U> String lookupSetterProperty(@NonNull Setter<T, U> setter) {
        SerializedLambda lambda = lookupSetterLambda(setter);
        String method = lambda.getImplMethodName();
        if (method.startsWith("set")) {
            return StringUtils.hump(method.substring(3));
        }
        throw new RuntimeException("Invalid setter method: " + method);
    }

    /**
     * 对象类型转换
     *
     * @param type 对象类型
     * @return 对象类型
     */
    public static Class<?> type2class(java.lang.reflect.Type type) {
        if (type instanceof ParameterizedType) {
            return (Class<?>) ((ParameterizedType) type).getRawType();
        }
        return (Class<?>) type;
    }

    /**
     * 获取对象所有父类/接口类型
     *
     * @param clazz 对象类型
     * @return 父类/接口类型列表
     */
    public static List<java.lang.reflect.Type> getGenericSuperclasses(@NonNull Class<?> clazz) {
        java.lang.reflect.Type superclass = clazz.getGenericSuperclass();
        java.lang.reflect.Type[] interfaces = clazz.getGenericInterfaces();
        if (superclass == null && ObjectUtils.isEmpty(interfaces)) {
            return Collections.emptyList();
        } else if (superclass == null) {
            return Arrays.asList(interfaces);
        } else if (ObjectUtils.isEmpty(interfaces)) {
            return Collections.singletonList(superclass);
        }
        List<java.lang.reflect.Type> superclasses = Lists.newArrayList(interfaces);
        superclasses.add(superclass);
        return superclasses;
    }

    /**
     * 查找对象父类/接口
     *
     * @param clazz      对象类型
     * @param superclass 父类/接口类型
     * @return 父类/接口类型
     */
    public static java.lang.reflect.Type lookupGenericSuperclass(@NonNull Class<?> clazz,
                                                                 @NonNull Class<?> superclass) {
        if (clazz == superclass) {
            return clazz;
        }
        for (java.lang.reflect.Type type : getGenericSuperclasses(clazz)) {
            Class<?> classed = type2class(type);
            if (Objects.equals(classed, superclass)) {
                return type;
            }
            java.lang.reflect.Type found = lookupGenericSuperclass(classed, superclass);
            if (found != null) {
                return found;
            }
        }
        return null;
    }

    /**
     * 查找对象父类/接口泛型
     *
     * @param clazz      对象类型
     * @param superclass 父类/接口类型
     * @return 泛型类型
     */
    public static java.lang.reflect.Type lookupSuperclassParameterized(@NonNull Class<?> clazz,
                                                                       @NonNull Class<?> superclass) {
        java.lang.reflect.Type supered = lookupGenericSuperclass(clazz, superclass);
        if (supered instanceof ParameterizedType) {
            java.lang.reflect.Type[] arguments = ((ParameterizedType) supered).getActualTypeArguments();
            return ObjectUtils.isEmpty(arguments) ? Object.class : arguments[0];
        }
        return Object.class;
    }

    /**
     * 将对象转换成长整形数字
     *
     * @param object 目标对象
     * @return 长整形数字
     */
    public static Long object2long(Object object) {
        if (object == null || StringUtils.isEmpty(object)) {
            return null;
        } else if (object instanceof Byte || object instanceof Short || object instanceof Integer
                || object instanceof Long) {
            return ((Number) object).longValue();
        }
        return Long.parseLong(object.toString());
    }

    /**
     * 将对象转换成属性名/值映射表
     *
     * @param object 对象实例
     * @return 对象属性名/值映射表
     */
    public static Map<String, Object> object2map(Object object) {
        if (object == null) {
            return Collections.emptyMap();
        }
        Map<String, Object> map = Maps.newHashMap();
        iterateFields(object.getClass(), (i, field) -> {
            map.put(field.getName(), getValue(object, field));
        });
        return map;
    }

    /**
     * 判断对象是否相同
     *
     * @param object 对象实例
     * @param other  对象实例
     * @return true/false
     */
    public static boolean equals(Object object, Object other) {
        if (Objects.equals(object, other)
                || (object instanceof Map && ((Map<?, ?>) object).isEmpty() && Objects.isNull(other))
                || (other instanceof Map && ((Map<?, ?>) other).isEmpty() && Objects.isNull(object))
                || (object instanceof Object[] && ((Object[]) object).length == 0 && Objects.isNull(other))
                || (other instanceof Object[] && ((Object[]) other).length == 0 && Objects.isNull(object))
                || (object instanceof Collection && ((Collection<?>) object).isEmpty() && Objects.isNull(other))
                || (other instanceof Collection && ((Collection<?>) other).isEmpty() && Objects.isNull(object))) {
            return true;
        } else if (object instanceof Map && other instanceof Map) {
            if (((Map<?, ?>) object).size() != ((Map<?, ?>) other).size()) {
                return false;
            }
            for (Map.Entry<?, ?> entry : ((Map<?, ?>) object).entrySet()) {
                if (!equals(entry.getValue(), ((Map<?, ?>) other).get(entry.getKey()))) {
                    return false;
                }
            }
            for (Map.Entry<?, ?> entry : ((Map<?, ?>) other).entrySet()) {
                if (!equals(entry.getValue(), ((Map<?, ?>) object).get(entry.getKey()))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Object[] && other instanceof Object[]) {
            if (((Object[]) object).length != ((Object[]) other).length) {
                return false;
            }
            for (Object value : (Object[]) object) {
                if (Stream.of((Object[]) other).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Object[]) other) {
                if (Stream.of((Object[]) object).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Object[] && other instanceof Collection) {
            if (((Object[]) object).length != ((Collection<?>) other).size()) {
                return false;
            }
            for (Object value : (Object[]) object) {
                if (((Collection<?>) other).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Collection<?>) other) {
                if (Stream.of((Object[]) object).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Collection && other instanceof Collection) {
            if (((Collection<?>) object).size() != ((Collection<?>) other).size()) {
                return false;
            }
            for (Object value : (Collection<?>) object) {
                if (((Collection<?>) other).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Collection<?>) other) {
                if (((Collection<?>) object).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Collection && other instanceof Object[]) {
            if (((Collection<?>) object).size() != ((Object[]) other).length) {
                return false;
            }
            for (Object value : (Collection<?>) object) {
                if (Stream.of((Object[]) other).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Object[]) other) {
                if (((Collection<?>) object).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    /**
     * 获取两个对象实例不同属性值
     *
     * @param <T>      数据类型
     * @param object   对象实例
     * @param other    对象实例
     * @param excludes 需要排除的属性名称集合
     * @return 不同属性值
     */
    public static <T> List<Modification> compare(T object, T other, @NonNull String... excludes) {
        if (object == null || other == null) {
            return Collections.emptyList();
        }
        Set<String> excludeFields = excludes.length == 0 ? Collections.emptySet() : Sets.newHashSet(excludes);
        List<Modification> modifications = Lists.newLinkedList();
        iterateFields(object.getClass(), (index, field) -> {
            if (excludeFields.isEmpty() || !excludeFields.contains(field.getName())) {
                Object before = getValue(object, field);
                Object after = getValue(other, field);
                if (!equals(before, after)) {
                    modifications.add(Modification.builder().name(field.getName()).before(before).after(after).build());
                }
            }
        });
        return modifications;
    }

    /**
     * 循环获取对象
     *
     * @param supplier 对象提供函数
     * @param function 对象处理函数（返回false则中断循环）
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T> T loop(@NonNull Supplier<T> supplier, @NonNull BiFunction<Integer, T, Boolean> function) {
        T t;
        int count = 1;
        do {
            t = supplier.get();
        } while (Boolean.TRUE.equals(function.apply(count++, t)));
        return t;
    }

    /**
     * 数字分组
     *
     * @param total 总数
     * @param size  分组大小
     * @return 分组数量
     */
    public static int group(int total, int size) {
        if (size < 1 || size >= total) {
            return 1;
        }
        return total % size == 0 ? total / size : total / size + 1;
    }

    /**
     * 对象列表拆分
     *
     * @param objects 对象列表
     * @param size    分组大小
     * @param <T>     对象类型
     * @return 对象分组列表
     */
    public static <T> List<List<T>> split(List<T> objects, int size) {
        return isEmpty(objects) ? Collections.emptyList() : group(objects, group(objects.size(), size));
    }

    /**
     * 对象列表拆分
     *
     * @param objects  对象列表
     * @param size     分组大小
     * @param consumer 对象分组处理器
     * @param <T>      对象类型
     */
    public static <T> void split(List<T> objects, int size, @NonNull BiConsumer<Integer, List<T>> consumer) {
        if (notEmpty(objects)) {
            group(objects, group(objects.size(), size), consumer);
        }
    }

    /**
     * 对象列表分组
     *
     * @param objects 对象列表
     * @param group   分组数量
     * @param <T>     对象类型
     * @return 对象分组列表
     */
    public static <T> List<List<T>> group(List<T> objects, int group) {
        if (isEmpty(objects)) {
            return Collections.emptyList();
        } else if (group < 2) {
            return Collections.singletonList(objects);
        }
        List<List<T>> groups = Lists.newArrayListWithCapacity(group);
        group(objects, group, (index, list) -> groups.add(list));
        return groups;
    }

    /**
     * 对象列表分组
     *
     * @param objects  对象列表
     * @param group    分组数量
     * @param consumer 对象分组处理器
     * @param <T>      对象类型
     */
    public static <T> void group(List<T> objects, int group, @NonNull BiConsumer<Integer, List<T>> consumer) {
        if (isEmpty(objects)) {
            return;
        }
        if (group < 2) {
            consumer.accept(0, objects);
        } else {
            int total = objects.size(), size = group(total, group);
            for (int i = 0; i < group; i++) {
                consumer.accept(i, objects.subList(i * size, Math.min((i + 1) * size, total)));
            }
        }
    }

    /**
     * 获取从缓存中获取字节数组
     *
     * @param buffer 直接缓存
     * @return 字节数组
     */
    public static byte[] getBytes(ByteBuffer buffer) {
        if (buffer == null) {
            return null;
        }
        byte[] bytes = new byte[buffer.remaining()];
        buffer.get(bytes, 0, bytes.length);
        return bytes;
    }

    /**
     * 将对象转换为对象流
     *
     * @param object 对象实例
     * @return 对象流实例
     */
    public static Stream<?> stream(Object object) {
        if (object instanceof Stream) {
            return (Stream<?>) object;
        } else if (object instanceof Object[]) {
            return Stream.of((Object[]) object);
        } else if (object instanceof Collection) {
            return ((Collection<?>) object).stream();
        }
        return Stream.of(object);
    }
}