package com.chelvc.framework.database.config;

import java.util.Collection;
import java.util.Comparator;
import java.util.function.Supplier;

import com.baomidou.mybatisplus.core.incrementer.IdentifierGenerator;
import com.baomidou.mybatisplus.core.injector.ISqlInjector;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.InnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.TenantLineInnerInterceptor;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.Handler;
import com.chelvc.framework.common.function.Provider;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.context.Transactor;
import com.chelvc.framework.database.interceptor.DeletedExcludeHandler;
import com.chelvc.framework.database.support.CustomerSqlInjector;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.transaction.annotation.Transactional;

/**
 * 数据库配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class DatabaseConfigurer {
    private final ApplicationContext applicationContext;

    @Bean
    public Transactor transactor() {
        return new Transactor() {
            @Override
            @Transactional(rollbackFor = Exception.class)
            public void execute(@NonNull Executor executor) {
                executor.execute();
            }

            @Override
            @Transactional(rollbackFor = Exception.class)
            public <T> T execute(@NonNull Supplier<T> executor) {
                return executor.get();
            }

            @Override
            @Transactional(rollbackFor = Exception.class)
            public void execute(@NonNull Handler executor) throws Exception {
                executor.handle();
            }

            @Override
            @Transactional(rollbackFor = Exception.class)
            public <T> T execute(@NonNull Provider<T> executor) throws Exception {
                return executor.get();
            }
        };
    }

    @Bean
    public ISqlInjector sqlInjector() {
        return new CustomerSqlInjector();
    }

    @Bean
    @ConditionalOnClass(name = "com.chelvc.framework.redis.context.RedisContextHolder")
    public IdentifierGenerator identifierGenerator() {
        return entity -> RedisContextHolder.identity();
    }

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();

        // 设置自定义租户隔离拦截器
        Collection<InnerInterceptor> wheres = this.applicationContext.getBeansOfType(InnerInterceptor.class).values();
        wheres.stream().sorted(Comparator.comparingInt(where ->
                ObjectUtils.ifNull(where.getClass().getAnnotation(Order.class), Order::value, () -> 0))
        ).forEach(interceptor::addInnerInterceptor);

        // 设置删除数据过滤拦截器
        interceptor.addInnerInterceptor(new TenantLineInnerInterceptor() {
            {
                this.setTenantLineHandler(new DeletedExcludeHandler());
            }
        });

        // 设置分页拦截器，数据过滤条件与拦截器设置顺序有关，分页拦截器最后设置才能在分页时同时使用数据隔离过滤条件
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor());
        return interceptor;
    }
}