package com.chelvc.framework.database.context;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.sql.Connection;
import java.sql.Statement;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.crypto.Cipher;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.Handler;
import com.chelvc.framework.common.function.Provider;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.annotation.Confidential;
import com.chelvc.framework.database.annotation.Unique;
import com.chelvc.framework.database.config.DatabaseProperties;
import com.chelvc.framework.database.entity.Creatable;
import com.chelvc.framework.database.entity.Deletable;
import com.chelvc.framework.database.entity.Entity;
import com.chelvc.framework.database.entity.Updatable;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.context.ApplicationContext;
import org.springframework.util.CollectionUtils;

/**
 * 数据库操作相关上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class DatabaseContextHolder {
    /**
     * 类对象/业务操作实例映射表
     */
    private static final Map<Class<?>, IService<Object>> CLASS_SERVICE_MAPPING = new ConcurrentHashMap<>();

    /**
     * 对象/唯一约束条件列表映射
     */
    private static final Map<Class<?>, List<UniqueContext<?>>> CLASS_UNIQUE_MAPPING = new ConcurrentHashMap<>();

    /**
     * 对象加密数据字段映射表
     */
    private static final Map<Class<?>, List<Field>> CLASS_CONFIDENTIAL_FIELD_MAPPING = new ConcurrentHashMap<>();

    /**
     * 配置属性
     */
    private static DatabaseProperties PROPERTIES;

    /**
     * 事务处理器
     */
    private static Transactor TRANSACTOR;

    /**
     * 全局SqlSessionTemplate实例
     */
    private static SqlSessionTemplate SESSION_TEMPLATE;

    /**
     * 数据加密密码处理器
     */
    private static final ThreadLocal<Cipher> ENCRYPT_CIPHER =
            ThreadLocal.withInitial(() -> getCipher(Cipher.ENCRYPT_MODE));

    /**
     * 数据解密密码处理器
     */
    private static final ThreadLocal<Cipher> DECRYPT_CIPHER =
            ThreadLocal.withInitial(() -> getCipher(Cipher.DECRYPT_MODE));

    private DatabaseContextHolder() {
    }

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    public static DatabaseProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (DatabaseProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(DatabaseProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取事务处理器
     *
     * @return 事务处理器
     */
    public static Transactor getTransactor() {
        if (TRANSACTOR == null) {
            synchronized (Transactor.class) {
                if (TRANSACTOR == null) {
                    TRANSACTOR = ApplicationContextHolder.getBean(Transactor.class);
                }
            }
        }
        return TRANSACTOR;
    }

    /**
     * 获取SqlSessionTemplate
     *
     * @return SqlSessionTemplate实例
     */
    public static SqlSessionTemplate getSessionTemplate() {
        if (SESSION_TEMPLATE == null) {
            synchronized (SqlSessionTemplate.class) {
                if (SESSION_TEMPLATE == null) {
                    SESSION_TEMPLATE = ApplicationContextHolder.getBean(SqlSessionTemplate.class);
                }
            }
        }
        return SESSION_TEMPLATE;
    }

    /**
     * 获取原生的数据库连接
     *
     * @return 数据库连接对象
     */
    public static Connection getConnection() {
        return getSessionTemplate().getConnection();
    }

    /**
     * 执行原生SQL
     *
     * @param sql SQL语句
     * @return true/false
     * @throws Exception 执行异常
     */
    public static boolean execute(@NonNull String sql) throws Exception {
        try (Statement statement = getConnection().createStatement()) {
            return statement.execute(sql);
        }
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Executor executor) {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     */
    public static <T> T transactional(@NonNull Supplier<T> executor) {
        return getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Handler executor) throws Exception {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     * @throws Exception 执行异常
     */
    public static <T> T transactional(@NonNull Provider<T> executor) throws Exception {
        return getTransactor().execute(executor);
    }

    /**
     * 根据表名获取表信息
     *
     * @param name 表名称
     * @return 表信息
     */
    public static TableInfo getTable(String name) {
        if (StringUtils.isEmpty(name)) {
            return null;
        }
        TableInfo table = TableInfoHelper.getTableInfo(name);
        if (table == null) {
            if (name.charAt(0) == '`' && name.charAt(name.length() - 1) == '`') {
                table = TableInfoHelper.getTableInfo(name.substring(1, name.length() - 1));
            } else {
                table = TableInfoHelper.getTableInfo("`" + name + "`");
            }
        }
        return table;
    }

    /**
     * 根据数据模型类型获取数据表信息
     *
     * @param type 数据模型类型
     * @return 数据表信息
     */
    public static TableInfo getTable(Class<?> type) {
        return getTable(type, true);
    }

    /**
     * 根据数据模型类型获取数据表信息
     *
     * @param type     数据模型类型
     * @param required 表信息是否必须存在
     * @return 数据表信息
     */
    public static TableInfo getTable(@NonNull Class<?> type, boolean required) {
        TableInfo table = TableInfoHelper.getTableInfo(type);
        if (required) {
            return AssertUtils.nonnull(table, () -> "Table not found: " + type.getName());
        }
        return table;
    }

    /**
     * 获取实体主键值
     *
     * @param entity 对象实体
     * @return 实体主键
     */
    public static Object getEntityIdentity(@NonNull Object entity) {
        if (entity instanceof Entity) {
            return ((Entity<?>) entity).getId();
        }
        TableInfo table = getTable(entity.getClass());
        List<TableFieldInfo> fields = table.getFieldList();
        if (ObjectUtils.notEmpty(fields)) {
            for (TableFieldInfo info : fields) {
                Field field = info.getField();
                if (field.isAnnotationPresent(TableId.class)) {
                    return ObjectUtils.getValue(entity, field);
                }
            }
        }
        return null;
    }

    /**
     * 将属性Getter方法引用转换成数据列信息
     *
     * @param getter 属性Getter方法引用
     * @return 数据列信息
     */
    public static ColumnCache getter2column(@NonNull GetterContext<?, ?> getter) {
        Class<?> clazz = getter.getReference();
        String property = getter.getProperty();
        Map<String, ColumnCache> columns = LambdaUtils.getColumnMap(clazz);
        ColumnCache column = columns == null ? null : columns.get(LambdaUtils.formatKey(property));
        return AssertUtils.nonnull(column, () -> "Column not found for property: " + clazz.getName() + "." + property);
    }

    /**
     * 根据字段对象转换Getter方法引用
     *
     * @param clazz Getter方法引用对象
     * @param field 字段对象
     * @param <T>   Getter方法实体类型
     * @param <R>   Getter方法返回类型
     * @return 方法引用
     */
    public static <T, R> GetterContext<T, R> field2getter(@NonNull Class<T> clazz, @NonNull Field field) {
        return new GetterContext<>(field.getName(), clazz, field.getDeclaringClass());
    }

    /**
     * 获取指定对象对应的唯一约束上下文
     *
     * @param clazz 数据实体对象
     * @param <T>   实体类型
     * @return 唯一约束上下文列表
     */
    @SuppressWarnings("unchecked")
    public static <T> List<UniqueContext<T>> getUniqueContexts(@NonNull Class<T> clazz) {
        List<?> contexts = CLASS_UNIQUE_MAPPING.computeIfAbsent(clazz, c -> {
            // 获取唯一约束属性名称/唯一约束注解实例映射
            Map<String, Field> fields = Maps.newHashMap();
            Map<Field, Unique> annotations = Maps.newHashMap();
            ObjectUtils.iterateFields(c, (index, field) -> {
                fields.put(field.getName(), field);
                Unique unique = field.getAnnotation(Unique.class);
                if (unique != null) {
                    annotations.put(field, unique);
                }
            });
            if (CollectionUtils.isEmpty(annotations)) {
                return Collections.emptyList();
            }

            // 构建唯一约束上下文
            List<UniqueContext<T>> uniques = annotations.entrySet().stream().map(entry -> {
                Field field = entry.getKey();
                Unique unique = entry.getValue();
                Stream<String> properties = Stream.concat(Stream.of(field.getName()), Stream.of(unique.composites()));
                List<GetterContext<T, ?>> getters = properties.map(String::trim).distinct()
                        .filter(StringUtils::notEmpty).map(property -> {
                            Field f = fields.get(property);
                            AssertUtils.nonnull(f, () -> "Field not found: " + clazz.getName() + "." + property);
                            return field2getter(clazz, f);
                        }).collect(Collectors.toList());
                return new UniqueContext<T>(field, unique, getters);
            }).collect(Collectors.toList());
            return Collections.unmodifiableList(uniques);
        });
        return (List<UniqueContext<T>>) contexts;
    }

    /**
     * 获取对象的机密字段列表
     *
     * @param clazz 对象类型
     * @return 字段列表
     */
    public static List<Field> getConfidentialFields(@NonNull Class<?> clazz) {
        return CLASS_CONFIDENTIAL_FIELD_MAPPING.computeIfAbsent(clazz, c -> {
            List<Field> fields = new LinkedList<>();
            ObjectUtils.iterateFields(clazz, (index, field) -> {
                if (field.isAnnotationPresent(Confidential.class)) {
                    AssertUtils.check(field.getType() == String.class,
                            () -> "Confidential annotation must using on string field: " + field);
                    fields.add(field);
                }
            });
            return fields.isEmpty() ? Collections.emptyList() : Collections.unmodifiableList(fields);
        });
    }

    /**
     * 是否开启数据加解密功能
     *
     * @return true/false
     */
    public static boolean isCodecEnabled() {
        DatabaseProperties.Codec codec = getProperties().getCodec();
        return StringUtils.notEmpty(codec.getSecret()) && StringUtils.notEmpty(codec.getIv());
    }

    /**
     * 判断对象是否是需要加解密的
     *
     * @param clazz 对象类型
     * @return true/false
     */
    public static boolean isConfidential(@NonNull Class<?> clazz) {
        return ObjectUtils.notEmpty(getConfidentialFields(clazz));
    }

    /**
     * 根据加解密模式获取密码处理器
     *
     * @param mode 加解密模式
     * @return 密码处理器
     */
    public static Cipher getCipher(int mode) {
        if (!isCodecEnabled()) {
            return null;
        }
        DatabaseProperties.Codec codec = getProperties().getCodec();
        return AESUtils.getCipher(mode, codec.getSecret(), codec.getIv());
    }

    /**
     * 获取数据编密码处理器
     *
     * @param consumer 密码处理器接收函数
     */
    public static void encoder(@NonNull Consumer<Cipher> consumer) {
        Cipher cipher = ENCRYPT_CIPHER.get();
        try {
            if (Objects.nonNull(cipher)) {
                consumer.accept(cipher);
            }
        } finally {
            ENCRYPT_CIPHER.remove();
        }
    }

    /**
     * 获取数据解码密码处理器
     *
     * @param consumer 密码处理器接收函数
     */
    public static void decoder(@NonNull Consumer<Cipher> consumer) {
        Cipher cipher = DECRYPT_CIPHER.get();
        try {
            if (Objects.nonNull(cipher)) {
                consumer.accept(cipher);
            }
        } finally {
            DECRYPT_CIPHER.remove();
        }
    }

    /**
     * 编码数据
     *
     * @param plaintext 数据明文
     * @return 数据密文
     */
    public static String encode(String plaintext) {
        if (plaintext == null) {
            return null;
        }
        Cipher cipher = getCipher(Cipher.ENCRYPT_MODE);
        return cipher == null ? plaintext : AESUtils.encode(cipher, plaintext);
    }

    /**
     * 编码敏感数据属性
     *
     * @param entity 数据实体
     */
    public static void encode(@NonNull Object entity) {
        if (isCodecEnabled() && isConfidential(entity.getClass())) {
            encoder(cipher -> encode(cipher, entity));
        }
    }

    /**
     * 编码敏感数据属性
     *
     * @param cipher 密码处理器
     * @param entity 数据实体
     */
    public static void encode(@NonNull Cipher cipher, @NonNull Object entity) {
        List<Field> fields = getConfidentialFields(entity.getClass());
        if (ObjectUtils.notEmpty(fields)) {
            fields.forEach(field -> {
                String plaintext = (String) ObjectUtils.getValue(entity, field);
                String ciphertext = AESUtils.encode(cipher, plaintext);
                ObjectUtils.setValue(entity, field, ciphertext);
            });
        }
    }

    /**
     * 解密数据
     *
     * @param ciphertext 数据密文
     * @return 数据明文
     */
    public static String decode(String ciphertext) {
        if (ciphertext == null) {
            return null;
        }
        Cipher cipher = getCipher(Cipher.DECRYPT_MODE);
        return cipher == null ? ciphertext : AESUtils.decode(cipher, ciphertext);
    }

    /**
     * 解码敏感数据属性
     *
     * @param entity 数据实体
     */
    public static void decode(@NonNull Object entity) {
        if (isCodecEnabled() && isConfidential(entity.getClass())) {
            decoder(cipher -> decode(cipher, entity));
        }
    }

    /**
     * 解码敏感数据属性
     *
     * @param cipher 密码处理器
     * @param entity 数据实体
     */
    public static void decode(@NonNull Cipher cipher, @NonNull Object entity) {
        List<Field> fields = getConfidentialFields(entity.getClass());
        if (ObjectUtils.notEmpty(fields)) {
            fields.forEach(field -> {
                String ciphertext = (String) ObjectUtils.getValue(entity, field);
                String plaintext = AESUtils.decode(cipher, ciphertext);
                ObjectUtils.setValue(entity, field, plaintext);
            });
        }
    }

    /**
     * 根据包名查找数据模型对象
     *
     * @param pack 包路径
     * @return 数据模型对象列表
     */
    public static List<Class<?>> lookupEntityClasses(@NonNull String pack) {
        return SpringUtils.lookupClasses(pack, clazz -> {
            int modifiers = clazz.getModifiers();
            return Entity.class.isAssignableFrom(clazz) && !Modifier.isInterface(modifiers)
                    && !Modifier.isAbstract(modifiers);
        });
    }

    /**
     * 获取指定对象对应的业务操作实例
     *
     * @param clazz 数据实体对象
     * @param <T>   数据对象类型
     * @return 业务操作实例
     */
    @SuppressWarnings("unchecked")
    public static <T> IService<T> lookupEntityService(@NonNull Class<T> clazz) {
        return (IService<T>) CLASS_SERVICE_MAPPING.computeIfAbsent(clazz, c -> {
            ApplicationContext applicationContext = ApplicationContextHolder.getApplicationContext();
            IService<Object> service = applicationContext.getBeansOfType(IService.class).values().stream()
                    .filter(s -> Objects.equals(s.getEntityClass(), clazz)).findAny().orElse(null);
            return AssertUtils.nonnull(service, () -> "Service not found for entity: " + clazz.getName());
        });
    }

    /**
     * 初始化实体默认属性值
     *
     * @param entity 数据实体
     */
    public static void initializeEntityDefaultValue(@NonNull Entity<?> entity) {
        // 设置新增属性默认值
        Long operator = SessionContextHolder.getId();
        if (operator != null && entity instanceof Creatable) {
            Creatable<?> creatable = (Creatable<?>) entity;
            if (creatable.getCreator() == null) {
                creatable.setCreator(operator);
            }
            if (creatable.getCreateTime() == null) {
                creatable.setCreateTime(new Date());
            }
        }

        // 设置修改属性默认值
        if (operator != null && entity instanceof Updatable) {
            Updatable<?> updatable = (Updatable<?>) entity;
            updatable.setUpdater(operator);
            updatable.setUpdateTime(new Date());
        }

        // 设置删除属性默认值
        if (entity instanceof Deletable) {
            Deletable<?> deletable = (Deletable<?>) entity;
            if (deletable.getDeleted() == null) {
                deletable.setDeleted(false);
            }
        }
    }

    /**
     * 初始化唯一约束查询对象
     *
     * @param clazz   对象类型
     * @param entity  对象实例
     * @param uniques 唯一约束上下文列表
     * @param <T>     实体类型
     * @return Lambda查询对象实例
     */
    public static <T> LambdaQueryWrapper<T> initializeUniqueQuery(@NonNull Class<T> clazz, @NonNull T entity,
                                                                  @NonNull List<UniqueContext<T>> uniques) {
        GetterContext.Query<T> query = new GetterContext.Query<>(clazz);
        if (uniques.size() > 1) {
            uniques.forEach(unique -> query.or(subquery -> unique.getGetters().forEach(getter -> {
                Object value = getter.apply(entity);
                if (value != null) {
                    subquery.eq(getter, value);
                }
            })));
        } else {
            uniques.get(0).getGetters().forEach(getter -> {
                Object value = getter.apply(entity);
                if (value != null) {
                    query.eq(getter, value);
                }
            });
        }
        return query;
    }

    /**
     * 将分页请求参数转换成MyBatis Plus分页参数
     *
     * @param paging 分页请求参数
     * @param <T>    目标对象类型泛型
     * @return MyBatis Plus分页参数
     */
    public static <T> Page<T> page(Paging paging) {
        return paging == null ? null : page(paging.getNumber(), paging.getSize(), paging.isCounting());
    }

    /**
     * 将分页请求参数转换成MyBatis Plus分页参数
     *
     * @param paging   分页请求参数
     * @param counting 是否获取总数
     * @param <T>      目标对象类型泛型
     * @return MyBatis Plus分页参数
     */
    public static <T> Page<T> page(Paging paging, boolean counting) {
        return paging == null ? null : page(paging.getNumber(), paging.getSize(), counting);
    }

    /**
     * 构建分页信息
     *
     * @param number 页码
     * @param size   页面大小
     * @param <T>    数据类型
     * @return 分页信息
     */
    public static <T> Page<T> page(long number, long size) {
        return page(number, size, true);
    }

    /**
     * 构建分页信息
     *
     * @param number   页码
     * @param size     页面大小
     * @param counting 是否获取总数
     * @param <T>      数据类型
     * @return 分页信息
     */
    public static <T> Page<T> page(long number, long size, boolean counting) {
        return new Page<>(number, size, counting);
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page MyBatis Plus分页参数
     * @param <T>  目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<T> page) {
        return pagination(page, (List<T>) ObjectUtils.ifNull(page, IPage::getRecords));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page MyBatis Plus分页参数
     * @param type 目标类型
     * @param <T>  目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<?> page, Class<T> type) {
        return pagination(page, ObjectUtils.copying((List<?>) ObjectUtils.ifNull(page, IPage::getRecords), type));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page    MyBatis Plus分页参数
     * @param adapter 对象适配函数
     * @param <T>     原始对象类型泛型
     * @param <R>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T, R> Pagination<R> pagination(IPage<T> page, @NonNull Function<List<T>, List<R>> adapter) {
        List<T> records = ObjectUtils.ifNull(page, IPage::getRecords);
        return pagination(page, CollectionUtils.isEmpty(records) ? Collections.emptyList() : adapter.apply(records));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page    MyBatis Plus分页参数
     * @param records 分页数据列表
     * @param <T>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<?> page, List<T> records) {
        return page == null ? null : Pagination.<T>builder().total(page.getTotal()).pages(page.getPages())
                .records(records).build();
    }

    /**
     * 分页记录转换
     *
     * @param page    原始分页信息
     * @param records 分页数据列表
     * @param <T>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(Pagination<?> page, List<T> records) {
        return page == null ? null : page.convert(records);
    }
}