package com.chelvc.framework.database.support;

import java.util.List;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.injector.AbstractMethod;
import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.chelvc.framework.common.util.StringUtils;
import lombok.NonNull;
import org.apache.ibatis.executor.keygen.Jdbc3KeyGenerator;
import org.apache.ibatis.executor.keygen.KeyGenerator;
import org.apache.ibatis.executor.keygen.NoKeyGenerator;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlSource;

/**
 * 数据插入方法抽象实现
 *
 * @author Woody
 * @date 2024/3/5
 */
public abstract class AbstractInsertMethod extends AbstractMethod {
    /**
     * 方法标识
     */
    protected final String id;

    public AbstractInsertMethod(@NonNull String id) {
        this.id = id;
    }

    /**
     * 将字段转换成列
     *
     * @param table  表信息
     * @param fields 字段列表
     * @return 列信息
     */
    protected String field2column(TableInfo table, List<TableFieldInfo> fields) {
        String columns = table.getKeyInsertSqlColumn(false) +
                this.filterTableFieldInfo(fields, null, TableFieldInfo::getInsertSqlColumn, EMPTY);
        return columns.substring(0, columns.length() - 1);
    }

    /**
     * 将字段转换成属性
     *
     * @param table  表信息
     * @param fields 字段列表
     * @return 属性信息
     */
    protected String field2property(TableInfo table, List<TableFieldInfo> fields) {
        return this.field2property(table, null, fields);
    }

    /**
     * 将字段转换成属性
     *
     * @param table  表信息
     * @param prefix 属性前缀
     * @param fields 字段列表
     * @return 属性信息
     */
    protected String field2property(TableInfo table, String prefix, List<TableFieldInfo> fields) {
        String properties = table.getKeyInsertSqlProperty(prefix, false) +
                this.filterTableFieldInfo(fields, null, i -> i.getInsertSqlProperty(prefix), EMPTY);
        return properties.substring(0, properties.length() - 1);
    }

    /**
     * 初始化方法脚本
     *
     * @param table 表信息
     * @return 脚本信息
     */
    protected abstract String initializeMethodScript(TableInfo table);

    @Override
    public MappedStatement injectMappedStatement(Class<?> mapperClass, Class<?> modelClass, TableInfo tableInfo) {
        String keyColumn = null;
        String keyProperty = null;
        KeyGenerator keyGenerator;
        if (StringUtils.isEmpty(tableInfo.getKeyProperty())) {
            keyGenerator = new NoKeyGenerator();
        } else if (tableInfo.getIdType() == IdType.AUTO) {
            keyColumn = tableInfo.getKeyColumn();
            keyProperty = tableInfo.getKeyProperty();
            keyGenerator = new Jdbc3KeyGenerator();
        } else if (tableInfo.getKeySequence() != null) {
            keyColumn = tableInfo.getKeyColumn();
            keyProperty = tableInfo.getKeyProperty();
            String method = this.getMethod(SqlMethod.INSERT_ONE);
            keyGenerator = TableInfoHelper.genKeyGenerator(method, tableInfo, this.builderAssistant);
        } else {
            keyGenerator = new NoKeyGenerator();
        }
        String script = this.initializeMethodScript(tableInfo);
        SqlSource source = this.languageDriver.createSqlSource(this.configuration, script, modelClass);
        return this.addInsertMappedStatement(
                mapperClass, modelClass, this.id, source, keyGenerator, keyProperty, keyColumn
        );
    }
}
