package com.chelvc.framework.jpush;

import java.lang.reflect.Field;
import java.security.GeneralSecurityException;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Arrays;
import java.util.Base64;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import javax.annotation.PostConstruct;
import javax.crypto.Cipher;

import cn.jiguang.common.ClientConfig;
import cn.jiguang.common.ServiceHelper;
import cn.jiguang.common.connection.ApacheHttpClient;
import cn.jiguang.common.resp.APIConnectionException;
import cn.jiguang.common.resp.APIRequestException;
import cn.jmessage.api.JMessageClient;
import cn.jmessage.api.common.model.RegisterInfo;
import cn.jmessage.api.user.UserInfoResult;
import cn.jmessage.api.user.UserStateListResult;
import cn.jmessage.api.user.UserStateResult;
import cn.jpush.api.JPushClient;
import cn.jpush.api.push.CIDResult;
import cn.jpush.api.push.PushClient;
import cn.jpush.api.push.PushResult;
import cn.jpush.api.push.model.BatchPushResult;
import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.notification.Notification;
import cn.jpush.api.report.ReceivedsResult;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.jpush.config.JPushProperties;
import com.chelvc.framework.jpush.context.JPushContextHolder;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.web.client.RestTemplate;

/**
 * 极光登录操作默认实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class DefaultJPushHandler implements JPushHandler {
    /**
     * 极光令牌验证地址
     */
    private static final String TOKEN2MOBILE_URL = "https://api.verification.jpush.cn/v1/web/loginTokenVerify";

    /**
     * 获取消息地址
     */
    private static final String GET_MESSAGE_URL =
            "https://report.im.jpush.cn/v2/users/%s/messages?count=%d&begin_time=%s&end_time=%s";

    /**
     * 获取汇总消息地址(首次)
     */
    private static final String GET_INITIAL_MESSAGE_URL =
            "https://report.im.jpush.cn/v2/messages?count=%d&begin_time=%s&end_time=%s";

    /**
     * 获取汇总消息地址(非首次)
     */
    private static final String GET_SUBSEQUENT_MESSAGE_URL = "https://report.im.jpush.cn/v2/messages?cursor=";

    /**
     * 获取历史消息地址
     */
    private static final String GET_HISTORY_MESSAGE_URL = "https://report.im.jpush.cn/v2/users/%s/messages?cursor=%s";

    private final RestTemplate restTemplate;
    private final JPushProperties properties;
    private PrivateKey privateKey;
    private JPushClient pushClient;
    private JMessageClient messageClient;

    @PostConstruct
    public void initialize() throws GeneralSecurityException {
        // 初始化登录认证配置
        JPushProperties.Login login = this.properties.getLogin();
        if (login != null && StringUtils.notEmpty(login.getSecret())) {
            // 初始化私钥
            byte[] secret = Base64.getDecoder().decode(login.getSecret());
            PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(secret);
            this.privateKey = KeyFactory.getInstance("RSA").generatePrivate(keySpec);
        }

        // 初始化消息推送配置
        JPushProperties.Push push = this.properties.getPush();
        if (push != null && StringUtils.notEmpty(push.getAppkey()) && StringUtils.notEmpty(push.getSecret())) {
            // 初始化推送客户端实例
            this.pushClient = new JPushClient(push.getSecret(), push.getAppkey()) {
                {
                    Field field = ObjectUtils.getField(JPushClient.class, "_pushClient");
                    ObjectUtils.setValue(this, field, new PushClient(push.getSecret(), push.getAppkey()) {
                        @Override
                        public CIDResult getCidList(int count, String type) throws APIConnectionException,
                                APIRequestException {
                            // 对批量推送CID排序，确保推送结果顺序与发送数据顺序一致
                            CIDResult result = super.getCidList(count, type);
                            if (result != null && !CollectionUtils.isEmpty(result.cidlist)) {
                                result.cidlist.sort(Comparator.naturalOrder());
                            }
                            return result;
                        }
                    });
                }
            };

            // 初始化消息客户端实例
            this.messageClient = new JMessageClient(push.getAppkey(), push.getSecret());
            this.messageClient.setHttpClient(new ApacheHttpClient(
                    ServiceHelper.getBasicAuthorization(push.getAppkey(), push.getSecret()),
                    null,
                    ClientConfig.getInstance()
            ));
        }
    }

    /**
     * 获取推送客户端实例
     *
     * @return 推送客户端实例
     */
    private JPushClient getPushClient() {
        return AssertUtils.nonnull(this.pushClient, () -> "JPush client has not been initialized");
    }

    /**
     * 获取消息客户端实例
     *
     * @return 消息客户端实例
     */
    private JMessageClient getMessageClient() {
        return AssertUtils.nonnull(this.messageClient, () -> "JMessage client has not been initialized");
    }

    /**
     * 获取聊天消息
     *
     * @param url 接口地址
     * @return 聊天消息
     */
    private ChatMessage getChatMessages(String url) {
        String appkey = ObjectUtils.ifNull(this.properties.getPush(), JPushProperties.Push::getAppkey);
        String secret = ObjectUtils.ifNull(this.properties.getPush(), JPushProperties.Push::getSecret);
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", ServiceHelper.getBasicAuthorization(appkey, secret));
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(headers);
        try {
            return this.restTemplate.exchange(url, HttpMethod.GET, entity, ChatMessage.class).getBody();
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return null;
        }
    }

    @Override
    public String token2mobile(@NonNull String token) {
        PrivateKey key = AssertUtils.nonnull(this.privateKey, () -> "Private key has not been initialized");

        // 根据令牌获取手机号密文
        Map<String, String> parameters = ImmutableMap.of("loginToken", token);
        String authorization = ObjectUtils.ifNull(this.properties.getLogin(), JPushProperties.Login::getAuthorization);
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", authorization);
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(parameters, headers);
        JPushMobile response;
        try {
            response = this.restTemplate.exchange(
                    TOKEN2MOBILE_URL, HttpMethod.POST, entity, JPushMobile.class
            ).getBody();
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return null;
        }
        if (response == null || response.getCode() == null || response.getCode() != 8000) {
            LoggingContextHolder.warn(log, response);
            return null;
        }

        // 解码手机号
        try {
            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.DECRYPT_MODE, key);
            return new String(cipher.doFinal(Base64.getDecoder().decode(response.getPhone())));
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean register(@NonNull RegisterInfo... registrations) {
        if (registrations.length == 0) {
            return false;
        }
        try {
            this.getMessageClient().registerUsers(registrations);
        } catch (Exception e) {
            int code;
            if (e instanceof APIRequestException
                    && ((code = ((APIRequestException) e).getErrorCode()) == 1011 || code == 899001)) {
                return true;
            }
            LoggingContextHolder.warn(log, e);
            return false;
        }
        return true;
    }

    @Override
    public boolean register(@NonNull Collection<RegisterInfo> registrations) {
        return !registrations.isEmpty() && this.register(registrations.toArray(new RegisterInfo[0]));
    }

    @Override
    public long push(@NonNull PushPayload payload) {
        // 针对生产环境IOS推送，需要设置apnsProduction为true
        if (JPushContextHolder.isProduction()) {
            payload.resetOptionsApnsProduction(true);
        }
        PushResult result;
        try {
            result = this.getPushClient().sendPush(payload);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return 0;
        }
        if (result != null && result.statusCode == 0) {
            return result.msg_id;
        }
        Integer code = result == null || result.error == null ? null : result.error.getCode();
        String message = result == null || result.error == null ? null : result.error.getMessage();
        LoggingContextHolder.warn(log, code, message);
        return 0;
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Message message, @NonNull String... aliases) {
        return this.push(JPushContextHolder.payload(platform, message, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Message message, @NonNull Collection<String> aliases) {
        return this.push(JPushContextHolder.payload(platform, message, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Notification notification, @NonNull String... aliases) {
        return this.push(JPushContextHolder.payload(platform, notification, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull Notification notification,
                     @NonNull Collection<String> aliases) {
        return this.push(JPushContextHolder.payload(platform, notification, aliases));
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull String... aliases) {
        return this.push(platform, title, content, Collections.emptyMap(), aliases);
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull Collection<String> aliases) {
        return this.push(platform, title, content, Collections.emptyMap(), aliases);
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull Map<String, String> extras, @NonNull String... aliases) {
        Notification notification = JPushContextHolder.notification(platform, title, content, extras);
        return this.push(platform, notification, aliases);
    }

    @Override
    public long push(@NonNull Platform platform, @NonNull String title, @NonNull String content,
                     @NonNull Map<String, String> extras, @NonNull Collection<String> aliases) {
        Notification notification = JPushContextHolder.notification(platform, title, content, extras);
        return this.push(platform, notification, aliases);
    }

    @Override
    public BatchPushResult push(@NonNull PushPayload... payloads) {
        AssertUtils.check(payloads.length > 0, () -> "payloads must not be empty");
        return this.push(Arrays.asList(payloads));
    }

    @Override
    public BatchPushResult push(@NonNull List<PushPayload> payloads) {
        AssertUtils.nonempty(payloads, () -> "payloads must not be empty");
        // 针对生产环境IOS推送，需要设置apnsProduction为true
        if (JPushContextHolder.isProduction()) {
            payloads.forEach(payload -> payload.resetOptionsApnsProduction(true));
        }

        // 批量推送消息
        try {
            return this.getPushClient().batchSendPushByAlias(payloads);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return null;
        }
    }

    /**
     * 获取消息投递结果
     *
     * @param ids 消息ID（多个ID之间使用","号隔开，最多最多支持 100 个消息ID）
     * @return 投递结果列表
     */
    private List<ReceivedsResult.Received> getDeliveries(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        ReceivedsResult result;
        try {
            result = this.getPushClient().getReceivedsDetail(ids);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return Collections.emptyList();
        }
        return ObjectUtils.ifNull(result.received_list, Collections::emptyList);
    }

    @Override
    public ReceivedsResult.Received getDelivery(@NonNull Long id) {
        List<ReceivedsResult.Received> deliveries = this.getDeliveries(String.valueOf(id));
        return CollectionUtils.isEmpty(deliveries) ? null : deliveries.get(0);
    }

    @Override
    public List<ReceivedsResult.Received> getDeliveries(@NonNull Long... ids) {
        return ids.length == 0 ? Collections.emptyList() : this.getDeliveries(Arrays.asList(ids));
    }

    @Override
    public List<ReceivedsResult.Received> getDeliveries(@NonNull Collection<Long> ids) {
        return this.getDeliveries(
                ids.stream().filter(Objects::nonNull).map(Object::toString).collect(Collectors.joining(","))
        );
    }

    @Override
    public UserInfoResult getUserInfo(@NonNull String username) {
        try {
            return this.getMessageClient().getUserInfo(username);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return null;
        }
    }

    @Override
    public UserStateResult getUserState(@NonNull String username) {
        try {
            return this.getMessageClient().getUserState(username);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return null;
        }
    }

    @Override
    public UserStateListResult[] getUserStates(@NonNull String... usernames) {
        if (usernames.length == 0) {
            return new UserStateListResult[0];
        }
        try {
            return this.getMessageClient().getUsersState(usernames);
        } catch (Exception e) {
            LoggingContextHolder.warn(log, e);
            return new UserStateListResult[0];
        }
    }

    @Override
    public ChatMessage getInitialMessages(@NonNull Period period, int count) {
        return this.getChatMessages(String.format(
                GET_INITIAL_MESSAGE_URL, count,
                DateUtils.format(period.getBegin(), true), DateUtils.format(period.getEnd(), true)
        ));
    }

    @Override
    public ChatMessage getSubsequentMessages(@NonNull String cursor) {
        return this.getChatMessages(GET_SUBSEQUENT_MESSAGE_URL + cursor);
    }

    @Override
    public ChatMessage getUserMessages(@NonNull String username, @NonNull Period period, int count) {
        return this.getChatMessages(String.format(
                GET_MESSAGE_URL, username, count, DateUtils.format(period.getBegin()), DateUtils.format(period.getEnd())
        ));
    }

    @Override
    public ChatMessage getUserHistoryMessages(@NonNull String username, @NonNull String cursor) {
        return this.getChatMessages(String.format(GET_HISTORY_MESSAGE_URL, username, cursor));
    }
}
