package com.chelvc.framework.jpush.context;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.Options;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.audience.Audience;
import cn.jpush.api.push.model.notification.IosAlert;
import cn.jpush.api.push.model.notification.IosNotification;
import cn.jpush.api.push.model.notification.Notification;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.jpush.config.JPushProperties;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.google.gson.JsonObject;
import lombok.NonNull;
import org.springframework.util.CollectionUtils;

/**
 * JPush上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class JPushContextHolder {
    /**
     * 安卓通知参数选项
     */
    private static Options ANDROID_OPTIONS;

    /**
     * 配置属性实例
     */
    private static JPushProperties PROPERTIES;

    private JPushContextHolder() {
    }

    /**
     * 获取配置属性
     *
     * @return 配置属性实例
     */
    public static JPushProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (JPushProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(JPushProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 判断是否是生产环境
     *
     * @return true/false
     */
    public static boolean isProduction() {
        return getProperties().isProduction();
    }

    /**
     * 将对象转换成别名
     *
     * @param objects 对象数组
     * @return 别名数组
     */
    public static Audience alias(Object... objects) {
        if (objects == null || objects.length == 0) {
            return null;
        }
        List<String> aliases = Lists.newArrayListWithCapacity(objects.length);
        for (Object object : objects) {
            if (object != null) {
                aliases.add(String.valueOf(object));
            }
        }
        return aliases.isEmpty() ? null : Audience.alias(aliases);
    }

    /**
     * 将对象转换成别名
     *
     * @param objects 对象集合
     * @return 别名数组
     */
    public static Audience alias(Collection<?> objects) {
        if (CollectionUtils.isEmpty(objects)) {
            return null;
        }
        List<String> aliases = objects.stream().filter(Objects::nonNull).map(String::valueOf)
                .collect(Collectors.toList());
        return aliases.isEmpty() ? null : Audience.alias(aliases);
    }

    /**
     * 平台信息适配
     *
     * @param platform 平台信息
     * @return 平台信息
     */
    public static cn.jpush.api.push.model.Platform adapter(Platform platform) {
        if (platform == Platform.IOS) {
            return cn.jpush.api.push.model.Platform.ios();
        } else if (platform == Platform.ANDROID) {
            return cn.jpush.api.push.model.Platform.android();
        }
        return null;
    }

    /**
     * 构建通知对象
     *
     * @param platform 平台信息
     * @param title    通知标题
     * @param content  通知内容
     * @param extras   扩展信息
     * @return 通知对象实例
     */
    public static Notification notification(@NonNull com.chelvc.framework.common.model.Platform platform,
                                            String title,
                                            String content, Map<String, String> extras) {
        if (platform == com.chelvc.framework.common.model.Platform.IOS) {
            return Notification.newBuilder().addPlatformNotification(IosNotification.newBuilder()
                    .addExtras(extras)
                    //设置ios标题
                    .setAlert(IosAlert.newBuilder().setTitleAndBody(title, null, content).build())
                    //如果要设置右侧大图，推送的时候携带 "mutable-content":true 说明是支持 iOS10 的 UNNotificationServiceExtension
                    .setMutableContent(true)
                    //如果不需要自定义声音，推送的时候，iOS 的 sound 值保持默认，传 default ，不要传空！
                    .setSound("default")
                    //设置角标自动+1
                    .incrBadge(1)
                    .build()).build();
        } else if (platform == com.chelvc.framework.common.model.Platform.ANDROID) {
            return Notification.android(content, title, extras);
        }
        throw new IllegalArgumentException("Not support platform: " + platform.name());
    }

    /**
     * 获取安卓通知可选参数
     *
     * @return 安卓通知参数选项
     */
    public static Options getAndroidNotifyOptions() {
        if (ANDROID_OPTIONS == null) {
            synchronized (Options.class) {
                if (ANDROID_OPTIONS == null) {
                    JPushProperties.Push push = getProperties().getPush();
                    ANDROID_OPTIONS = Options.newBuilder().setThirdPartyChannelV2(ImmutableMap.of(
                            "huawei", initializeAndroidChannel(push.getHuawei()),
                            "honor", initializeAndroidChannel(push.getHonor()),
                            "xiaomi", initializeAndroidChannel(push.getXiaomi()),
                            "oppo", initializeAndroidChannel(push.getOppo()),
                            "vivo", initializeAndroidChannel(push.getVivo())
                    )).build();
                }
            }
        }
        return ANDROID_OPTIONS;
    }

    /**
     * 初始化安卓渠道
     *
     * @param attribute 配置属性
     * @return 渠道信息
     */
    public static JsonObject initializeAndroidChannel(@NonNull JPushProperties.Attribute attribute) {
        JsonObject channel = new JsonObject();
        if (StringUtils.notEmpty(attribute.getChannel())) {
            channel.addProperty("channel_id", attribute.getChannel());
        }
        if (StringUtils.notEmpty(attribute.getImportance())) {
            channel.addProperty("importance", attribute.getImportance());
        }
        if (StringUtils.notEmpty(attribute.getDistribution())) {
            channel.addProperty("distribution", attribute.getDistribution());
        }
        if (Objects.nonNull(attribute.getClassification())) {
            channel.addProperty("classification", attribute.getClassification());
        }
        return channel;
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param aliases  用户别名数组
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Message message,
                                      @NonNull String... aliases) {
        return new PushPayload.Builder().setPlatform(adapter(platform)).setAudience(Audience.alias(aliases))
                .setMessage(message).build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform 目标平台
     * @param message  透传消息
     * @param aliases  用户别名集合
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Message message,
                                      @NonNull Collection<String> aliases) {
        return new PushPayload.Builder().setPlatform(adapter(platform)).setAudience(Audience.alias(aliases))
                .setMessage(message).build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param aliases      用户别名数组
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      @NonNull String... aliases) {
        PushPayload.Builder builder = new PushPayload.Builder().setPlatform(adapter(platform))
                .setAudience(Audience.alias(aliases)).setNotification(notification);
        if (platform == Platform.ANDROID) {
            builder.setOptions(getAndroidNotifyOptions());
        }
        return builder.build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param platform     目标平台
     * @param notification 推送通知
     * @param aliases      用户别名集合
     * @return 消息推送载体
     */
    public static PushPayload payload(@NonNull Platform platform, @NonNull Notification notification,
                                      @NonNull Collection<String> aliases) {
        PushPayload.Builder builder = new PushPayload.Builder().setPlatform(adapter(platform))
                .setAudience(Audience.alias(aliases)).setNotification(notification);
        if (platform == Platform.ANDROID) {
            builder.setOptions(getAndroidNotifyOptions());
        }
        return builder.build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param target   目标用户
     * @param platform 目标平台
     * @param message  透传消息
     * @return 消息推送载体
     */
    public static PushPayload target(@NonNull String target, @NonNull Platform platform, @NonNull Message message) {
        return new PushPayload.Builder().setPlatform(adapter(platform)).setMessage(message).setTarget(target).build();
    }

    /**
     * 构建消息推送载体对象
     *
     * @param target       目标用户
     * @param platform     目标平台
     * @param notification 推送通知
     * @return 消息推送载体
     */
    public static PushPayload target(@NonNull String target, @NonNull Platform platform,
                                     @NonNull Notification notification) {
        PushPayload.Builder builder = new PushPayload.Builder().setTarget(target).setPlatform(adapter(platform))
                .setNotification(notification);
        if (platform == Platform.ANDROID) {
            builder.setOptions(getAndroidNotifyOptions());
        }
        return builder.build();
    }
}
