package com.chelvc.framework.kafka.config;

import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Objects;
import java.util.Set;
import java.util.regex.Pattern;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.kafka.context.KafkaContextHolder;
import com.chelvc.framework.kafka.interceptor.KafkaSessionInterceptor;
import lombok.RequiredArgsConstructor;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.kafka.ConcurrentKafkaListenerContainerFactoryConfigurer;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.core.env.Environment;
import org.springframework.kafka.annotation.KafkaListenerConfigurer;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerConfigUtils;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerEndpoint;
import org.springframework.kafka.config.KafkaListenerEndpointRegistrar;
import org.springframework.kafka.config.KafkaListenerEndpointRegistry;
import org.springframework.kafka.config.MethodKafkaListenerEndpoint;
import org.springframework.kafka.core.ConsumerFactory;
import org.springframework.kafka.listener.MessageListenerContainer;
import org.springframework.kafka.listener.RecordInterceptor;
import org.springframework.kafka.support.converter.BatchMessagingMessageConverter;
import org.springframework.kafka.support.converter.MessagingMessageConverter;
import org.springframework.kafka.support.converter.RecordMessageConverter;

/**
 * Kafka配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class KafkaConfigurer implements KafkaListenerConfigurer {
    private final ApplicationContext applicationContext;

    @Bean
    public RecordMessageConverter recordMessageConverter() {
        return new MessagingMessageConverter() {
            @Override
            protected Object extractAndConvertValue(ConsumerRecord<?, ?> record, Type type) {
                return KafkaContextHolder.deserialize(record, type);
            }
        };
    }

    @Bean
    public RecordInterceptor<Object, Object> recordInterceptor() {
        return new KafkaSessionInterceptor();
    }

    @Bean
    public KafkaListenerContainerFactory<?> kafkaListenerContainerFactory(
            ConcurrentKafkaListenerContainerFactoryConfigurer configurer,
            ConsumerFactory<Object, Object> consumerFactory) {
        ConcurrentKafkaListenerContainerFactory<Object, Object> containerFactory =
                new ConcurrentKafkaListenerContainerFactory<>();
        configurer.configure(containerFactory, consumerFactory);
        containerFactory.setRecordInterceptor(this.recordInterceptor());
        containerFactory.getContainerProperties().setDeliveryAttemptHeader(true);
        return containerFactory;
    }

    @Bean
    public KafkaListenerContainerFactory<?> batchListenerContainerFactory(
            ConcurrentKafkaListenerContainerFactoryConfigurer configurer,
            ConsumerFactory<Object, Object> consumerFactory) {
        ConcurrentKafkaListenerContainerFactory<Object, Object> containerFactory =
                new ConcurrentKafkaListenerContainerFactory<>();
        configurer.configure(containerFactory, consumerFactory);
        containerFactory.setBatchListener(true);
        containerFactory.setRecordInterceptor(this.recordInterceptor());
        containerFactory.getContainerProperties().setDeliveryAttemptHeader(true);
        containerFactory.setMessageConverter(new BatchMessagingMessageConverter(this.recordMessageConverter()));
        return containerFactory;
    }

    @Override
    public void configureKafkaListeners(KafkaListenerEndpointRegistrar registrar) {
        registrar.setEndpointRegistry(new KafkaListenerEndpointRegistry() {
            private final KafkaListenerEndpointRegistry proxy = applicationContext.getBean(
                    KafkaListenerConfigUtils.KAFKA_LISTENER_ENDPOINT_REGISTRY_BEAN_NAME,
                    KafkaListenerEndpointRegistry.class
            );

            /**
             * 环境隔离
             *
             * @param original 原始标记
             * @return 环境隔离标记
             */
            private String isolate(String original) {
                Environment environment = ApplicationContextHolder.getEnvironment();
                return KafkaContextHolder.isolate(environment.resolvePlaceholders(original));
            }

            /**
             * 隔离消息监听器端点环境
             *
             * @param endpoint 消息监听器端点实例
             */
            private void isolateListenerEndpointEnvironment(MethodKafkaListenerEndpoint<?, ?> endpoint) {
                if (StringUtils.notEmpty(endpoint.getGroup())) {
                    endpoint.setGroup(this.isolate(endpoint.getGroup()));
                }
                if (StringUtils.notEmpty(endpoint.getGroupId())) {
                    endpoint.setGroupId(this.isolate(endpoint.getGroupId()));
                }
                if (Objects.nonNull(endpoint.getTopicPattern())) {
                    String pattern = endpoint.getTopicPattern().pattern();
                    if (StringUtils.notEmpty(pattern)) {
                        endpoint.setTopicPattern(Pattern.compile(this.isolate(pattern)));
                    }
                }
                Collection<String> topics = endpoint.getTopics();
                if (ObjectUtils.notEmpty(topics)) {
                    endpoint.setTopics(topics.stream().map(this::isolate).toArray(String[]::new));
                }
            }

            @Override
            public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
                this.proxy.setApplicationContext(applicationContext);
            }

            @Override
            public MessageListenerContainer getListenerContainer(String id) {
                return this.proxy.getListenerContainer(id);
            }

            @Override
            public Set<String> getListenerContainerIds() {
                return this.proxy.getListenerContainerIds();
            }

            @Override
            public Collection<MessageListenerContainer> getListenerContainers() {
                return this.proxy.getListenerContainers();
            }

            @Override
            public Collection<MessageListenerContainer> getAllListenerContainers() {
                return this.proxy.getAllListenerContainers();
            }

            @Override
            public void registerListenerContainer(KafkaListenerEndpoint endpoint,
                                                  KafkaListenerContainerFactory<?> factory) {
                this.registerListenerContainer(endpoint, factory, false);
            }

            @Override
            public void registerListenerContainer(KafkaListenerEndpoint endpoint,
                                                  KafkaListenerContainerFactory<?> factory, boolean startImmediately) {
                this.isolateListenerEndpointEnvironment((MethodKafkaListenerEndpoint<?, ?>) endpoint);
                this.proxy.registerListenerContainer(endpoint, factory, startImmediately);
            }

            @Override
            public void destroy() {
                this.proxy.destroy();
            }

            @Override
            public int getPhase() {
                return this.proxy.getPhase();
            }

            @Override
            public boolean isAutoStartup() {
                return this.proxy.isAutoStartup();
            }

            @Override
            public void start() {
                this.proxy.start();
            }

            @Override
            public void stop() {
                this.proxy.stop();
            }

            @Override
            public void stop(Runnable callback) {
                this.proxy.stop(callback);
            }

            @Override
            public boolean isRunning() {
                return this.proxy.isRunning();
            }

            @Override
            public void onApplicationEvent(ContextRefreshedEvent event) {
                this.proxy.onApplicationEvent(event);
            }
        });
    }
}
