package com.chelvc.framework.location.config;

import java.util.List;
import java.util.stream.Collectors;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.location.LocationHandler;
import com.chelvc.framework.location.support.CacheableLocationHandler;
import com.chelvc.framework.location.support.DelegatingLocationHandler;
import com.chelvc.framework.location.support.JuheLocationHandler;
import com.chelvc.framework.location.support.TencentLocationHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.client.RestTemplate;

/**
 * 地址定位配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class LocationConfigurer {
    private final RestTemplate restTemplate;
    private final LocationProperties properties;

    /**
     * 初始化地址定位处理器实例
     *
     * @param client 客户端配置
     * @return 地址定位处理器实例
     */
    private LocationHandler initializeLocationHandler(LocationProperties.Client client) {
        LocationProperties.Channel channel = client.getChannel();
        if (channel == LocationProperties.Channel.JUHE) {
            return new JuheLocationHandler(client.getKey(), this.restTemplate);
        } else if (channel == LocationProperties.Channel.TENCENT) {
            return new TencentLocationHandler(client.getKey(), this.restTemplate);
        }
        throw new UnsupportedOperationException(String.valueOf(channel));
    }

    @Bean
    public LocationHandler locationHandler() {
        List<LocationHandler> handlers = this.properties.getClients().stream().map(this::initializeLocationHandler)
                .collect(Collectors.toList());
        if (ObjectUtils.isEmpty(handlers)) {
            return null;
        }
        LocationHandler delegating = new DelegatingLocationHandler(handlers);
        if (this.properties.getCaching().isEnabled()) {
            return new CacheableLocationHandler(delegating, this.properties.getCaching().getExpiration());
        }
        return delegating;
    }
}
