package com.chelvc.framework.location.support;

import java.util.List;
import java.util.function.Function;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 地址定位处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingLocationHandler implements LocationHandler {
    private final List<LocationHandler> handlers;

    public DelegatingLocationHandler(@NonNull List<LocationHandler> handlers) {
        this.handlers = AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
    }

    /**
     * 遍历所有地址定位处理器直到定位成功
     *
     * @param callback 地址定位处理回调函数
     * @return 地址信息
     */
    private Address execute(Function<LocationHandler, Address> callback) {
        Exception unsupported = null;
        for (LocationHandler handler : this.handlers) {
            try {
                Address address = callback.apply(handler);
                if (address != null) {
                    return address;
                }
            } catch (Exception e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = e;
                } else {
                    log.warn("Address locate failed: {}", e.getMessage());
                }
            }
        }
        if (unsupported != null) {
            log.warn("Address locate failed: {}", unsupported.getMessage());
        }
        return null;
    }

    @Override
    public Address ip2address(@NonNull String ip) {
        return this.execute(handler -> handler.ip2address(ip));
    }

    @Override
    public Address mobile2address(@NonNull String mobile) {
        return this.execute(handler -> handler.mobile2address(mobile));
    }

    @Override
    public Address location2address(@NonNull String location) {
        return this.execute(handler -> handler.location2address(location));
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        return this.execute(handler -> handler.location2address(longitude, latitude));
    }
}
