package com.chelvc.framework.location.support;

import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.Area;
import com.chelvc.framework.location.LocationHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.client.RestTemplate;

/**
 * 腾讯地址定位处理实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class TencentLocationHandler implements LocationHandler {
    /**
     * IP定位接口地址
     */
    private static final String IP2ADDRESS_URL = "https://apis.map.qq.com/ws/location/v1/ip?ip=%s&key=%s";

    /**
     * 经纬度定位接口地址
     */
    private static final String LOCATION2ADDRESS_URL = "https://apis.map.qq.com/ws/geocoder/v1/?location=%s&key=%s";

    private final String key;
    private final RestTemplate restTemplate;

    public TencentLocationHandler(@NonNull String key, @NonNull RestTemplate restTemplate) {
        this.key = key;
        this.restTemplate = restTemplate;
    }

    /**
     * 执行定位请求
     *
     * @param url       定位接口地址
     * @param parameter 定位参数
     * @return 地址信息
     */
    private Address execute(String url, String parameter) {
        boolean debug = log.isDebugEnabled();
        url = String.format(url, parameter, this.key);
        if (debug) {
            log.debug("Tencent location request: {}", url);
        }
        TencentLocationResponse response;
        try {
            response = this.restTemplate.getForObject(url, TencentLocationResponse.class);
        } catch (Exception e) {
            log.warn("Tencent locate failed: {}", e.getMessage());
            return null;
        }
        if (debug) {
            log.debug("Tencent locate response: {}", response);
        }
        if (response == null || response.getStatus() == null || response.getStatus() != 0) {
            log.warn("Tencent locate failed: {}", response);
            return null;
        }
        TencentLocationResponse.Position position = response.getResult().getPosition();
        if (!Region.isRegion(position.getAdcode())) {
            log.warn("Invalid tencent locate adcode: {}", position.getAdcode());
            return null;
        }

        // 构建地址信息
        Region region = Objects.requireNonNull(Region.of(position.getAdcode()));
        TencentLocationResponse.Location location = response.getResult().getLocation();
        Double longitude = ObjectUtils.ifNull(location, TencentLocationResponse.Location::getLongitude);
        Double latitude = ObjectUtils.ifNull(location, TencentLocationResponse.Location::getLatitude);
        Address address = Address.builder().id(region.getCode()).longitude(longitude).latitude(latitude)
                .address(response.getResult().getAddress()).build();
        address.setName(Stream.of(position.getProvince(), position.getCity(), position.getDistrict())
                .filter(StringUtils::notEmpty).collect(Collectors.joining("·")
                ));
        address.setProvince(Area.builder().id(region.getProvince().getCode()).name(position.getProvince()).build());
        Region city = region.getCity();
        if (city != null) {
            address.setCity(Area.builder().id(city.getCode()).name(position.getCity()).build());
        }
        if (region.getType() == Region.Type.DISTRICT) {
            address.setDistrict(Area.builder().id(region.getCode()).name(position.getDistrict()).build());
        }
        return address;
    }

    @Override
    public Address ip2address(@NonNull String ip) {
        return this.execute(IP2ADDRESS_URL, ip);
    }

    @Override
    public Address mobile2address(String mobile) {
        throw new UnsupportedOperationException("Not support method");
    }

    @Override
    public Address location2address(@NonNull String location) {
        return this.execute(LOCATION2ADDRESS_URL, location);
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        return this.location2address(latitude + "," + longitude);
    }
}
