package com.chelvc.framework.oauth.context;

import java.text.ParseException;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.function.Adapter;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Sets;
import com.nimbusds.jwt.JWT;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.JWTParser;
import lombok.NonNull;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.server.resource.web.BearerTokenResolver;
import org.springframework.security.oauth2.server.resource.web.DefaultBearerTokenResolver;

/**
 * OAuth上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class OAuthContextHolder {
    /**
     * Jwt唯一标识
     */
    public static final String JWT_CLAIM_JTI = "jti";

    /**
     * Jwt应用范围
     */
    public static final String JWT_CLAIM_SCOPE = "scope";

    /**
     * Jwt是否匿名
     */
    public static final String JWT_CLAIM_ANONYMOUS = "anonymous";

    /**
     * Jwt用户账号标识
     */
    public static final String JWT_CLAIM_USER_NAME = "user_name";

    /**
     * Jwt用户权限标识
     */
    public static final String JWT_CLAIM_AUTHORITIES = "authorities";

    /**
     * Bearer令牌解析器
     */
    private static final BearerTokenResolver BEARER_TOKEN_RESOLVER = new DefaultBearerTokenResolver();

    private OAuthContextHolder() {
    }

    /**
     * 令牌转换
     *
     * @param token 令牌信息
     * @return JWT对象实例
     */
    public static JWT parse(String token) {
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        try {
            return JWTParser.parse(token);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 构建访问令牌标识
     *
     * @param id 主体标识
     * @return 令牌标识
     */
    public static String key(@NonNull Object id) {
        return "token:" + SessionContextHolder.getTerminal() + ":" + id;
    }

    /**
     * 获取请求令牌
     *
     * @param request Http请求对象
     * @return 令牌信息
     */
    public static String getToken(HttpServletRequest request) {
        return StringUtils.ifEmpty(ObjectUtils.ifNull(request, BEARER_TOKEN_RESOLVER::resolve), (String) null);
    }

    /**
     * 获取Jwt属性集合
     *
     * @param jwt JWT对象实例
     * @return Jwt属性集合
     */
    public static JWTClaimsSet getJwtClaims(JWT jwt) {
        if (Objects.isNull(jwt)) {
            return null;
        }
        try {
            return jwt.getJWTClaimsSet();
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取Jwt属性值
     *
     * @param jwt      JWT对象实例
     * @param function 属性获取方法
     * @param <T>      属性值类型
     * @return 属性值
     */
    public static <T> T getJwtClaim(JWT jwt, @NonNull Adapter<JWTClaimsSet, T> function) {
        JWTClaimsSet claims = ObjectUtils.ifNull(jwt, OAuthContextHolder::getJwtClaims);
        if (claims == null) {
            return null;
        }
        try {
            return function.apply(claims);
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            }
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_USER_NAME), Long::parseLong);
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(JWT jwt) {
        return getJwtClaim(jwt, claims -> StringUtils.ifEmpty(
                claims.getStringClaim(JWT_CLAIM_USER_NAME), Long::parseLong
        ));
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_JTI), (String) null);
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> claims.getStringClaim(JWT_CLAIM_JTI)), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param jwt Jwt对象
     * @return 应用范围
     */
    public static String getScope(Jwt jwt) {
        List<String> scopes = jwt == null ? null : jwt.getClaimAsStringList(JWT_CLAIM_SCOPE);
        return StringUtils.ifEmpty(ObjectUtils.isEmpty(scopes) ? null : scopes.get(0), (String) null);
    }

    /**
     * 获取应用范围
     *
     * @param jwt JWT对象
     * @return 应用范围
     */
    public static String getScope(JWT jwt) {
        return StringUtils.ifEmpty(getJwtClaim(jwt, claims -> {
            String[] scopes = claims.getStringArrayClaim(JWT_CLAIM_SCOPE);
            return ObjectUtils.isEmpty(scopes) ? null : scopes[0];
        }), (String) null);
    }

    /**
     * 是否是匿名用户
     *
     * @param jwt Jwt对象
     * @return true/false
     */
    public static boolean isAnonymous(Jwt jwt) {
        Boolean anonymous = jwt == null ? null : jwt.getClaimAsBoolean(JWT_CLAIM_ANONYMOUS);
        return anonymous == null || Boolean.TRUE.equals(anonymous);
    }

    /**
     * 是否是匿名用户
     *
     * @param jwt JWT对象
     * @return true/false
     */
    public static boolean isAnonymous(JWT jwt) {
        Boolean anonymous = getJwtClaim(jwt, claims -> claims.getBooleanClaim(JWT_CLAIM_ANONYMOUS));
        return anonymous == null || Boolean.TRUE.equals(anonymous);
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(JWT_CLAIM_AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(JWT jwt) {
        String[] authorities = getJwtClaim(jwt, claims -> claims.getStringArrayClaim(JWT_CLAIM_AUTHORITIES));
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }
}
