package com.chelvc.framework.redis.config;

import java.lang.reflect.Type;
import java.time.Duration;
import java.util.concurrent.atomic.AtomicInteger;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.redis.annotation.RedisMQConsumer;
import com.chelvc.framework.redis.stream.DefaultRedisMQListenerContainer;
import com.chelvc.framework.redis.stream.RedisMQListener;
import com.chelvc.framework.redis.stream.RedisMQListenerContainer;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.jsontype.impl.LaissezFaireSubTypeValidator;
import io.lettuce.core.event.EventBus;
import io.lettuce.core.resource.ClientResources;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.data.redis.RedisAutoConfiguration;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.core.task.TaskExecutor;
import org.springframework.data.redis.cache.RedisCacheConfiguration;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.data.redis.cache.RedisCacheWriter;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializationContext;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Component;

/**
 * Redis缓存配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@EnableCaching
@AutoConfigureAfter(RedisAutoConfiguration.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RedisConfigurer extends CachingConfigurerSupport implements ApplicationRunner {
    private final TaskExecutor executor;
    private final ClientResources clientResources;
    private final LettuceConnectionFactory connectionFactory;
    private final AtomicInteger counter = new AtomicInteger(0);

    /**
     * 注册消息监听器容器
     *
     * @param listener 消息监听器实例
     * @param <T>      消息对象类型
     */
    @SuppressWarnings("unchecked")
    private <T> void registerListenerContainer(RedisMQListener<T> listener) {
        Class<?> clazz = AopProxyUtils.ultimateTargetClass(listener);
        RedisMQConsumer annotation = AssertUtils.nonnull(
                clazz.getAnnotation(RedisMQConsumer.class),
                () -> "@RedisMQConsumer annotation is missing: " + clazz.getName()
        );
        Type type = ObjectUtils.lookupSuperclassParameterized(clazz, RedisMQListener.class);
        RedisMQListenerContainer<T> container = ApplicationContextHolder.register(
                this.counter.incrementAndGet(), RedisMQListenerContainer.class, DefaultRedisMQListenerContainer::new
        );
        try {
            container.initialize(type, listener, annotation);
        } catch (Exception e) {
            log.error("RedisMQListenerContainer initialize failed", e);
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public void run(ApplicationArguments args) throws Exception {
        // Redis事件配置
        EventBus bus = this.clientResources.eventBus();
        ApplicationContextHolder.getBeans(RedisEventConfigurerAdapter.class).forEach(
                (key, adapter) -> adapter.configure(bus)
        );

        // 注册消息监听器
        ApplicationContextHolder.getBeans(RedisMQListener.class).forEach(
                (key, listener) -> this.registerListenerContainer(listener)
        );
    }

    @Bean
    public KeyGenerator keyGenerator() {
        return (target, method, params) -> {
            StringBuilder builder = new StringBuilder();
            builder.append(target.getClass().getName());
            builder.append(method.getName());
            builder.append("&");
            for (Object param : params) {
                if (param != null) {
                    builder.append(param.getClass().getName());
                    builder.append("&");
                    builder.append(JacksonContextHolder.serialize(param));
                    builder.append("&");
                }
            }
            return DigestUtils.sha256Hex(builder.toString());
        };
    }

    @Bean
    public CacheManager cacheManager() {
        RedisCacheWriter writer = RedisCacheWriter.nonLockingRedisCacheWriter(this.connectionFactory);
        RedisSerializationContext.SerializationPair<?> serialization =
                RedisSerializationContext.SerializationPair.fromSerializer(this.jackson2JsonRedisSerializer());
        RedisCacheConfiguration redisCacheConfiguration = RedisCacheConfiguration.defaultCacheConfig()
                .disableCachingNullValues().entryTtl(Duration.ofMinutes(10)).serializeValuesWith(serialization);
        return RedisCacheManager.builder(writer).cacheDefaults(redisCacheConfiguration).build();
    }

    @Bean
    @Primary
    public RedisMessageListenerContainer redisMessageListenerContainer(RedisProperties properties) {
        RedisProperties.Listener config = properties.getListener();
        RedisMessageListenerContainer container = new RedisMessageListenerContainer();
        container.setTaskExecutor(this.executor);
        container.setConnectionFactory(this.connectionFactory);
        container.setRecoveryInterval(config.getRecoveryInterval());
        container.setMaxSubscriptionRegistrationWaitingTime(config.getMaxSubscriptionRegistrationWaitingTime());
        return container;
    }

    @Bean
    public RedisTemplate<String, Object> redisTemplate() {
        RedisTemplate<String, Object> template = new RedisTemplate<>();
        template.setConnectionFactory(this.connectionFactory);
        template.setKeySerializer(RedisSerializer.string());
        template.setValueSerializer(this.jackson2JsonRedisSerializer());
        template.setHashKeySerializer(RedisSerializer.string());
        template.setHashValueSerializer(this.jackson2JsonRedisSerializer());
        template.afterPropertiesSet();
        return template;
    }

    @Bean
    public Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer() {
        // objectMapper for redis serialization
        ObjectMapper mapper = new ObjectMapper();
        mapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
        mapper.setSerializationInclusion(JsonInclude.Include.ALWAYS);
        mapper.activateDefaultTyping(LaissezFaireSubTypeValidator.instance, ObjectMapper.DefaultTyping.NON_FINAL);
        mapper.configure(MapperFeature.USE_ANNOTATIONS, true);
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        // jackson2JsonRedisSerializer
        Jackson2JsonRedisSerializer<Object> serializer = new Jackson2JsonRedisSerializer<>(Object.class);
        serializer.setObjectMapper(mapper);
        return serializer;
    }
}
