package com.chelvc.framework.redis.context;

import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.ThreadLocalRandom;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Supplier;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.util.IdentityUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.data.redis.RedisProperties;
import org.springframework.data.redis.RedisSystemException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.RedisStreamCommands;
import org.springframework.data.redis.connection.RedisStringCommands;
import org.springframework.data.redis.connection.ReturnType;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.stream.ByteRecord;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.connection.stream.RecordId;
import org.springframework.data.redis.connection.stream.StreamInfo;
import org.springframework.data.redis.connection.stream.StreamOffset;
import org.springframework.data.redis.core.RedisConnectionUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.DigestUtils;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.data.redis.core.types.Expiration;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.stream.StreamListener;
import org.springframework.data.redis.stream.StreamMessageListenerContainer;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ErrorHandler;

/**
 * Redis上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class RedisContextHolder {
    /**
     * 消息ID标识
     */
    private static final String ID = "id";

    /**
     * 消息主题标识
     */
    private static final String TOPIC = "topic";

    /**
     * 消息载体标识
     */
    private static final String PAYLOAD = "payload";

    /**
     * 延时时间标识
     */
    private static final String DELAYING = "delaying";

    /**
     * 默认锁超时时间（秒）
     */
    private static final long DEFAULT_LOCK_TIMEOUT = 60;

    /**
     * 随机串Rdids Key前缀
     */
    private static final String RANDOM_REDIS_PREFIX = "random:";

    /**
     * 序列号Rdids Key前缀
     */
    private static final String SEQUENCE_REDIS_PREFIX = "sequence:";

    /**
     * 雪花算法序列标识
     */
    private static final String SNOWFLAKE_SEQUENCE_KEY = "snowflake:sequence";

    /**
     * 唯一标识日期格式化处理器
     */
    private static final DateTimeFormatter IDENTITY_DATETIME_FORMATTER = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");

    /**
     * Redis解锁脚本（原子操作）
     */
    private static final String UNLOCK_SCRIPT =
            "if redis.call(\"get\",KEYS[1]) == ARGV[1] then return redis.call(\"del\",KEYS[1]) else return 0 end";

    /**
     * Redis解锁脚本SHA（原子操作）
     */
    private static final String UNLOCK_SCRIPT_SHA = DigestUtils.sha1DigestAsHex(UNLOCK_SCRIPT);

    /**
     * Redis获取脚本（如果值不存在则初始化）
     */
    private static final RedisScript<Object> GET_WITH_INITIAL_SCRIPT = new DefaultRedisScript<>(
            "local value = redis.call('GET', KEYS[1]) if value then return value " +
                    "else redis.call('SET', KEYS[1], ARGV[1]) return ARGV[1] end", Object.class
    );

    /**
     * Redis获取脚本（如果值不存在则根据过期时间初始化）
     */
    private static final RedisScript<Object> GET_WITH_INITIAL_DURATION_SCRIPT = new DefaultRedisScript<>(
            "local value = redis.call('GET', KEYS[1]) if value then return value " +
                    "else redis.call('SETEX', KEYS[1], ARGV[2], ARGV[1]) return ARGV[1] end", Object.class
    );

    /**
     * Redis自增1脚本（如果值不存在则初始化）
     */
    private static final RedisScript<Long> INCR_WITH_INITIAL_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then return redis.call('INCR', KEYS[1]) " +
                    "else redis.call('SET', KEYS[1], ARGV[1]) return redis.call('INCR', KEYS[1]) end", Long.class
    );

    /**
     * Redis自增指定增量值脚本（如果值不存在则初始化）
     */
    private static final RedisScript<Long> INCRBY_WITH_INITIAL_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then " +
                    "return redis.call('INCRBY', KEYS[1], ARGV[1]) " +
                    "else redis.call('SET', KEYS[1], ARGV[2]) " +
                    "return redis.call('INCRBY', KEYS[1], ARGV[1]) end", Long.class
    );

    /**
     * Redis自增脚本（如果值不存在则根据过期时间初始化）
     */
    private static final RedisScript<Long> INCR_WITH_INITIAL_DURATION_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then " +
                    "return redis.call('INCR', KEYS[1]) " +
                    "else redis.call('SETEX', KEYS[1], ARGV[2], ARGV[1]) " +
                    "return redis.call('INCR', KEYS[1]) end", Long.class
    );

    /**
     * Redis自增指定增量值脚本（如果值不存在则根据过期时间初始化）
     */
    private static final RedisScript<Long> INCRBY_WITH_INITIAL_DURATION_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then " +
                    "return redis.call('INCRBY', KEYS[1], ARGV[1]) " +
                    "else redis.call('SETEX', KEYS[1], ARGV[3], ARGV[2]) " +
                    "return redis.call('INCRBY', KEYS[1], ARGV[1]) end", Long.class
    );

    /**
     * Redis自减1脚本（如果值不存在则初始化）
     */
    private static final RedisScript<Long> DECR_WITH_INITIAL_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then return redis.call('DECR', KEYS[1]) " +
                    "else redis.call('SET', KEYS[1], ARGV[1]) return redis.call('DECR', KEYS[1]) end", Long.class
    );

    /**
     * Redis自减指定减量值脚本（如果值不存在则初始化）
     */
    private static final RedisScript<Long> DECRBY_WITH_INITIAL_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then " +
                    "return redis.call('DECRBY', KEYS[1], ARGV[1]) " +
                    "else redis.call('SET', KEYS[1], ARGV[2]) " +
                    "return redis.call('DECRBY', KEYS[1], ARGV[1]) end", Long.class
    );

    /**
     * Redis自减脚本（如果值不存在则根据过期时间初始化）
     */
    private static final RedisScript<Long> DECR_WITH_INITIAL_DURATION_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then " +
                    "return redis.call('DECR', KEYS[1]) " +
                    "else redis.call('SETEX', KEYS[1], ARGV[2], ARGV[1]) " +
                    "return redis.call('DECR', KEYS[1]) end", Long.class
    );

    /**
     * Redis自减指定减量值脚本（如果值不存在则根据过期时间初始化）
     */
    private static final RedisScript<Long> DECRBY_WITH_INITIAL_DURATION_SCRIPT = new DefaultRedisScript<>(
            "if redis.call('EXISTS', KEYS[1]) == 1 then " +
                    "return redis.call('DECRBY', KEYS[1], ARGV[1]) " +
                    "else redis.call('SETEX', KEYS[1], ARGV[3], ARGV[2]) " +
                    "return redis.call('DECRBY', KEYS[1], ARGV[1]) end", Long.class
    );

    /**
     * 雪花算法序列自增脚本
     */
    private static final RedisScript<Long> INCR_SNOWFLAKE_SEQUENCE_SCRIPT = new DefaultRedisScript<>(
            "local exists = redis.call('EXISTS', KEYS[1]) " +
                    "if exists == 1 then local workerId = redis.call('GET', KEYS[1]) " +
                    "workerId = (workerId + 1) % 1024 redis.call('SET', KEYS[1], workerId) return workerId " +
                    "else redis.call('SET', KEYS[1], 0) return 0 end", Long.class
    );

    /**
     * 新增Stream消息流脚本
     */
    private static RedisScript<String> STREAM_ADD_SCRIPT;

    /**
     * Redis连接工厂
     */
    private static RedisConnectionFactory CONNECTION_FACTORY;

    /**
     * 0号库Redis连接工厂
     */
    private static RedisConnectionFactory DEFAULT_CONNECTION_FACTORY;

    /**
     * ID生成器实例
     */
    private static IdentityUtils.Generator IDENTITY_GENERATOR;

    /**
     * 当前RedisTemplate实例
     */
    private static RedisTemplate<String, Object> REDIS_TEMPLATE;

    /**
     * 字符串RedisTemplate实例
     */
    private static RedisTemplate<String, Object> STRING_TEMPLATE;

    /**
     * 默认RedisTemplate实例
     */
    private static RedisTemplate<String, Object> DEFAULT_TEMPLATE;

    /**
     * 配置属性
     */
    private static com.chelvc.framework.redis.config.RedisProperties PROPERTIES;

    private RedisContextHolder() {
    }

    /**
     * 获取Redis配置
     *
     * @param database 数据库下标
     * @return Redis配置实例
     */
    public static RedisStandaloneConfiguration getConfiguration(int database) {
        RedisProperties properties = ApplicationContextHolder.getBean(RedisProperties.class);
        RedisStandaloneConfiguration configuration = new RedisStandaloneConfiguration();
        configuration.setDatabase(database);
        configuration.setPort(properties.getPort());
        configuration.setHostName(properties.getHost());
        configuration.setPassword(properties.getPassword());
        return configuration;
    }

    /**
     * 获取Redis连接工厂
     *
     * @param configuration Redis配置
     * @return Redis连接工厂实例
     */
    public static RedisConnectionFactory getConnectionFactory(@NonNull RedisStandaloneConfiguration configuration) {
        LettuceConnectionFactory connectionFactory = new LettuceConnectionFactory(configuration);
        connectionFactory.afterPropertiesSet();
        return connectionFactory;
    }

    /**
     * 获取Redis连接工厂对象
     *
     * @return Redis连接工厂对象
     */
    public static RedisConnectionFactory getConnectionFactory() {
        if (CONNECTION_FACTORY == null) {
            synchronized (RedisConnectionFactory.class) {
                if (CONNECTION_FACTORY == null) {
                    CONNECTION_FACTORY = ApplicationContextHolder.getBean(RedisConnectionFactory.class);
                }
            }
        }
        return CONNECTION_FACTORY;
    }

    /**
     * 获取0号库Redis连接工厂对象
     *
     * @return Redis连接工厂对象
     */
    public static RedisConnectionFactory getDefaultConnectionFactory() {
        if (DEFAULT_CONNECTION_FACTORY == null) {
            synchronized (RedisConnectionFactory.class) {
                if (DEFAULT_CONNECTION_FACTORY == null) {
                    DEFAULT_CONNECTION_FACTORY = getConnectionFactory(getConfiguration(0));
                }
            }
        }
        return DEFAULT_CONNECTION_FACTORY;
    }

    /**
     * 获取当前RedisTemplate实例
     *
     * @return RedisTemplate实例
     */
    public static RedisTemplate<String, Object> getRedisTemplate() {
        if (REDIS_TEMPLATE == null) {
            synchronized (RedisTemplate.class) {
                if (REDIS_TEMPLATE == null) {
                    REDIS_TEMPLATE = ApplicationContextHolder.getBean("redisTemplate");
                }
            }
        }
        return REDIS_TEMPLATE;
    }

    /**
     * 获取当前RedisTemplate实例
     *
     * @param database 数据库下标
     * @return RedisTemplate实例
     */
    public static RedisTemplate<String, Object> getRedisTemplate(int database) {
        RedisSerializer<?> valueSerializer =
                ApplicationContextHolder.getBean(Jackson2JsonRedisSerializer.class);
        RedisTemplate<String, Object> template = new RedisTemplate<>();
        template.setConnectionFactory(getConnectionFactory(getConfiguration(database)));
        template.setKeySerializer(RedisSerializer.string());
        template.setValueSerializer(valueSerializer);
        template.setHashKeySerializer(RedisSerializer.string());
        template.setHashValueSerializer(valueSerializer);
        template.afterPropertiesSet();
        return template;
    }

    /**
     * 获取基于字符串配置的0号库RedisTemplate实例
     *
     * @return RedisTemplate实例
     */
    public static RedisTemplate<String, Object> getStringTemplate() {
        if (STRING_TEMPLATE == null) {
            synchronized (RedisTemplate.class) {
                if (STRING_TEMPLATE == null) {
                    RedisTemplate<String, Object> template = new RedisTemplate<>();
                    template.setConnectionFactory(getDefaultConnectionFactory());
                    template.setKeySerializer(RedisSerializer.string());
                    template.setValueSerializer(RedisSerializer.string());
                    template.setHashKeySerializer(RedisSerializer.string());
                    template.setHashValueSerializer(RedisSerializer.string());
                    template.afterPropertiesSet();

                    STRING_TEMPLATE = template;
                }
            }
        }
        return STRING_TEMPLATE;
    }

    /**
     * 获取基于默认配置的0号库RedisTemplate实例
     *
     * @return RedisTemplate实例
     */
    public static RedisTemplate<String, Object> getDefaultTemplate() {
        if (DEFAULT_TEMPLATE == null) {
            synchronized (RedisTemplate.class) {
                if (DEFAULT_TEMPLATE == null) {
                    DEFAULT_TEMPLATE = getRedisTemplate(0);
                }
            }
        }
        return DEFAULT_TEMPLATE;
    }

    /**
     * 获取新增Stream消息流脚本
     *
     * @return Lua脚本
     */
    private static RedisScript<String> getStreamAddScript() {
        if (STREAM_ADD_SCRIPT == null) {
            synchronized (RedisContextHolder.class) {
                if (STREAM_ADD_SCRIPT == null) {
                    int capacity = getProperties().getStream().getCapacity();
                    if (capacity > 0) {
                        STREAM_ADD_SCRIPT = new DefaultRedisScript<>(String.format(
                                "return redis.call('XADD', KEYS[1], 'MAXLEN', '~', %d, '*', unpack(ARGV))", capacity
                        ), String.class);
                    } else {
                        STREAM_ADD_SCRIPT = new DefaultRedisScript<>(
                                "return redis.call('XADD', KEYS[1], '*', unpack(ARGV))", String.class
                        );
                    }
                }
            }
        }
        return STREAM_ADD_SCRIPT;
    }

    /**
     * 环境隔离
     *
     * @param original 原始标记
     * @return 环境隔离标记
     */
    public static String isolate(@NonNull String original) {
        String namespace = getProperties().getNamespace();
        return StringUtils.isEmpty(namespace) ? original : (namespace + original);
    }

    /**
     * 初始化雪花算法序列
     *
     * @return 雪花算法序列
     */
    private static Long initializeSnowflakeSequence() {
        List<String> keys = Collections.singletonList(SNOWFLAKE_SEQUENCE_KEY);
        return getDefaultTemplate().execute(INCR_SNOWFLAKE_SEQUENCE_SCRIPT, keys);
    }

    /**
     * 初始化雪花算法ID生成器
     *
     * @return ID生成器实例
     */
    private static IdentityUtils.Generator initializeIdentityGenerator() {
        // 获取雪花算法序列，并将序列补齐10位
        String sequence = StringUtils.rjust(Long.toBinaryString(initializeSnowflakeSequence()), 10, '0');

        // 基于序列拆分workerId和datacenterId
        long datacenterId = Integer.parseInt(sequence.substring(0, 5), 2);
        long workerId = Integer.parseInt(sequence.substring(5, 10), 2);
        return new IdentityUtils.Generator(workerId, datacenterId);
    }

    /**
     * 获取基于雪花算法ID生成器
     *
     * @return ID生成器实例
     */
    public static IdentityUtils.Generator getIdentityGenerator() {
        if (IDENTITY_GENERATOR == null) {
            synchronized (IdentityUtils.Generator.class) {
                if (IDENTITY_GENERATOR == null) {
                    IDENTITY_GENERATOR = initializeIdentityGenerator();
                }
            }
        }
        return IDENTITY_GENERATOR;
    }

    /**
     * 获取消息流配置属性
     *
     * @return 配置属性
     */
    public static com.chelvc.framework.redis.config.RedisProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (com.chelvc.framework.redis.config.RedisProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES =
                            ApplicationContextHolder.getBean(com.chelvc.framework.redis.config.RedisProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 执行Redis操作
     *
     * @param function 操作方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T execute(@NonNull Function<RedisConnection, T> function) {
        return execute(getConnectionFactory(), function);
    }

    /**
     * 执行Redis操作
     *
     * @param factory  连接工厂
     * @param function 操作方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T execute(@NonNull RedisConnectionFactory factory,
                                @NonNull Function<RedisConnection, T> function) {
        return execute(factory, function, e -> {
            log.warn("Redis execute failed: {}", e.getMessage());
            return null;
        });
    }

    /**
     * 执行Redis操作
     *
     * @param function 操作方法
     * @param failure  操作失败回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T execute(@NonNull Function<RedisConnection, T> function,
                                @NonNull Function<Exception, T> failure) {
        return execute(getConnectionFactory(), function, failure);
    }

    /**
     * 执行Redis操作
     *
     * @param factory  连接工厂
     * @param function 操作方法
     * @param failure  操作失败回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T execute(@NonNull RedisConnectionFactory factory, @NonNull Function<RedisConnection, T> function,
                                @NonNull Function<Exception, T> failure) {
        RedisConnection connection = null;
        try {
            connection = factory.getConnection();
            return function.apply(connection);
        } catch (Exception e) {
            return failure.apply(e);
        } finally {
            if (connection != null) {
                RedisConnectionUtils.releaseConnection(connection, factory, false);
            }
        }
    }

    /**
     * 加锁（阻塞），加锁成功则返回锁标识
     *
     * @param name 锁名称
     * @return 锁标识
     */
    public static String lock(String name) {
        return lock(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT));
    }

    /**
     * 加锁（阻塞），加锁成功则返回锁标识
     *
     * @param name     锁名称
     * @param duration 锁有效期
     * @return 锁标识
     */
    public static String lock(@NonNull String name, @NonNull Duration duration) {
        String token;
        long expiration = System.currentTimeMillis() + duration.toMillis();
        do {
            if (StringUtils.isEmpty(token = tryLock(name, duration))) {
                ThreadUtils.sleep(200);
            }
        } while (StringUtils.isEmpty(token) && System.currentTimeMillis() <= expiration && !Thread.interrupted());
        if (StringUtils.isEmpty(token)) {
            throw new IllegalStateException("Redis lock timeout");
        }
        return token;
    }

    /**
     * 判断锁标识是否已加锁
     *
     * @param name 锁名称
     * @return true/false
     */
    public static boolean locked(@NonNull String name) {
        byte[] key = name.getBytes(StandardCharsets.UTF_8);
        return execute(getDefaultConnectionFactory(), connection -> {
            byte[] value = connection.get(key);
            return value != null && StringUtils.notEmpty(new String(value, StandardCharsets.UTF_8));
        }, e -> false);
    }

    /**
     * 尝试加锁（无阻塞），如果加锁成功则返回锁标识，否则返回null
     *
     * @param name 锁名称
     * @return 锁标识
     */
    public static String tryLock(String name) {
        return tryLock(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT));
    }

    /**
     * 尝试加锁（无阻塞），如果加锁成功则返回锁标识，否则返回null
     *
     * @param name     锁名称
     * @param duration 锁有效期
     * @return 锁标识
     */
    public static String tryLock(@NonNull String name, @NonNull Duration duration) {
        String token = UUID.randomUUID().toString();
        byte[] key = name.getBytes(StandardCharsets.UTF_8);
        byte[] value = token.getBytes(StandardCharsets.UTF_8);
        return execute(getDefaultConnectionFactory(), connection -> {
            Boolean success = connection.set(key, value, Expiration.from(duration),
                    RedisStringCommands.SetOption.SET_IF_ABSENT);
            return Boolean.TRUE.equals(success) ? token : null;
        }, e -> {
            log.warn("Redis lock failed: {}", e.getMessage());
            return null;
        });
    }

    /**
     * 解锁
     *
     * @param name  锁名称
     * @param token 锁标识
     * @return true/false
     */
    public static boolean unlock(@NonNull String name, @NonNull String token) {
        byte[][] args = new byte[][]{name.getBytes(StandardCharsets.UTF_8), token.getBytes(StandardCharsets.UTF_8)};
        return execute(getDefaultConnectionFactory(), connection -> {
            Long result;
            try {
                result = connection.scriptingCommands().evalSha(UNLOCK_SCRIPT_SHA, ReturnType.INTEGER, 1, args);
            } catch (Exception e) {
                result = connection.scriptingCommands().eval(UNLOCK_SCRIPT.getBytes(), ReturnType.INTEGER, 1, args);
            }
            return result != null && result > 0;
        }, e -> {
            log.warn("Redis unlock failed: {}", e.getMessage());
            return false;
        });
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param executor 回调方法
     */
    public static void lockAround(String name, @NonNull Executor executor) {
        lockAround(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT), executor);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param duration 有效时间（毫秒）
     * @param executor 回调方法
     */
    public static void lockAround(String name, Duration duration, @NonNull Executor executor) {
        String token = lock(name, duration);
        try {
            executor.execute();
        } finally {
            unlock(name, token);
        }
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param executor 回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T lockAround(String name, Supplier<T> executor) {
        return lockAround(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT), executor);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param duration 有效时间
     * @param executor 回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T lockAround(String name, Duration duration, Supplier<T> executor) {
        String token = lock(name, duration);
        try {
            return executor.get();
        } finally {
            unlock(name, token);
        }
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param executor 回调方法
     * @param failure  枷锁失败回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T lockAround(String name, @NonNull Supplier<T> executor, @NonNull Supplier<T> failure) {
        return lockAround(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT), executor, failure);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param duration 有效时间
     * @param executor 回调方法
     * @param failure  枷锁失败回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T lockAround(String name, Duration duration, @NonNull Supplier<T> executor,
                                   @NonNull Supplier<T> failure) {
        String token;
        try {
            token = lock(name, duration);
        } catch (IllegalStateException e) {
            return failure.get();
        }
        try {
            return executor.get();
        } finally {
            unlock(name, token);
        }
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param executor 回调方法
     */
    public static void tryLockAround(String name, @NonNull Executor executor) {
        tryLockAround(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT), executor);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param duration 有效时间
     * @param executor 回调方法
     */
    public static void tryLockAround(String name, Duration duration, @NonNull Executor executor) {
        String token = tryLock(name, duration);
        if (StringUtils.notEmpty(token)) {
            try {
                executor.execute();
            } finally {
                unlock(name, token);
            }
        }
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param supplier 回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T tryLockAround(String name, Supplier<T> supplier) {
        return tryLockAround(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT), supplier);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param duration 有效时间
     * @param supplier 回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T tryLockAround(String name, Duration duration, Supplier<T> supplier) {
        return tryLockAround(name, duration, supplier, () -> null);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param supplier 回调方法
     * @param failure  枷锁失败回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T tryLockAround(String name, @NonNull Supplier<T> supplier, @NonNull Supplier<T> failure) {
        return tryLockAround(name, Duration.ofSeconds(DEFAULT_LOCK_TIMEOUT), supplier, failure);
    }

    /**
     * 加/解锁环绕执行方法
     *
     * @param name     锁名称
     * @param duration 有效时间
     * @param supplier 回调方法
     * @param failure  枷锁失败回调方法
     * @param <T>      返回数据类型泛型
     * @return 执行结果
     */
    public static <T> T tryLockAround(String name, Duration duration, @NonNull Supplier<T> supplier,
                                      @NonNull Supplier<T> failure) {
        String token = tryLock(name, duration);
        if (StringUtils.isEmpty(token)) {
            return failure.get();
        }
        try {
            return supplier.get();
        } finally {
            unlock(name, token);
        }
    }

    /**
     * 生成分布式唯一ID
     *
     * @return ID值
     */
    public static long identity() {
        return getIdentityGenerator().next();
    }

    /**
     * 生成唯一序号，默认1分钟有效期
     *
     * @return 唯一序号
     */
    public static long sequence() {
        return sequence(Duration.ofMinutes(1));
    }

    /**
     * 生成唯一序号
     * <p>
     * 唯一标识格式：yyyyMMddHHmmss + 五位随机数，例如：202108251933453756
     *
     * @param duration 唯一标识有效期
     * @return 唯一序号
     */
    public static long sequence(@NonNull Duration duration) {
        ThreadLocalRandom random = ThreadLocalRandom.current();
        RedisConnectionFactory factory = getDefaultConnectionFactory();
        for (int i = 0; i < 10; i++) {
            String time = IDENTITY_DATETIME_FORMATTER.format(LocalDateTime.now());
            String value = StringUtils.rjust(String.valueOf(random.nextInt(100000)), 5, '0');
            long identity = Long.parseLong(time + value);
            byte[] key = (SEQUENCE_REDIS_PREFIX + identity).getBytes(StandardCharsets.UTF_8);
            Boolean success = execute(factory, connection -> connection.set(key, ObjectUtils.EMPTY_BYTES,
                    Expiration.from(duration), RedisStringCommands.SetOption.SET_IF_ABSENT));
            if (Boolean.TRUE.equals(success)) {
                return identity;
            }
        }
        throw new RuntimeException("Generate identity attempts limit exceeded");
    }

    /**
     * 从队列中弹出数据
     *
     * @param queue    队列名称
     * @param supplier 数据提供者
     * @param <T>      数据类型
     * @return 数据对象实例
     */
    public static <T> T pop(@NonNull String queue, @NonNull Supplier<List<T>> supplier) {
        return pop(getRedisTemplate(), queue, supplier);
    }

    /**
     * 从队列中弹出数据
     *
     * @param template Redis操作模版实例
     * @param queue    队列名称
     * @param supplier 数据提供者
     * @param <K>      键类型
     * @param <V>      值类型
     * @return 数据对象实例
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    public static <K, V> V pop(@NonNull RedisTemplate<K, ?> template, @NonNull K queue,
                               @NonNull Supplier<List<V>> supplier) {
        // 尝试从队列中获取数据
        V object = (V) template.opsForList().rightPop(queue);
        if (object == null) {
            // 获取redis锁
            object = lockAround(queue + ":lock", () -> {
                V v = (V) template.opsForList().rightPop(queue);
                if (v == null) {
                    // 加载队列数据
                    List<V> objects = supplier.get();
                    if (CollectionUtils.isEmpty(objects)) {
                        return null;
                    }
                    template.opsForList().leftPushAll(queue, (Collection) objects);
                    v = (V) template.opsForList().rightPop(queue);
                }
                return v;
            });
        }
        return object;
    }

    /**
     * 批量获取Redis值
     *
     * @param keys 键名称列表
     * @return 键/值映射表
     */
    public static Map<String, String> get(@NonNull List<String> keys) {
        return get(getRedisTemplate(), keys);
    }

    /**
     * 批量获取Redis值
     *
     * @param keys 键名称列表
     * @return 键/值映射表
     */
    public static Map<String, String> get(@NonNull RedisTemplate<String, ?> template, @NonNull List<String> keys) {
        return get(template, keys, key -> key, value -> new String(value, StandardCharsets.UTF_8));
    }

    /**
     * 批量获取Redis值
     *
     * @param keys         键名称列表
     * @param keyAdapter   键适配器
     * @param valueAdapter 值适配器
     * @param <T>          数据类型
     * @return 键/值映射表
     */
    public static <T> Map<String, T> get(List<String> keys, @NonNull Function<String, String> keyAdapter,
                                         @NonNull Function<byte[], T> valueAdapter) {
        return get(getRedisTemplate(), keys, keyAdapter, valueAdapter);
    }

    /**
     * 批量获取Redis值
     *
     * @param template     Redis操作模版实例
     * @param keys         键名称列表
     * @param keyAdapter   键适配器
     * @param valueAdapter 值适配器
     * @param <K>          键类型
     * @param <V>          值类型
     * @return 键/值映射表
     */
    public static <K, V> Map<K, V> get(@NonNull RedisTemplate<K, ?> template, List<String> keys,
                                       @NonNull Function<String, K> keyAdapter,
                                       @NonNull Function<byte[], V> valueAdapter) {
        if (CollectionUtils.isEmpty(keys)) {
            return Collections.emptyMap();
        }
        return execute(template.getConnectionFactory(), connection -> {
            List<byte[]> values = connection.mGet(
                    keys.stream().map(name -> name.getBytes(StandardCharsets.UTF_8)).toArray(byte[][]::new)
            );
            if (CollectionUtils.isEmpty(values)) {
                return Collections.emptyMap();
            }
            Map<K, V> mapping = Maps.newHashMapWithExpectedSize(keys.size());
            for (int i = 0; i < keys.size(); i++) {
                String key = keys.get(i);
                byte[] bytes = values.get(i);
                if (StringUtils.isEmpty(key) || bytes == null || bytes.length == 0) {
                    continue;
                }
                V value = valueAdapter.apply(bytes);
                if (StringUtils.notEmpty(value)) {
                    mapping.put(keyAdapter.apply(key), value);
                }
            }
            return mapping;
        }, e -> Collections.emptyMap());
    }

    /**
     * 获取键值，如果键不存在则初始化
     *
     * @param key          键名称
     * @param initialValue 初始化值
     * @param <T>          数据类型
     * @return 值
     */
    public static <T> T get(@NonNull String key, @NonNull T initialValue) {
        return get(getRedisTemplate(), key, initialValue);
    }

    /**
     * 获取键值，如果键不存在则初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param initialValue 初始化值
     * @param <K>          键类型
     * @param <V>          值类型
     * @return 值
     */
    @SuppressWarnings("unchecked")
    public static <K, V> V get(@NonNull RedisTemplate<K, ?> template, @NonNull K key, @NonNull V initialValue) {
        return (V) template.execute(GET_WITH_INITIAL_SCRIPT, Collections.singletonList(key), initialValue);
    }

    /**
     * 获取键值，如果键不存在则初始化
     *
     * @param key          键名称
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @param <T>          数据类型
     * @return 值
     */
    public static <T> T get(@NonNull String key, @NonNull T initialValue, @NonNull Duration duration) {
        return get(getRedisTemplate(), key, initialValue, duration);
    }

    /**
     * 获取键值，如果键不存在则初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @param <K>          键类型
     * @param <V>          值类型
     * @return 值
     */
    @SuppressWarnings("unchecked")
    public static <K, V> V get(@NonNull RedisTemplate<K, ?> template, @NonNull K key, @NonNull V initialValue,
                               @NonNull Duration duration) {
        return (V) template.execute(GET_WITH_INITIAL_DURATION_SCRIPT, Collections.singletonList(key),
                initialValue, duration.getSeconds());
    }

    /**
     * 数字自增1，如果键不存在则对值初始化
     *
     * @param key          键名称
     * @param initialValue 初始化值
     * @return 自增后的值
     */
    public static Long increment(@NonNull String key, long initialValue) {
        return increment(getRedisTemplate(), key, initialValue);
    }

    /**
     * 数字自增1，如果键不存在则对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param initialValue 初始化值
     * @param <K>          键类型
     * @return 自增后的值
     */
    public static <K> Long increment(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long initialValue) {
        return template.execute(INCR_WITH_INITIAL_SCRIPT, Collections.singletonList(key), initialValue);
    }

    /**
     * 数字自增指定增量值，如果键不存在则对值初始化
     *
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @return 自增后的值
     */
    public static Long increment(@NonNull String key, long delta, long initialValue) {
        return increment(getRedisTemplate(), key, delta, initialValue);
    }

    /**
     * 数字自增指定增量值，如果键不存在则对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @param <K>          键类型
     * @return 自增后的值
     */
    public static <K> Long increment(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long delta,
                                     long initialValue) {
        return template.execute(INCRBY_WITH_INITIAL_SCRIPT, Collections.singletonList(key), delta, initialValue);
    }

    /**
     * 数字自增1，如果键不存在则根据有效时间对值初始化
     *
     * @param key          键名称
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @return 自增后的值
     */
    public static Long increment(@NonNull String key, long initialValue, @NonNull Duration duration) {
        return increment(getRedisTemplate(), key, initialValue, duration);
    }

    /**
     * 数字自增1，如果键不存在则根据有效时间对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @param <K>          键类型
     * @return 自增后的值
     */
    public static <K> Long increment(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long initialValue,
                                     @NonNull Duration duration) {
        return template.execute(INCR_WITH_INITIAL_DURATION_SCRIPT, Collections.singletonList(key), initialValue,
                duration.getSeconds());
    }

    /**
     * 数字自增指定增量值，如果键不存在则根据有效时间对值初始化
     *
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @return 自增后的值
     */
    public static Long increment(@NonNull String key, long delta, long initialValue, @NonNull Duration duration) {
        return increment(getRedisTemplate(), key, delta, initialValue, duration);
    }

    /**
     * 数字自增指定增量值，如果键不存在则根据有效时间对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @param <K>          键类型
     * @return 自增后的值
     */
    public static <K> Long increment(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long delta,
                                     long initialValue, @NonNull Duration duration) {
        return template.execute(INCRBY_WITH_INITIAL_DURATION_SCRIPT, Collections.singletonList(key), delta,
                initialValue, duration.getSeconds());
    }

    /**
     * 数字自减1，如果键不存在则对值初始化
     *
     * @param key          键名称
     * @param initialValue 初始化值
     * @return 自减后的值
     */
    public static Long decrement(@NonNull String key, long initialValue) {
        return decrement(getRedisTemplate(), key, initialValue);
    }

    /**
     * 数字自减1，如果键不存在则对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param initialValue 初始化值
     * @param <K>          键类型
     * @return 自减后的值
     */
    public static <K> Long decrement(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long initialValue) {
        return template.execute(DECR_WITH_INITIAL_SCRIPT, Collections.singletonList(key), initialValue);
    }

    /**
     * 数字自减指定减量值，如果键不存在则对值初始化
     *
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @return 自减后的值
     */
    public static Long decrement(@NonNull String key, long delta, long initialValue) {
        return decrement(getRedisTemplate(), key, delta, initialValue);
    }

    /**
     * 数字自减指定减量值，如果键不存在则对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @param <K>          键类型
     * @return 自减后的值
     */
    public static <K> Long decrement(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long delta,
                                     long initialValue) {
        return template.execute(DECRBY_WITH_INITIAL_SCRIPT, Collections.singletonList(key), delta, initialValue);
    }

    /**
     * 数字自减1，如果键不存在则根据有效时间对值初始化
     *
     * @param key          键名称
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @return 自减后的值
     */
    public static Long decrement(@NonNull String key, long initialValue, @NonNull Duration duration) {
        return decrement(getRedisTemplate(), key, initialValue, duration);
    }

    /**
     * 数字自减1，如果键不存在则根据有效时间对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @param <K>          键类型
     * @return 自减后的值
     */
    public static <K> Long decrement(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long initialValue,
                                     @NonNull Duration duration) {
        return template.execute(DECR_WITH_INITIAL_DURATION_SCRIPT, Collections.singletonList(key), initialValue,
                duration.getSeconds());
    }

    /**
     * 数字自减指定减量值，如果键不存在则根据有效时间对值初始化
     *
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @return 自减后的值
     */
    public static Long decrement(@NonNull String key, long delta, long initialValue, @NonNull Duration duration) {
        return decrement(getRedisTemplate(), key, delta, initialValue, duration);
    }

    /**
     * 数字自减指定减量值，如果键不存在则根据有效时间对值初始化
     *
     * @param template     Redis操作模版实例
     * @param key          键名称
     * @param delta        减量值
     * @param initialValue 初始化值
     * @param duration     有效时间
     * @param <K>          键类型
     * @return 自减后的值
     */
    public static <K> Long decrement(@NonNull RedisTemplate<K, ?> template, @NonNull K key, long delta,
                                     long initialValue, @NonNull Duration duration) {
        return template.execute(DECRBY_WITH_INITIAL_DURATION_SCRIPT, Collections.singletonList(key), delta,
                initialValue, duration.getSeconds());
    }

    /**
     * 生成随机字符串
     *
     * @return 随机字符串
     */
    public static String random() {
        return random(StringUtils::random, Duration.ofSeconds(60));
    }

    /**
     * 生成随机字符串
     *
     * @param length 随机字符串长度
     * @return 随机字符串
     */
    public static String random(int length) {
        return random(() -> StringUtils.random(length), Duration.ofSeconds(60));
    }

    /**
     * 生成随机字符串
     *
     * @param chars 随机串字符数组
     * @return 随机字符串
     */
    public static String random(@NonNull char[] chars) {
        return random(chars, 4);
    }

    /**
     * 生成随机字符串
     *
     * @param duration 随机串保存时间
     * @return 随机字符串
     */
    public static String random(@NonNull Duration duration) {
        return random(StringUtils::random, duration);
    }

    /**
     * 生成随机字符串
     *
     * @param chars  随机串字符数组
     * @param length 随机字符串长度
     * @return 随机字符串
     */
    public static String random(@NonNull char[] chars, int length) {
        return random(chars, length, Duration.ofSeconds(60));
    }

    /**
     * 生成随机字符串
     *
     * @param chars    随机串字符数组
     * @param length   随机字符串长度
     * @param duration 随机串保存时间
     * @return 随机字符串
     */
    public static String random(@NonNull char[] chars, int length, @NonNull Duration duration) {
        return random(() -> StringUtils.random(chars, length), duration);
    }

    /**
     * 生成随机字符串
     *
     * @param supplier 随机字符串生成器
     * @param duration 随机串保存时间
     * @return 随机字符串
     */
    public static String random(@NonNull Supplier<String> supplier, @NonNull Duration duration) {
        RedisConnectionFactory factory = getDefaultConnectionFactory();
        for (int i = 0; i < 10; i++) {
            String value = supplier.get();
            byte[] key = (RANDOM_REDIS_PREFIX + value).getBytes(StandardCharsets.UTF_8);
            Boolean success = execute(factory, connection -> connection.set(key, ObjectUtils.EMPTY_BYTES,
                    Expiration.from(duration), RedisStringCommands.SetOption.SET_IF_ABSENT));
            if (Boolean.TRUE.equals(success)) {
                return value;
            }
        }
        throw new RuntimeException("Generate random attempts limit exceeded");
    }

    /**
     * 获取当前时间距过期时间的时长
     *
     * @param expiration 过期时间
     * @return 持续时长
     */
    public static Duration duration(@NonNull Date expiration) {
        return Duration.ofMillis(Math.max(expiration.getTime() - System.currentTimeMillis(), 1000));
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     * @return 消息记录ID
     */
    public static RecordId send(@NonNull String topic, @NonNull Object payload) {
        return send(topic, payload, (Long) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @return 消息记录ID
     */
    public static RecordId send(@NonNull String topic, @NonNull Object payload, Duration delaying) {
        return send(topic, payload, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @return 消息记录ID
     */
    public static RecordId send(@NonNull String topic, @NonNull Object payload, Date delaying) {
        return send(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @return 消息记录ID
     */
    public static RecordId send(@NonNull String topic, @NonNull Object payload, Long delaying) {
        List<String> args = Lists.newLinkedList();
        args.add(PAYLOAD);
        args.add(JacksonUtils.serialize(payload));
        if (delaying != null) {
            args.add(DELAYING);
            args.add(String.valueOf(delaying));
        }
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            args.add(SessionContextHolder.NAME);
            args.add(JacksonUtils.serialize(session));
        }
        List<String> keys = Collections.singletonList(isolate(topic));
        String id = getStringTemplate().execute(getStreamAddScript(), keys, args.toArray());
        return StringUtils.ifEmpty(id, RecordId::of);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param consumer 目标消费者
     * @param ids      目标记录ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic,
                                         @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                         @NonNull RecordId... ids) {
        return claim(topic, consumer.getGroup(), consumer.getName(), ids);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param group    消费者组
     * @param consumer 目标消费者名称
     * @param ids      目标记录ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic, @NonNull String group, @NonNull String consumer,
                                         @NonNull RecordId... ids) {
        return claim(topic, group, consumer, Duration.ZERO, ids);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param consumer 目标消费者
     * @param idle     消息空闲时间
     * @param ids      目标ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic,
                                         @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                         @NonNull Duration idle, @NonNull RecordId... ids) {
        return claim(topic, consumer.getGroup(), consumer.getName(), idle, ids);
    }

    /**
     * 转移消息
     *
     * @param topic    消息主题
     * @param group    消费者组
     * @param consumer 目标消费者名称
     * @param idle     消息空闲时间
     * @param ids      目标ID数组
     * @return 转移成功消息记录列表
     */
    public static List<ByteRecord> claim(@NonNull String topic, @NonNull String group, @NonNull String consumer,
                                         @NonNull Duration idle, @NonNull RecordId... ids) {
        if (ObjectUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        RedisStreamCommands.XClaimOptions options = RedisStreamCommands.XClaimOptions.minIdle(idle).ids(ids);
        return execute(getDefaultConnectionFactory(),
                connection -> connection.streamCommands().xClaim(topic.getBytes(), group, consumer, options));
    }

    /**
     * 消息消费确认
     *
     * @param topic 消息主题
     * @param group 消费者组
     * @param ids   消息记录ID数组
     */
    public static void ack(@NonNull String topic, @NonNull String group, @NonNull RecordId... ids) {
        if (ObjectUtils.notEmpty(ids)) {
            getStringTemplate().opsForStream().acknowledge(topic, group, ids);
        }
    }

    /**
     * 删除消费者
     *
     * @param topic 消息主题
     * @param ids   消息记录ID数组
     */
    public static void delete(@NonNull String topic, @NonNull RecordId... ids) {
        if (ObjectUtils.notEmpty(ids)) {
            getStringTemplate().opsForStream().delete(topic, ids);
        }
    }

    /**
     * 删除消费者
     *
     * @param topic    消息主题
     * @param consumer 消费者信息
     */
    public static void delete(@NonNull String topic,
                              @NonNull org.springframework.data.redis.connection.stream.Consumer consumer) {
        getStringTemplate().opsForStream().deleteConsumer(topic, consumer);
    }

    /**
     * 发送心跳包
     *
     * @param topic 消息主题
     */
    public static void heartbeat(@NonNull String topic) {
        List<String> keys = Collections.singletonList(topic);
        getStringTemplate().execute(getStreamAddScript(), keys, PAYLOAD, StringUtils.EMPTY);
    }

    /**
     * 判断消息是否是心跳包
     *
     * @param record 消息记录
     * @return true/false
     */
    public static boolean isHeartbeat(MapRecord<String, String, String> record) {
        Map<String, String> value = ObjectUtils.ifNull(record, MapRecord::getValue);
        return ObjectUtils.isEmpty(value) || StringUtils.isEmpty(value.get(PAYLOAD));
    }

    /**
     * 序列化消息记录
     *
     * @param record 消息记录
     * @return 消息内容
     */
    public static String serialize(@NonNull MapRecord<String, String, String> record) {
        Map<String, String> value = record.getValue();
        ImmutableMap.Builder<String, String> builder = ImmutableMap.builder();
        builder.put(ID, record.getId().getValue());
        builder.put(TOPIC, Objects.requireNonNull(record.getStream()));
        String payload = value.get(PAYLOAD);
        if (StringUtils.notEmpty(payload)) {
            builder.put(PAYLOAD, payload);
        }
        String session = value.get(SessionContextHolder.NAME);
        if (StringUtils.notEmpty(session)) {
            builder.put(SessionContextHolder.NAME, session);
        }
        String delaying = value.get(DELAYING);
        if (StringUtils.notEmpty(delaying)) {
            builder.put(DELAYING, delaying);
        }
        return JacksonUtils.serialize(builder.build());
    }

    /**
     * 反序列化消息记录
     *
     * @param message 消息内容
     * @return 消息记录实例
     */
    @SuppressWarnings("unchecked")
    public static MapRecord<String, String, String> deserialize(@NonNull String message) {
        Map<String, String> content = JacksonUtils.deserialize(message, Map.class);
        String id = content.remove(ID);
        String topic = content.remove(TOPIC);
        return MapRecord.create(topic, content).withId(RecordId.of(id));
    }

    /**
     * 消费消息
     *
     * @param record   消息记录
     * @param type     消息类型
     * @param consumer 消息消费者
     */
    @SuppressWarnings("unchecked")
    public static <T> void consume(@NonNull MapRecord<String, String, String> record, @NonNull Type type,
                                   @NonNull Consumer<T> consumer) {
        Map<String, String> value = record.getValue();
        T payload = (T) JacksonUtils.deserialize(value.get(PAYLOAD), type);
        Session session = JacksonUtils.deserialize(value.get(SessionContextHolder.NAME), Session.class);
        SessionContextHolder.setSession(session);
        try {
            consumer.accept(payload);
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }

    /**
     * 获取消费者信息
     *
     * @param topic 消息主题
     * @param group 消费者组
     * @return 消费者信息
     */
    public static StreamInfo.XInfoConsumers consumers(@NonNull String topic, @NonNull String group) {
        return getStringTemplate().opsForStream().consumers(topic, group);
    }

    /**
     * 获取消息延时时间戳
     *
     * @param record 消息记录
     * @return 时间戳
     */
    public static Long getMessageDelaying(@NonNull MapRecord<String, String, String> record) {
        String delaying = ObjectUtils.ifNull(record.getValue(), value -> value.get(DELAYING));
        return StringUtils.ifEmpty(delaying, Long::parseLong);
    }

    /**
     * 初始化消费者组
     *
     * @param topic 消息主题
     * @param group 消费者组
     */
    public static void initializeConsumerGroup(@NonNull String topic, @NonNull String group) {
        try {
            getStringTemplate().opsForStream().createGroup(topic, group);
        } catch (RedisSystemException e) {
            if (StringUtils.isEmpty(e.getMessage()) || !e.getMessage().contains("BUSYGROUP")) {
                throw e;
            }
        }
    }

    /**
     * 初始化消息监听器容器
     *
     * @param listener 消息监听器
     * @param consumer 消费者信息
     * @param offset   消息偏移信息
     * @param batch    批量获取消息数量
     * @param executor 消息获取执行器
     * @return 消息监听器容器
     */
    public static StreamMessageListenerContainer<String, MapRecord<String, String, String>>
    initializeMessageListenerContainer(@NonNull StreamListener<String, MapRecord<String, String, String>> listener,
                                       @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                       @NonNull StreamOffset<String> offset, int batch,
                                       @NonNull java.util.concurrent.Executor executor) {
        return initializeMessageListenerContainer(listener, consumer, offset, batch, executor, null);
    }

    /**
     * 初始化消息监听器容器
     *
     * @param listener     消息监听器
     * @param consumer     消费者信息
     * @param offset       消息偏移信息
     * @param batch        批量获取消息数量
     * @param executor     消息获取执行器
     * @param errorHandler 异常处理器
     * @return 消息监听器容器
     */
    public static StreamMessageListenerContainer<String, MapRecord<String, String, String>>
    initializeMessageListenerContainer(@NonNull StreamListener<String, MapRecord<String, String, String>> listener,
                                       @NonNull org.springframework.data.redis.connection.stream.Consumer consumer,
                                       @NonNull StreamOffset<String> offset, int batch,
                                       @NonNull java.util.concurrent.Executor executor, ErrorHandler errorHandler) {
        StreamMessageListenerContainer.
                StreamMessageListenerContainerOptionsBuilder<String, MapRecord<String, String, String>>
                builder = StreamMessageListenerContainer.StreamMessageListenerContainerOptions.builder()
                .batchSize(batch).pollTimeout(Duration.ZERO).executor(executor).serializer(RedisSerializer.string());
        if (errorHandler != null) {
            builder.errorHandler(errorHandler);
        }
        StreamMessageListenerContainer<String, MapRecord<String, String, String>> container =
                StreamMessageListenerContainer.create(getDefaultConnectionFactory(), builder.build());
        container.receive(consumer, offset, listener);
        return container;
    }
}
