package com.chelvc.framework.redis.stream;

import java.lang.reflect.Type;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.annotation.RedisMQConsumer;
import com.chelvc.framework.redis.config.RedisProperties;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.RedisSystemException;
import org.springframework.data.redis.connection.stream.Consumer;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.connection.stream.ReadOffset;
import org.springframework.data.redis.connection.stream.StreamInfo;
import org.springframework.data.redis.connection.stream.StreamOffset;
import org.springframework.data.redis.stream.StreamMessageListenerContainer;

/**
 * 消息监听器容器默认实现
 *
 * @param <T> 监听消息对象类型
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DefaultRedisMQListenerContainer<T> implements RedisMQListenerContainer<T> {
    private final String name = StringUtils.uuid();
    private String topic;
    private String group;
    private int batch;
    private Consumer consumer;
    private ExecutorService executor;
    private StreamOffset<String> offset;
    private ScheduledExecutorService cleaner;
    private MessageStreamListener<T> listener;
    private Executor starter;
    private StreamMessageListenerContainer<String, MapRecord<String, String, String>> container;

    /**
     * 执行心跳逻辑
     */
    private void clear() {
        // 发送心跳包
        try {
            RedisContextHolder.heartbeat(this.topic);
        } catch (Throwable t) {
            log.warn("Consumer heartbeat message send failed: {}, {}", this.consumer, t.getMessage());
        }

        // 执行消费者清理逻辑
        int idle = RedisContextHolder.getProperties().getStream().getIdle() * 2 * 1000;
        StreamInfo.XInfoConsumers consumers = RedisContextHolder.consumers(this.topic, this.group);
        for (int i = 0, length = consumers.size(); i < length; i++) {
            StreamInfo.XInfoConsumer consumer = consumers.get(i);
            if (Objects.equals(consumer.consumerName(), this.name) || consumer.idleTimeMs() < idle) {
                continue;
            }
            Consumer source = Consumer.from(consumer.groupName(), consumer.consumerName());
            if (consumer.pendingCount() == 0) {
                RedisContextHolder.delete(this.topic, source);
                log.info("Consumer deleted: {}", source);
                continue;
            }
            new ConsumerPendingCleaner<>(this.topic, source, this.consumer, this.listener, this.executor).start();
        }
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull RedisMQListener<T> listener,
                           @NonNull RedisMQConsumer annotation) {
        int concurrency = annotation.concurrency();
        AssertUtils.check(concurrency > 0, () -> "Consumer concurrency must be greater than 0");
        ExecutorService executor = new ThreadPoolExecutor(
                concurrency, concurrency, 60L, TimeUnit.SECONDS, new LinkedBlockingQueue<>()
        );
        this.initialize(type, annotation.topic(), annotation.group(), annotation.batch(), listener, executor);
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull String topic, @NonNull String group, int batch,
                           @NonNull RedisMQListener<T> listener, @NonNull ExecutorService executor) {
        AssertUtils.nonempty(topic, () -> "Consumer topic must not be empty");
        AssertUtils.nonempty(group, () -> "Consumer group must not be empty");
        AssertUtils.check(batch > 0, () -> "Consumer batch must be greater than 0");
        RedisProperties.Stream properties = RedisContextHolder.getProperties().getStream();
        AssertUtils.check(properties.getIdle() > 0, () -> "Consumer idle must be greater than 0");
        Environment environment = ApplicationContextHolder.getEnvironment();
        this.topic = RedisContextHolder.isolate(environment.resolvePlaceholders(topic));
        this.group = RedisContextHolder.isolate(environment.resolvePlaceholders(group));
        this.batch = batch;
        this.consumer = Consumer.from(this.group, this.name);
        this.offset = StreamOffset.create(this.topic, ReadOffset.lastConsumed());
        this.listener = new MessageStreamListener<>(
                type, this.topic, this.consumer, listener, this.executor = executor
        );

        // 初始化消费者容器启动函数，当检测到消费者组不存在则自动重新创建消费者并重启消费者容器
        this.starter = () -> {
            // 停止当前容器（如果存在）
            if (this.container != null) {
                try {
                    this.container.stop();
                } catch (Throwable t) {
                    log.warn("Consumer container stop failed: {}, {}", this.consumer, t.getMessage());
                }
            }

            // 初始化消费者组
            RedisContextHolder.initializeConsumerGroup(this.topic, this.group);

            // 初始化消息监听器容器
            this.container = RedisContextHolder.initializeMessageListenerContainer(
                    this.listener, this.consumer, this.offset, this.batch, this.executor,
                    t -> {
                        if (!(t instanceof RedisSystemException) || StringUtils.isEmpty(t.getMessage())
                                || !t.getMessage().contains("NOGROUP")) {
                            log.error("Consumer handle exception: {}", this.consumer, t);
                            return;
                        }

                        // 如果是消费者组不存在异常则重启容器
                        log.warn("Consumer group has been destroyed and the container will be restarted");
                        this.starter.execute();
                    }
            );
            this.container.start();
        };
        this.starter.execute();

        // 初始化消费者清理定时器
        this.cleaner = Executors.newScheduledThreadPool(1);
        this.cleaner.scheduleAtFixedRate(this::clear, 0, properties.getIdle(), TimeUnit.SECONDS);
    }

    @Override
    public void destroy() throws Exception {
        try {
            if (this.cleaner != null) {
                this.cleaner.shutdown();
            }
        } finally {
            try {
                if (this.container != null) {
                    this.container.stop();
                }
            } finally {
                if (this.executor != null) {
                    this.executor.shutdown();
                }
            }
        }
    }
}
