package com.chelvc.framework.redis.stream;

import java.lang.reflect.Type;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Executor;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.connection.stream.Consumer;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.data.redis.stream.StreamListener;

/**
 * 消息流监听器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class MessageStreamListener<T> implements StreamListener<String, MapRecord<String, String, String>> {
    /**
     * ZSET批量获取同时增加分数值脚本
     */
    @SuppressWarnings("rawtypes")
    private static final RedisScript<List> RANGE_AND_INCREMENT_SCRIPT = new DefaultRedisScript<>(
            "local records = redis.call('ZRANGEBYSCORE', KEYS[1], ARGV[1], ARGV[2], 'LIMIT', 0, ARGV[4]) " +
                    "for i = 1, #records do redis.call('ZINCRBY', KEYS[1], ARGV[3], records[i]) end return records",
            List.class
    );

    private final Type type;
    private final String topic;
    private final String delayed;
    private final Consumer consumer;
    private final RedisMQListener<T> listener;
    private final Executor executor;

    @SuppressWarnings("unchecked")
    public MessageStreamListener(Type type, String topic, Consumer consumer, RedisMQListener<T> listener,
                                 Executor executor) {
        this.type = type;
        this.topic = topic;
        this.consumer = consumer;
        this.listener = listener;
        this.executor = executor;
        this.delayed = this.topic + ":" + this.consumer.getGroup();

        // 初始化延时消息消费服务
        this.executor.execute(() -> {
            List<String> keys = Collections.singletonList(this.delayed);
            while (!Thread.interrupted()) {
                // 批量获取延时消息（100条），为避免重复获取将消息延时时间增加1分钟
                String timestamp = String.valueOf(System.currentTimeMillis());
                List<String> messages = RedisContextHolder.getStringTemplate().execute(
                        RANGE_AND_INCREMENT_SCRIPT, keys, "-1", timestamp, "60000", "100"
                );

                // 如果当前没有延时消息则暂停1秒后继续
                if (ObjectUtils.isEmpty(messages)) {
                    ThreadUtils.sleep(1000);
                    continue;
                }

                // 延时消息处理
                messages.forEach(this::processing);
            }
        });
    }

    /**
     * 消息消费处理
     *
     * @param message 消息内容（JSON格式）
     */
    private void processing(String message) {
        this.executor.execute(() -> {
            try {
                MapRecord<String, String, String> record = RedisContextHolder.deserialize(message);
                if (RedisContextHolder.isHeartbeat(record)) {
                    // 如果是心跳消息则删除相关数据
                    RedisContextHolder.delete(this.topic, record.getId());
                    RedisContextHolder.getStringTemplate().opsForZSet().remove(this.delayed, message);
                } else if (this.processing(record)) {
                    RedisContextHolder.getStringTemplate().opsForZSet().remove(this.delayed, message);
                }
            } catch (Throwable t) {
                log.error("Message consume failed: {}, {}", this.consumer, message, t);
            }
        });
    }

    /**
     * 消息消费处理
     *
     * @param record 消息记录实例
     * @return true/false
     */
    private boolean processing(MapRecord<String, String, String> record) {
        if (log.isDebugEnabled()) {
            log.debug("Consume message: {}, {}", this.consumer, record);
        }

        // 消费消息
        try {
            RedisContextHolder.consume(record, this.type, this.listener::consume);
        } catch (Throwable t) {
            log.error("Message consume failed: {}, {}", this.consumer, record, t);
            return false;
        }
        return true;
    }

    /**
     * 消息延时处理
     *
     * @param record    消息记录实例
     * @param timestamp 延时时间戳
     * @return true/false
     */
    private boolean delaying(MapRecord<String, String, String> record, long timestamp) {
        if (log.isDebugEnabled()) {
            log.debug("Delaying message: {}, {}", this.consumer, record);
        }
        try {
            String message = RedisContextHolder.serialize(record);
            RedisContextHolder.getStringTemplate().opsForZSet().add(this.delayed, message, timestamp);
        } catch (Throwable t) {
            log.error("Message delaying failed: {}, {}", this.consumer, record, t);
            return false;
        }
        return true;
    }

    @Override
    public void onMessage(MapRecord<String, String, String> record) {
        this.executor.execute(() -> {
            // 如果是心跳消息则延时3秒后删除该消息，以防止同一topic的心跳消息立即删除后其他消费组无法消费到心跳消息的问题
            if (RedisContextHolder.isHeartbeat(record)) {
                this.delaying(record, System.currentTimeMillis() + 3000);
                RedisContextHolder.ack(this.topic, this.consumer.getGroup(), record.getId());
                return;
            }

            boolean success;
            Long delaying = RedisContextHolder.getMessageDelaying(record);
            if (delaying != null && delaying - System.currentTimeMillis() >= 1000) {
                // 消息延时处理
                success = this.delaying(record, delaying);
            } else if (!(success = this.processing(record))) {
                // 消息处理异常延迟10秒后重试
                success = this.delaying(record, System.currentTimeMillis() + 10000);
            }
            if (success) {
                // 消息消费确认
                RedisContextHolder.ack(this.topic, this.consumer.getGroup(), record.getId());
            }
        });
    }
}
