package com.chelvc.framework.rocketmq.config;

import java.lang.reflect.Type;
import java.util.concurrent.atomic.AtomicInteger;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.consumer.MultipleRocketMQListenerContainer;
import com.chelvc.framework.rocketmq.consumer.RocketMQListener;
import com.chelvc.framework.rocketmq.consumer.RocketMQListenerContainer;
import com.chelvc.framework.rocketmq.consumer.SingleRocketMQListenerContainer;
import com.chelvc.framework.rocketmq.producer.DefaultRocketMQProducerFactory;
import com.chelvc.framework.rocketmq.producer.RocketMQProducerFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.SessionCredentialsProvider;
import org.apache.rocketmq.client.apis.StaticSessionCredentialsProvider;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

/**
 * RocketMQ配置类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RocketMQConfigurer implements ApplicationRunner {
    private final RocketMQProperties properties;
    private final AtomicInteger counter = new AtomicInteger(0);

    @Bean
    public ClientServiceProvider provider() {
        return ClientServiceProvider.loadService();
    }

    @Bean
    public ClientConfiguration configuration() {
        SessionCredentialsProvider sessionCredentialsProvider =
                new StaticSessionCredentialsProvider(this.properties.getAccessKey(), this.properties.getSecretKey());
        return ClientConfiguration.newBuilder().setEndpoints(this.properties.getEndpoints())
                .setCredentialProvider(sessionCredentialsProvider).build();
    }

    @Bean
    public RocketMQProducerFactory producerFactory() {
        return new DefaultRocketMQProducerFactory(this.provider(), this.configuration());
    }

    /**
     * 注册消息监听器容器
     *
     * @param listener 消息监听器实例
     * @param <T>      消息对象类型
     */
    @SuppressWarnings("unchecked")
    private <T> void registerListenerContainer(RocketMQListener<T> listener) {
        Class<?> clazz = AopProxyUtils.ultimateTargetClass(listener);
        RocketMQConsumer annotation = AssertUtils.nonnull(
                clazz.getAnnotation(RocketMQConsumer.class),
                () -> "@RocketMQConsumer annotation is missing: " + clazz.getName()
        );
        Type type = ObjectUtils.lookupSuperclassParameterized(clazz, RocketMQListener.class);
        RocketMQListenerContainer<T> container = ApplicationContextHolder.register(
                this.counter.incrementAndGet(), RocketMQListenerContainer.class, () -> {
                    boolean multiple = Math.max(annotation.batch(), 1) > 1;
                    return multiple ? new MultipleRocketMQListenerContainer<>() :
                            new SingleRocketMQListenerContainer<>();
                });
        try {
            container.initialize(type, this.provider(), this.configuration(), listener, annotation);
        } catch (Exception e) {
            log.error("RocketMQListenerContainer initialize failed", e);
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public void run(ApplicationArguments args) throws Exception {
        // 注册消息监听器
        ApplicationContextHolder.getBeans(RocketMQListener.class).forEach(
                (key, listener) -> this.registerListenerContainer(listener)
        );
    }
}
