package com.chelvc.framework.rocketmq.consumer;

import java.lang.reflect.Type;
import java.util.Collections;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.consumer.ConsumeResult;
import org.apache.rocketmq.client.apis.consumer.FilterExpression;
import org.apache.rocketmq.client.apis.consumer.FilterExpressionType;
import org.apache.rocketmq.client.apis.consumer.PushConsumer;
import org.apache.rocketmq.client.apis.consumer.PushConsumerBuilder;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.springframework.core.env.Environment;

/**
 * 消息单个消费监听器容器
 *
 * @param <T> 监听消息对象类型
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class SingleRocketMQListenerContainer<T> implements RocketMQListenerContainer<T> {
    private Type type;
    private String topic;
    private String group;
    private PushConsumer consumer;
    private RocketMQListener<T> listener;

    /**
     * 消费消息
     *
     * @param message 消息视图
     * @return 消费结果
     */
    @SuppressWarnings("unchecked")
    private ConsumeResult consume(MessageView message) {
        try {
            RocketMQContextHolder.deserialize(message, this.type, payload -> {
                Long timestamp = RocketMQContextHolder.getMessageDelaying(message);
                if (timestamp == null || timestamp <= System.currentTimeMillis()) {
                    // 正常消费
                    this.listener.consume((T) payload);
                } else {
                    // 延时消费
                    Pair<String, String> topic = Pair.of(message.getTopic(), message.getTag().orElse(null));
                    Message delayed = RocketMQContextHolder.serialize(topic, payload, null, timestamp);
                    RocketMQContextHolder.send(delayed);
                }
            });
            return ConsumeResult.SUCCESS;
        } catch (Throwable t) {
            String id = message.getMessageId().toString();
            log.error("Message consume failed: {}:{}, {}", this.topic, this.group, id, t);
            return ConsumeResult.FAILURE;
        }
    }

    /**
     * 关闭消息消费者
     *
     * @param consumer 消息消费者实例
     */
    private void shutdown(PushConsumer consumer) {
        if (consumer != null) {
            try {
                consumer.close();
            } catch (Throwable t) {
                log.warn("Consumer shutdown failed: {}:{}, {}", this.topic, this.group, t.getMessage());
            }
        }
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull ClientServiceProvider provider,
                           @NonNull ClientConfiguration configuration, @NonNull RocketMQListener<T> listener,
                           @NonNull RocketMQConsumer annotation) {
        // 参数配置有效性校验
        AssertUtils.nonempty(annotation.topic(), () -> "Consumer topic must not be empty");
        AssertUtils.nonempty(annotation.group(), () -> "Consumer group must not be empty");
        AssertUtils.nonempty(annotation.tag(), () -> "Consumer tag must not be empty");
        AssertUtils.check(annotation.concurrency() > 0, () -> "Consumer concurrency must be greater than 0");

        // 初始化消费者实例
        this.type = type;
        Environment environment = ApplicationContextHolder.getEnvironment();
        this.topic = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.topic()));
        this.group = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.group()));
        this.listener = listener;
        FilterExpression filter = new FilterExpression(annotation.tag(), FilterExpressionType.TAG);
        PushConsumerBuilder builder = provider.newPushConsumerBuilder().setClientConfiguration(configuration)
                .setConsumerGroup(this.group).setConsumptionThreadCount(annotation.concurrency())
                .setSubscriptionExpressions(Collections.singletonMap(this.topic, filter))
                .setMessageListener(this::consume);
        try {
            this.consumer = builder.build();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void destroy() throws Exception {
        this.shutdown(this.consumer);
    }
}
