package com.chelvc.framework.rocketmq.context;

import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Reference;
import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.rocketmq.config.RocketMQProperties;
import com.chelvc.framework.rocketmq.producer.RocketMQProducerFactory;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.message.MessageBuilder;
import org.apache.rocketmq.client.apis.message.MessageId;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.Transaction;
import org.apache.rocketmq.client.apis.producer.TransactionResolution;
import org.apache.rocketmq.client.java.exception.ProxyTimeoutException;
import org.apache.rocketmq.client.java.exception.TooManyRequestsException;

/**
 * RocketMQ上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class RocketMQContextHolder {
    /**
     * 消息类型标识
     */
    private static final String TYPE = "type";

    /**
     * 延时时间标识
     */
    private static final String DELAYING = "delaying";

    /**
     * 单次最大延迟时间（毫秒），阿里云RocketMQ延时消息最大支持24小时
     */
    private static final int MAX_DELAY_TIME = 24 * 60 * 60 * 1000;

    /**
     * 消息发送降级队列
     */
    private static final BlockingQueue<MessageSending> MESSAGE_SENDING_QUEUE = new LinkedBlockingQueue<>();

    /**
     * 配置属性
     */
    private static RocketMQProperties PROPERTIES;

    /**
     * 客户端服务提供者
     */
    private static ClientServiceProvider SERVICE_PROVIDER;

    /**
     * 消息生产者工厂
     */
    private static RocketMQProducerFactory PRODUCER_FACTORY;

    static {
        // 初始化消息发送降级处理线程
        ThreadUtils.run(() -> {
            while (!Thread.interrupted()) {
                // 尝试从队列中获取降级处理信息，如果为空则等待1分钟后继续
                MessageSending sending = MESSAGE_SENDING_QUEUE.peek();
                if (sending == null) {
                    ThreadUtils.sleep(60000);
                    continue;
                }

                // 降级处理，重试消息发送
                try {
                    sending.retry();
                } catch (ProxyTimeoutException | TooManyRequestsException e) {
                    // 如果因网络超时或触发TPS限制则等待1分钟后继续
                    log.warn("RocketMQ message send failed: {}", e.getMessage());
                    ThreadUtils.sleep(60000);
                    continue;
                } catch (Throwable t) {
                    // 如出现其他异常则抛弃该消息
                    log.error("RocketMQ message send failed", t);
                }

                // 降级处理完成后移除队列
                MESSAGE_SENDING_QUEUE.remove();
            }
        });
    }

    private RocketMQContextHolder() {
    }

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    public static RocketMQProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (RocketMQProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(RocketMQProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取服务提供者
     *
     * @return 服务提供者实例
     */
    public static ClientServiceProvider getServiceProvider() {
        if (SERVICE_PROVIDER == null) {
            synchronized (ClientServiceProvider.class) {
                if (SERVICE_PROVIDER == null) {
                    SERVICE_PROVIDER = ApplicationContextHolder.getBean(ClientServiceProvider.class);
                }
            }
        }
        return SERVICE_PROVIDER;
    }

    /**
     * 获取消息工厂
     *
     * @return 消息工厂对象实例
     */
    public static MessageBuilder getMessageBuilder() {
        return getServiceProvider().newMessageBuilder();
    }

    /**
     * 获取消息生产者工厂
     *
     * @return 生产者工厂实例
     */
    public static RocketMQProducerFactory getProducerFactory() {
        if (PRODUCER_FACTORY == null) {
            synchronized (RocketMQProducerFactory.class) {
                if (PRODUCER_FACTORY == null) {
                    PRODUCER_FACTORY = ApplicationContextHolder.getBean(RocketMQProducerFactory.class);
                }
            }
        }
        return PRODUCER_FACTORY;
    }

    /**
     * 环境隔离
     *
     * @param original 原始标记
     * @return 环境隔离标记
     */
    public static String isolate(@NonNull String original) {
        String namespace = getProperties().getNamespace();
        return StringUtils.isEmpty(namespace) ? original : (namespace + original);
    }

    /**
     * 消息主题拆分
     *
     * @param original 原始消息主题
     * @return 消息主题/标签部分
     */
    public static Pair<String, String> topic(@NonNull String original) {
        int separator = original.indexOf(':');
        String topic = separator > 0 ? StringUtils.trim(original.substring(0, separator)) : original;
        String tag = separator > 0 ? StringUtils.trim(original.substring(separator + 1)) : StringUtils.EMPTY;
        return Pair.of(isolate(topic), tag);
    }

    /**
     * 消息序列化
     *
     * @param topic    消息主体
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @return 消息对象实例
     */
    public static Message serialize(@NonNull Pair<String, String> topic, @NonNull Object payload, String ordering,
                                    Long delaying) {
        byte[] body = JacksonUtils.serialize(payload).getBytes(StandardCharsets.UTF_8);
        String session = JacksonUtils.serialize(SessionContextHolder.getSession(false));
        MessageBuilder builder = getMessageBuilder().setTopic(topic.getLeft()).setBody(body)
                .addProperty(TYPE, payload.getClass().getName());
        if (Objects.nonNull(session)) {
            builder.addProperty(SessionContextHolder.NAME, session);
        }
        if (StringUtils.notEmpty(topic.getRight())) {
            builder.setTag(topic.getRight());
        }
        if (StringUtils.notEmpty(ordering)) {
            builder.setMessageGroup(ordering);
        }
        if (Objects.nonNull(delaying)) {
            long now = System.currentTimeMillis();
            long timestamp = now + Math.min(Math.max(delaying - now, 0), MAX_DELAY_TIME);
            builder.setDeliveryTimestamp(timestamp);
            builder.addProperty(DELAYING, String.valueOf(delaying));
        }
        return builder.build();
    }

    /**
     * 获取消息体类型
     *
     * @param message 消息视图
     * @return 消息体对象类型
     */
    public static Class<?> type(@NonNull MessageView message) {
        String name = AssertUtils.nonempty(message.getProperties().get(TYPE), () -> "Message type unspecified");
        try {
            return Class.forName(name);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 消息反序列化
     *
     * @param message 消息视图
     * @param type    消息类型
     * @return 消息体实例
     */
    public static Object deserialize(@NonNull MessageView message, @NonNull Type type) {
        return JacksonUtils.deserialize(StandardCharsets.UTF_8.decode(message.getBody()).toString(), type);
    }

    /**
     * 消息反序列化
     *
     * @param message  消息视图
     * @param consumer 消息消费者
     */
    public static void deserialize(@NonNull MessageView message, @NonNull Consumer<Object> consumer) {
        deserialize(message, type(message), consumer);
    }

    /**
     * 消息反序列化
     *
     * @param message  消息视图
     * @param type     消息类型
     * @param consumer 消息消费者
     */
    public static void deserialize(@NonNull MessageView message, @NonNull Type type,
                                   @NonNull Consumer<Object> consumer) {
        Object payload = deserialize(message, type);
        Session session = JacksonUtils.deserialize(
                message.getProperties().get(SessionContextHolder.NAME), Session.class
        );
        SessionContextHolder.setSession(session);
        try {
            consumer.accept(payload);
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }

    /**
     * 消息反序列化
     *
     * @param messages 消息视图集合
     * @param type     消息类型
     * @param function 消息处理函数
     * @param <T>      消息类型
     * @return 消息列表
     */
    public static <T> List<T> deserialize(@NonNull Collection<MessageView> messages, @NonNull Type type,
                                          @NonNull BiFunction<Object, Session, T> function) {
        if (ObjectUtils.isEmpty(messages)) {
            return Collections.emptyList();
        }
        return messages.stream().map(message -> {
            Object payload = deserialize(message, type);
            Session session = JacksonUtils.deserialize(
                    message.getProperties().get(SessionContextHolder.NAME), Session.class
            );
            return function.apply(payload, session);
        }).filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 获取消息延时时间
     *
     * @param message 消息视图
     * @return 延时时间戳
     */
    public static Long getMessageDelaying(MessageView message) {
        return StringUtils.ifEmpty(message.getProperties().get(DELAYING), Long::parseLong);
    }

    /**
     * 发送消息
     *
     * @param message 消息对象实例
     */
    public static void send(@NonNull Message message) {
        Producer producer = getProducerFactory().getProducer(message.getTopic());
        try {
            producer.send(message);
        } catch (ProxyTimeoutException | TooManyRequestsException e) {
            log.warn("RocketMQ message send failed: {}", e.getMessage());

            // 降级处理
            try {
                MESSAGE_SENDING_QUEUE.put(new MessageSending(producer, message));
            } catch (InterruptedException ignore) {
            }
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void send(@NonNull String topic, @NonNull Object payload) {
        send(topic, payload, null, (Long) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering) {
        send(topic, payload, ordering, (Long) null);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Duration delaying) {
        send(topic, payload, null, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Date delaying) {
        send(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, Long delaying) {
        send(topic, payload, null, delaying);
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering, Duration delaying) {
        send(topic, payload, ordering, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering, Date delaying) {
        send(topic, payload, ordering, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void send(@NonNull String topic, @NonNull Object payload, String ordering, Long delaying) {
        send(serialize(topic(topic), payload, ordering, delaying));
    }

    /**
     * 发送异步消息
     *
     * @param message 消息对象实例
     */
    public static void sendAsync(@NonNull Message message) {
        Producer producer = getProducerFactory().getProducer(message.getTopic());
        producer.sendAsync(message).exceptionally(e -> {
            if (e instanceof ProxyTimeoutException || e instanceof TooManyRequestsException) {
                log.warn("RocketMQ message async send failed: {}", e.getMessage());

                // 降级处理
                try {
                    MESSAGE_SENDING_QUEUE.put(new MessageSending(producer, message));
                } catch (InterruptedException ignore) {
                }
            } else {
                log.error("RocketMQ message async send failed: {}", message.getTopic(), e);
            }
            return null;
        });
    }

    /**
     * 发送异步消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload) {
        sendAsync(topic, payload, null, (Long) null);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering) {
        sendAsync(topic, payload, ordering, (Long) null);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, Duration delaying) {
        sendAsync(topic, payload, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, Date delaying) {
        sendAsync(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, Long delaying) {
        sendAsync(topic, payload, null, delaying);
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering, Duration delaying) {
        sendAsync(topic, payload, ordering, delaying == null ? null : System.currentTimeMillis() + delaying.toMillis());
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering, Date delaying) {
        sendAsync(topic, payload, ordering, ObjectUtils.ifNull(delaying, Date::getTime));
    }

    /**
     * 发送异步消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     */
    public static void sendAsync(@NonNull String topic, @NonNull Object payload, String ordering, Long delaying) {
        sendAsync(serialize(topic(topic), payload, ordering, delaying));
    }

    /**
     * 发送事务消息
     *
     * @param message  消息对象实例
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    @SuppressWarnings("unchecked")
    public static <T> void sendTransaction(@NonNull Message message, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Producer producer = getProducerFactory().getProducer(message.getTopic(), view -> {
            Reference<TransactionResolution> resolution = new Reference<>();
            deserialize(view, target -> resolution.set(checker.apply((T) target)));
            return AssertUtils.nonnull(resolution.get(), () -> "Transaction checker result must not be null");
        });
        try {
            Transaction transaction = producer.beginTransaction();
            MessageId id = producer.send(message, transaction).getMessageId();
            if (Boolean.TRUE.equals(executor.apply(String.valueOf(id)))) {
                transaction.commit();
            } else {
                transaction.rollback();
            }
        } catch (ProxyTimeoutException | TooManyRequestsException e) {
            log.warn("RocketMQ message send failed: {}", e.getMessage());

            // 降级处理
            try {
                MESSAGE_SENDING_QUEUE.put(new TransactionalMessageSending(producer, message, executor));
            } catch (InterruptedException ignore) {
            }
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, null, (Long) null, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, ordering, (Long) null, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, Duration delaying,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Long timestamp = delaying == null ? null : System.currentTimeMillis() + delaying.toMillis();
        sendTransaction(topic, payload, timestamp, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, Date delaying,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, ObjectUtils.ifNull(delaying, Date::getTime), executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, Long delaying,
                                           @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, null, delaying, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           Duration delaying, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        Long timestamp = delaying == null ? null : System.currentTimeMillis() + delaying.toMillis();
        sendTransaction(topic, payload, ordering, timestamp, executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           Date delaying, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(topic, payload, ordering, ObjectUtils.ifNull(delaying, Date::getTime), executor, checker);
    }

    /**
     * 发送事务消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param delaying 延时消息时间
     * @param executor 本地事务执行方法
     * @param checker  本地事务检测回调方法
     * @param <T>      消息类型
     */
    public static <T> void sendTransaction(@NonNull String topic, @NonNull T payload, String ordering,
                                           Long delaying, @NonNull Function<String, Boolean> executor,
                                           @NonNull Function<T, TransactionResolution> checker) {
        sendTransaction(serialize(topic(topic), payload, ordering, delaying), executor, checker);
    }
}
