package com.chelvc.framework.security.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.util.Objects;

import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.model.Result;
import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.util.Base64Utils;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdviceAdapter;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

/**
 * 接口参数加解密拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@ControllerAdvice
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class ControllerCryptoInterceptor extends RequestBodyAdviceAdapter implements ResponseBodyAdvice<Object> {
    private final SecurityProperties properties;

    /**
     * 获取AES加解密向量
     *
     * @return 向量字符串
     */
    private String iv() {
        Session session = SessionContextHolder.getSession(false);
        String version = String.valueOf(ObjectUtils.ifNull(session, Session::getVersion));
        String timestamp = String.valueOf(ObjectUtils.ifNull(session, Session::getTimestamp));
        byte[] bytes = Base64Utils.encode((timestamp + version).getBytes());
        return new String(bytes, 0, Math.min(bytes.length, 16));
    }

    /**
     * 获取目标方法@Crypto注解
     *
     * @param method 目标方法
     * @return @Crypto注解实例
     */
    private Crypto getCryptoAnnotation(MethodParameter method) {
        Crypto annotation = method.getMethodAnnotation(Crypto.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(Crypto.class);
        }
        return annotation;
    }

    @Override
    public boolean supports(MethodParameter method, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.output();
    }

    @Override
    public boolean supports(MethodParameter method, Type type, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.input();
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage message, MethodParameter method, Type type,
                                           Class<? extends HttpMessageConverter<?>> clazz) throws IOException {
        byte[] ciphertext = Base64.decodeBase64(HttpUtils.getBody(message.getBody()));
        String secret = Objects.requireNonNull(this.properties.getSecret()), iv = this.iv();
        byte[] body;
        try {
            body = AESUtils.decode(ciphertext, secret, iv);
        } catch (Exception e) {
            throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null, "Invalid ciphertext");
        }
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return message.getHeaders();
            }

            @Override
            public InputStream getBody() throws IOException {
                return new ByteArrayInputStream(body);
            }
        };
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter method, MediaType media,
                                  Class<? extends HttpMessageConverter<?>> clazz,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        if (body == null) {
            return null;
        }
        String secret = Objects.requireNonNull(this.properties.getSecret()), iv = this.iv();
        if (body instanceof Result) {
            Result<?> result = (Result<?>) body;
            if (result.getData() == null) {
                return result;
            }
            String ciphertext = AESUtils.encode(JacksonContextHolder.serialize(result.getData()), secret, iv);
            return Result.of(result.getCode(), ciphertext, result.getMessage());
        }
        return AESUtils.encode(JacksonContextHolder.serialize(body), secret, iv);
    }
}
