package com.chelvc.framework.security.interceptor;

import java.util.Objects;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.model.Session;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Security;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.Ordered;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 请求安全拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SecurityValidateInterceptor implements HandlerInterceptor, WebMvcConfigurer {
    private final SecurityProperties properties;

    /**
     * 获取接口安全注解实例
     *
     * @param handler 接口处理器实例
     * @return 安全注解实例
     */
    private Security getSecurityAnnotation(Object handler) {
        if (!(handler instanceof HandlerMethod)) {
            return null;
        }
        HandlerMethod method = (HandlerMethod) handler;
        Security annotation = method.getMethodAnnotation(Security.class);
        if (annotation == null) {
            return method.getMethod().getDeclaringClass().getAnnotation(Security.class);
        }
        return annotation;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        Security annotation = this.getSecurityAnnotation(handler);
        if (annotation == null || (!annotation.sign() && !annotation.header())) {
            return true;
        }

        // 请求头校验
        Session session = SessionContextHolder.getSession();
        if (annotation.header()) {
            // 非空请求头校验
            if (session.getPlatform() == null || session.getTerminal() == null
                    || StringUtils.isEmpty(session.getVersion()) || session.getTimestamp() == null) {
                throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null, "Invalid header");
            }

            // 请求时间戳校验
            long now = System.currentTimeMillis(), duration = this.properties.getDuration();
            if (duration > 0 && (session.getTimestamp() > now || now - session.getTimestamp() > duration)) {
                throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null, "请校准系统时间");
            }
        }

        // 数字签名校验
        if (annotation.sign()) {
            String payload = HttpUtils.serialize(request, true);
            String secret = Objects.requireNonNull(this.properties.getSecret());
            String plaintext = secret + session.getPlatform() + session.getTerminal() + session.getVersion() +
                    session.getTimestamp() + payload;
            String ciphertext = CodecUtils.md5(plaintext);
            String signature = request.getHeader(SessionContextHolder.HEADER_SIGNATURE);
            LoggingContextHolder.debug(log, plaintext, ciphertext, payload);
            if (!Objects.equals(ciphertext, signature)) {
                throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null, "Invalid signature");
            }
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE);
    }
}
