package com.chelvc.framework.sms.support;

import java.time.Duration;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.sms.Captcha;
import com.chelvc.framework.sms.CaptchaSmsHandler;
import com.chelvc.framework.sms.SmsSession;
import com.chelvc.framework.sms.TemplateSmsHandler;
import com.chelvc.framework.sms.config.SmsProperties;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 验证码短信处理器默认实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DefaultCaptchaSmsHandler implements CaptchaSmsHandler {
    private final TemplateSmsHandler handler;
    private final SmsProperties.Captcha properties;

    public DefaultCaptchaSmsHandler(TemplateSmsHandler handler, @NonNull SmsProperties.Captcha properties) {
        this.handler = handler;
        this.properties = properties;
    }

    /**
     * 构建令牌Redis标识
     *
     * @param mobile 手机号
     * @return Redis标识
     */
    private String key(String mobile) {
        return "sms:captcha:" + mobile;
    }

    /**
     * 生成随机验证码
     *
     * @param length 验证码长度
     * @return 验证码
     */
    private String random(int length) {
        ThreadLocalRandom random = ThreadLocalRandom.current();
        if (length == 1) {
            return String.valueOf(random.nextInt(0, 10));
        }
        int origin = (int) Math.pow(10, length - 1);
        return String.valueOf(random.nextInt(origin, origin * 10));
    }

    /**
     * 判断手机号是否是验证码白名单
     *
     * @param mobile 手机号
     * @return true/false
     */
    private boolean isCaptchaWhitelisted(String mobile) {
        Set<?> mobiles = ApplicationContextHolder.getSafeProperty("sms.captcha.whitelist", Set.class);
        return ObjectUtils.notEmpty(mobiles) && mobiles.contains(mobile);
    }

    @Override
    public SmsSession send(@NonNull String mobile) {
        return this.send(mobile, this.properties.getLength());
    }

    @Override
    public SmsSession send(@NonNull String mobile, int length) {
        AssertUtils.check(length > 0, () -> "Captcha length must be greater than 0");

        // 短信验证码发送频率限制
        String lock = "sms:captcha:interval:" + mobile;
        String secret = RedisContextHolder.tryLock(lock, Duration.ofSeconds(this.properties.getInterval()));
        AssertUtils.available(StringUtils.notEmpty(secret), () -> "短信发送过于频繁，请稍后再试");

        // 如果目标手机号属于白名单手机号则将手机号后{{length}}位作为验证码，否则获取真实的验证码
        try {
            String key = this.key(mobile);
            String id = StringUtils.uuid();
            boolean whitelisted = this.isCaptchaWhitelisted(mobile);
            String code = whitelisted ? mobile.substring(mobile.length() - length) : this.random(length);
            Captcha captcha = Captcha.builder().id(id).code(code).mobile(mobile).build();
            if (!whitelisted) {
                Objects.requireNonNull(this.handler).send(this.properties.getTemplate(), captcha);
            }
            RedisContextHolder.getDefaultTemplate().opsForValue()
                    .set(key, captcha, this.properties.getExpiration(), TimeUnit.SECONDS);
            return SmsSession.builder().id(id).expiration(this.properties.getExpiration()).build();
        } catch (RuntimeException e) {
            // 保证验证码发送失败场景下及时释放锁
            RedisContextHolder.unlock(lock, secret);
            throw e;
        }
    }

    @Override
    public Captcha getCaptcha(@NonNull String mobile) {
        return (Captcha) RedisContextHolder.getDefaultTemplate().opsForValue().get(this.key(mobile));
    }

    @Override
    public void using(@NonNull String mobile, @NonNull Function<Captcha, Boolean> function) {
        Captcha captcha = this.getCaptcha(mobile);
        if (Boolean.TRUE.equals(function.apply(captcha))) {
            RedisContextHolder.getDefaultTemplate().delete(this.key(mobile));
        }
    }

    @Override
    public boolean check(@NonNull String id, @NonNull String mobile, @NonNull String code) {
        Captcha captcha = this.getCaptcha(mobile);
        return captcha != null && captcha.check(id, mobile, code);
    }

    @Override
    public boolean validate(@NonNull String id, @NonNull String mobile, @NonNull String code) {
        boolean available = this.check(id, mobile, code);
        if (available) {
            RedisContextHolder.getDefaultTemplate().delete(this.key(mobile));
        }
        return available;
    }
}
