package com.chelvc.framework.sms.support;

import java.util.List;
import java.util.function.Function;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.sms.NormalSmsHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 普通短信发送处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingNormalSmsHandler implements NormalSmsHandler {
    private final List<NormalSmsHandler> handlers;

    public DelegatingNormalSmsHandler(@NonNull List<NormalSmsHandler> handlers) {
        this.handlers = AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
    }

    /**
     * 遍历所有短信处理器并执行短信发送处理，直到发送成功
     *
     * @param callback 短信处理回调函数
     * @return true/false
     */
    private boolean execute(Function<NormalSmsHandler, Boolean> callback) {
        Exception unsupported = null;
        for (NormalSmsHandler handler : this.handlers) {
            try {
                if (Boolean.TRUE.equals(callback.apply(handler))) {
                    return true;
                }
            } catch (Exception e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = e;
                } else {
                    log.warn("Normal SMS send failed: {}", e.getMessage());
                }
            }
        }
        if (unsupported != null) {
            log.warn("Normal SMS send failed: {}", unsupported.getMessage());
        }
        return false;
    }

    @Override
    public boolean send(@NonNull String content, @NonNull String... mobiles) {
        return mobiles.length > 0 && this.execute(handler -> handler.send(content, mobiles));
    }
}
