package com.chelvc.framework.upload.support;

import java.io.File;
import java.io.InputStream;
import java.util.List;
import java.util.function.Function;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.upload.UploadHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 文件上传处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingUploadHandler implements UploadHandler {
    private final List<UploadHandler> handlers;

    public DelegatingUploadHandler(@NonNull List<UploadHandler> handlers) {
        this.handlers = AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
    }

    /**
     * 遍历所有短信处理器并执行文件上传处理，直到上传成功
     *
     * @param callback 文件上传处理回调函数
     */
    private String execute(Function<UploadHandler, String> callback) {
        RuntimeException unsupported = null;
        for (int i = 0, last = this.handlers.size() - 1; i <= last; i++) {
            try {
                return callback.apply(this.handlers.get(i));
            } catch (RuntimeException e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = e;
                } else if (i < last) {
                    log.warn("File upload failed: {}", e.getMessage());
                } else {
                    throw e;
                }
            }
        }
        if (unsupported != null) {
            throw unsupported;
        }
        return null;
    }

    @Override
    public String upload(@NonNull File file) {
        return this.execute(handler -> handler.upload(file));
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix) {
        return this.execute(handler -> handler.upload(stream, suffix));
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix, long length) {
        return this.execute(handler -> handler.upload(stream, suffix, length));
    }
}
