package com.chelvc.framework.upload.support;

import java.io.File;
import java.io.InputStream;
import java.util.Objects;

import com.chelvc.framework.base.config.HttpClientProperties;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.upload.UploadHandler;
import com.chelvc.framework.upload.config.UploadProperties;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.http.HttpProtocol;
import com.qcloud.cos.model.ObjectMetadata;
import com.qcloud.cos.region.Region;
import lombok.NonNull;

/**
 * 基于腾讯COS文件上传实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class TencentUploadHandler implements UploadHandler {
    private final String path;
    private final String region;
    private final COSClient client;

    public TencentUploadHandler(@NonNull UploadProperties.Client properties, @NonNull HttpClientProperties httpClient) {
        this.path = Objects.requireNonNull(properties.getPath());
        this.region = Objects.requireNonNull(properties.getRegion());

        // 构建客户端配置
        ClientConfig config = new ClientConfig();
        config.setRegion(new Region(this.region));
        config.setHttpProtocol(HttpProtocol.https);
        config.setMaxConnectionsCount(httpClient.getMaxTotal());
        config.setConnectionTimeout(httpClient.getConnectTimeout());
        config.setSocketTimeout(httpClient.getSocketTimeout());
        config.setConnectionRequestTimeout(httpClient.getConnectionRequestTimeout());

        // 初始化客户端实例
        String id = Objects.requireNonNull(properties.getId());
        String secret = Objects.requireNonNull(properties.getSecret());
        this.client = new COSClient(new BasicCOSCredentials(id, secret), config);
    }

    /**
     * 将文件名称转换成URL地址
     *
     * @param filename 文件名称
     * @return URL地址
     */
    private String name2url(String filename) {
        return "https://" + this.path + ".cos." + this.region + ".myqcloud.com/" + filename;
    }

    @Override
    public String upload(@NonNull File file) {
        String suffix = FileUtils.getSuffix(file);
        String filename = StringUtils.uuid() + StringUtils.ifEmpty(suffix, name -> "." + name, StringUtils.EMPTY);
        this.client.putObject(this.path, filename, file);
        return this.name2url(filename);
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix) {
        return this.upload(stream, suffix, FileUtils.getLength(stream));
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix, long length) {
        ObjectMetadata metadata = new ObjectMetadata();
        metadata.setContentLength(length);
        String filename = StringUtils.uuid() + StringUtils.ifEmpty(suffix, name -> "." + name, StringUtils.EMPTY);
        this.client.putObject(this.path, filename, stream, metadata);
        return this.name2url(filename);
    }

    @Override
    public void destroy() {
        this.client.shutdown();
    }
}
