package com.chelvc.framework.wechat;

import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.chelvc.framework.wechat.context.WechatContextHolder;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.util.CollectionUtils;
import org.springframework.web.client.RestTemplate;

/**
 * 微信公众号操作默认实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class DefaultWechatPublicHandler implements WechatPublicHandler {
    /**
     * 发送微信服务号模板消息接口地址
     */
    private static final String SEND_TEMPLATE_MESSAGE_URL =
            "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=";

    /**
     * 批量获取关注用户openid地址
     */
    private static final String OPENID_URL =
            "https://api.weixin.qq.com/cgi-bin/user/get?access_token=%s&next_openid=%s";

    /**
     * 获取公众号粉丝unionId信息地址
     */
    private static final String USER_URL = "https://api.weixin.qq.com/cgi-bin/user/info/batchget?access_token=";

    /**
     * 获取微信网页令牌接口地址
     */
    private static final String CODE2TOKEN_URL = "https://api.weixin.qq.com/sns/oauth2/access_token" +
            "?appid=%s&secret=%s&code=%s&grant_type=authorization_code";

    private final RestTemplate restTemplate;
    private final WechatHandler wechatHandler;
    private final WechatProperties.Public properties;

    /**
     * 获取密码处理实例
     *
     * @param mode 处理模式
     * @return 密码处理实例
     */
    private Cipher getCipher(int mode) {
        byte[] secret = Base64.decodeBase64(this.properties.getKey() + StringUtils.EQUAL);
        try {
            Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
            SecretKeySpec key = new SecretKeySpec(secret, AESUtils.ALGORITHM);
            IvParameterSpec iv = new IvParameterSpec(Arrays.copyOfRange(secret, 0, 16));
            cipher.init(mode, key, iv);
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public String getId() {
        return StringUtils.ifEmpty(this.properties.getId(), this.properties.getAppid());
    }

    @Override
    public String getAppid() {
        return this.properties.getAppid();
    }

    @Override
    public String getAccessToken() {
        return this.wechatHandler.getAccessToken(this.properties.getAppid(), this.properties.getSecret());
    }

    @Override
    public String refreshAccessToken() {
        return this.wechatHandler.refreshAccessToken(this.properties.getAppid(), this.properties.getSecret());
    }

    @Override
    public WechatWebToken code2token(@NonNull String code) {
        String url = String.format(CODE2TOKEN_URL, this.properties.getAppid(), this.properties.getSecret(), code);
        return WechatContextHolder.execute(() -> this.restTemplate.getForObject(url, WechatWebToken.class));
    }

    @Override
    public String decrypt(String ciphertext) {
        if (StringUtils.isEmpty(ciphertext)) {
            return ciphertext;
        }

        // 解密数据
        byte[] original;
        Cipher cipher = this.getCipher(Cipher.DECRYPT_MODE);
        try {
            original = cipher.doFinal(Base64.decodeBase64(ciphertext));
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }

        // 删除解密后明文的补位字符
        int pad = original[original.length - 1];
        if (pad < 1 || pad > 32) {
            pad = 0;
        }
        byte[] bytes = Arrays.copyOfRange(original, 0, original.length - pad);

        // 分离16位随机字符串,网络字节序和AppId
        int length = 0;
        for (int i = 16; i < 20; i++) {
            length <<= 8;
            length |= bytes[i] & 0xff;
        }

        String content = new String(Arrays.copyOfRange(original, 20, 20 + length), StandardCharsets.UTF_8);
        String appid = new String(Arrays.copyOfRange(original, 20 + length, bytes.length));
        AssertUtils.check(Objects.equals(appid, this.properties.getAppid()), () -> "Invalid appid: " + appid);
        return content;
    }

    @Override
    public WechatUser getUser(@NonNull String openid) {
        return this.wechatHandler.getUser(this.getAccessToken(), openid, this::refreshAccessToken);
    }

    @Override
    public void iterateOpenids(@NonNull Consumer<List<String>> consumer) {
        this.iterateOpenids((openids, total) -> consumer.accept(openids));
    }

    @Override
    public void iterateOpenids(@NonNull BiConsumer<List<String>, Integer> consumer) {
        int total = 0;
        String next = null;
        do {
            final String _next = ObjectUtils.ifNull(next, StringUtils.EMPTY);
            String url = String.format(OPENID_URL, this.getAccessToken(), _next);
            WechatPublicOpenids response = WechatContextHolder.execute(
                    () -> this.restTemplate.getForObject(url, WechatPublicOpenids.class),
                    (status, message) -> {
                        // 如果访问令牌过期或无效则更新访问令牌后重试
                        if (status == 40001 || status == 42001) {
                            return WechatContextHolder.execute(() -> this.restTemplate.getForObject(
                                    String.format(OPENID_URL, this.refreshAccessToken(), _next),
                                    WechatPublicOpenids.class
                            ));
                        }
                        throw new RuntimeException(status + ", " + message);
                    });
            List<String> openids = ObjectUtils.ifNull(response.getData(), WechatPublicOpenids.Openids::getOpenid);
            if (CollectionUtils.isEmpty(openids)) {
                break;
            }
            consumer.accept(openids, total += openids.size());
            next = response.getNextOpenid();
        } while (StringUtils.notEmpty(next));
    }

    @Override
    public List<WechatPublicSubscriber> getSubscribers(@NonNull Collection<String> openids) {
        if (CollectionUtils.isEmpty(openids)) {
            return Collections.emptyList();
        }
        String url = USER_URL + this.getAccessToken();
        Map<?, ?> body = ImmutableMap.of(
                "user_list",
                openids.stream().map(openid -> ImmutableMap.of("openid", openid)).collect(Collectors.toList())
        );
        WechatPublicSubscribers response = WechatContextHolder.execute(
                () -> this.restTemplate.postForObject(url, body, WechatPublicSubscribers.class),
                (status, message) -> {
                    // 如果访问令牌过期或无效则更新访问令牌后重试
                    if (status == 40001 || status == 42001) {
                        return WechatContextHolder.execute(() -> this.restTemplate.postForObject(
                                USER_URL + this.refreshAccessToken(), body, WechatPublicSubscribers.class
                        ));
                    }
                    throw new RuntimeException(status + ", " + message);
                });
        return ObjectUtils.ifNull(response.getSubscribers(), Collections::emptyList);
    }

    @Override
    public String sign(long timestamp, @NonNull String... elements) {
        MessageDigest digest = DigestUtils.getDigest("SHA-1");
        String original = Stream.concat(
                Stream.of(this.properties.getToken(), String.valueOf(timestamp)), Stream.of(elements)
        ).sorted().collect(Collectors.joining());
        return Hex.encodeHexString(digest.digest(original.getBytes()));
    }

    @Override
    public long push(@NonNull String openid, @NonNull String template, @NonNull Map<?, ?> parameters) {
        return this.push(openid, template, StringUtils.EMPTY, parameters);
    }

    @Override
    public long push(@NonNull String openid, @NonNull String template, @NonNull String redirect,
                     @NonNull Map<?, ?> parameters) {
        String url = SEND_TEMPLATE_MESSAGE_URL + this.getAccessToken();
        String appid = (redirect = redirect.trim()).isEmpty() ? StringUtils.EMPTY : this.wechatHandler.getAppid();
        Map<String, Object> body = ImmutableMap.of(
                "touser", openid,
                "template_id", template,
                "url", StringUtils.EMPTY,
                "miniprogram", ImmutableMap.of("appid", appid, "pagepath", redirect),
                "data", parameters
        );
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        WechatPublicMessage response;
        try {
            response = this.restTemplate.exchange(url, HttpMethod.POST, entity, WechatPublicMessage.class).getBody();
            if (response != null && (response.getErrcode() == 40001 || response.getErrcode() == 42001)) {
                // 如果是令牌过期则刷新令牌后重试
                url = SEND_TEMPLATE_MESSAGE_URL + this.refreshAccessToken();
                response = this.restTemplate.exchange(
                        url, HttpMethod.POST, entity, WechatPublicMessage.class
                ).getBody();
            }
        } catch (Exception e) {
            LoggingContextHolder.warn(log, template, e.getMessage());
            return 0;
        }
        if (response != null && response.getErrcode() == 0) {
            return response.getId();
        }
        LoggingContextHolder.warn(log, template, response);
        return 0;
    }
}
