package com.chelvc.framework.wechat.context;

import java.util.Map;
import java.util.function.BiFunction;
import java.util.function.Supplier;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.wechat.WechatPublicHandler;
import com.chelvc.framework.wechat.WechatResponse;
import com.github.wxpay.sdk.WXPayUtil;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;

/**
 * 微信上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class WechatContextHolder {
    /**
     * 微信公众号处理器映射表
     */
    private static Map<String, WechatPublicHandler> WECHAT_PUBLIC_HANDLER_MAPPING;

    private WechatContextHolder() {
    }

    /**
     * 将键/值对转换成xml
     *
     * @param map 键/值对
     * @return xml
     */
    public static String map2xml(@NonNull Map<String, String> map) {
        try {
            return WXPayUtil.mapToXml(map);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将xml转换成键/值对
     *
     * @param xml XML
     * @return 键/值对
     */
    public static Map<String, String> xml2map(@NonNull String xml) {
        try {
            return WXPayUtil.xmlToMap(xml);
        } catch (Exception e) {
            throw new RuntimeException();
        }
    }

    /**
     * 参数签名
     *
     * @param key        签名密钥
     * @param parameters 签名参数
     * @return 签名信息
     */
    public static String sign(@NonNull String key, @NonNull Map<String, String> parameters) {
        try {
            return WXPayUtil.generateSignature(parameters, key);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 解码手机号
     *
     * @param ciphertext 会话信息密文
     * @param secret     密钥
     * @param iv         偏移量
     * @return 手机号明文
     */
    public static String mobile(String ciphertext, @NonNull String secret, @NonNull String iv) {
        String plaintext = AESUtils.decode(ciphertext, Base64.decodeBase64(secret), Base64.decodeBase64(iv));
        return (String) ObjectUtils.ifNull(
                JacksonContextHolder.deserialize(plaintext, Map.class), map -> map.get("phoneNumber")
        );
    }

    /**
     * 执行请求操作
     *
     * @param supplier 接口请求回调犯法
     * @param <T>      微信响应数据类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T execute(@NonNull Supplier<T> supplier) {
        return execute(supplier, (status, message) -> {
            throw new RuntimeException(status + ", " + message);
        });
    }

    /**
     * 执行请求操作
     *
     * @param supplier 接口请求回调犯法
     * @param failure  请求失败处理回调接口
     * @param <T>      微信响应数据类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T execute(@NonNull Supplier<T> supplier,
                                                       @NonNull BiFunction<Integer, String, T> failure) {
        T response = supplier.get();
        AssertUtils.nonnull(response, () -> "Invalid wechat server response");
        int status = ObjectUtils.ifNull(response.getErrcode(), 0);
        return status == 0 ? response : failure.apply(status, response.getErrmsg());
    }

    /**
     * 获取微信公众号处理器
     *
     * @param id 微信公众号配置ID
     * @return 微信公众号处理器
     */
    public static WechatPublicHandler getWechatPublicHandler(@NonNull String id) {
        if (WECHAT_PUBLIC_HANDLER_MAPPING == null) {
            synchronized (WechatContextHolder.class) {
                if (WECHAT_PUBLIC_HANDLER_MAPPING == null) {
                    WECHAT_PUBLIC_HANDLER_MAPPING = ApplicationContextHolder.getBeans(WechatPublicHandler.class);
                }
            }
        }
        WechatPublicHandler handler = WECHAT_PUBLIC_HANDLER_MAPPING.get(id);
        return AssertUtils.nonnull(handler, () -> "Wechat public handler not found: " + id);
    }
}
